@@ -3,13 +3,14 @@
 
 #include <algorithm>
 #include <cstdint>
+#include <functional>
 #include <iostream>
 #include <limits>
+#include <unordered_set>
 #include <utility>
 #include <vector>
 
 #include <gbwt/utils.h>
-#include <sdsl/int_vector.hpp>
 
 #include <gbwtgraph/io.h>
 #include <gbwtgraph/utils.h>
@@ -40,8 +41,7 @@ struct MinimizerHeader
   constexpr static size_t        FLAG_KEY_OFFSET = 0;
 
   // Flags for old compatible versions.
-  constexpr static std::uint32_t FIRST_VERSION   = 4;
-  constexpr static std::uint64_t FIRST_FLAG_MASK = 0x0000;
+  // (X_VERSION, X_FLAG_MASK)
 
   MinimizerHeader();
   MinimizerHeader(size_t kmer_length, size_t window_length, size_t initial_capacity, double max_load_factor, size_t key_bits);
@@ -67,9 +67,58 @@ struct MinimizerHeader
 
 //------------------------------------------------------------------------------
 
+/*
+  Types used for internal representation of positions in the minimizer index.
+  These are exposed through the interface.
+*/
+
+typedef std::uint64_t code_type;
+typedef std::uint64_t payload_type;
+
+struct hit_type
+{
+  code_type    pos;
+  payload_type payload;
+
+  // Payload is irrelevant for comparisons.
+  bool operator==(const hit_type& another) const { return (this->pos == another.pos); }
+  bool operator!=(const hit_type& another) const { return (this->pos != another.pos); }
+  bool operator<(const hit_type& another) const { return (this->pos < another.pos); }
+  bool operator>(const hit_type& another) const { return (this->pos > another.pos); }
+};
+
+struct Position
+{
+  // Constants for the encoding between pos_t and code_type.
+  constexpr static size_t    OFFSET_BITS = 10;
+  constexpr static size_t    ID_OFFSET   = OFFSET_BITS + 1;
+  constexpr static code_type REV_MASK    = static_cast<code_type>(1) << OFFSET_BITS;
+  constexpr static code_type OFF_MASK    = REV_MASK - 1;
+
+  // Is the offset small enough to fit in the low-order bits of the encoding?
+  static bool valid_offset(const pos_t& pos) { return (offset(pos) <= OFF_MASK); }
+
+  // Encode pos_t as code_type.
+  static code_type encode(const pos_t& pos)
+  {
+    return (static_cast<code_type>(gbwtgraph::id(pos)) << ID_OFFSET) |
+           (static_cast<code_type>(gbwtgraph::is_rev(pos)) << OFFSET_BITS) |
+           (static_cast<code_type>(gbwtgraph::offset(pos)) & OFF_MASK);
+  }
+
+  // Decode code_type as pos_t.
+  static pos_t decode(code_type pos) { return make_pos_t(pos >> ID_OFFSET, pos & REV_MASK, pos & OFF_MASK); }
+
+  // Return the node id in the encoded position.
+  static nid_t id(code_type pos) { return (pos >> ID_OFFSET); }
+};
+
+//------------------------------------------------------------------------------
+
 /*
   A kmer encoded using 2 bits/character in a 64-bit integer. The encoding is only
-  defined if all characters in the kmer are valid.
+  defined if all characters in the kmer are valid. The highest bit indicates
+  whether the corresponding value in the hash table is a position or a pointer.
 */
 
 struct Key128;
@@ -79,24 +128,33 @@ struct Key64
 public:
   // Internal representation.
   typedef std::uint64_t key_type;
+  typedef key_type value_type;
   key_type key;
 
   // Empty key.
   constexpr Key64() : key(EMPTY_KEY) {}
 
   // No key.
-  constexpr static Key64 no_key() { return Key64(NO_KEY); }
+  constexpr static Key64 no_key() { return Key64(NO_KEY & KEY_MASK); }
 
   // Implicit conversion for testing.
   constexpr Key64(key_type value) : key(value) {}
 
+  // Get a representation of the actual key.
+  value_type get_key() const { return (this->key & KEY_MASK); }
+
   // Comparisons.
-  bool operator==(Key64 another) const { return (this->key == another.key); }
-  bool operator!=(Key64 another) const { return (this->key != another.key); }
-  bool operator<(Key64 another) const { return (this->key < another.key); }
+  bool operator==(Key64 another) const { return (this->get_key() == another.get_key()); }
+  bool operator!=(Key64 another) const { return (this->get_key() != another.get_key()); }
+  bool operator<(Key64 another) const { return (this->get_key() < another.get_key()); }
+
+  // Is the corresponding value a pointer?
+  bool is_pointer() const { return (this->key & IS_POINTER); }
+  void clear_pointer() { this->key &= ~IS_POINTER; }
+  void set_pointer() { this->key |= IS_POINTER; }
 
   // Hash of the key.
-  size_t hash() const { return gbwt::wang_hash_64(this->key); }
+  size_t hash() const { return wang_hash_64(this->key & KEY_MASK); }
 
   // Move the kmer forward, with c as the next character. Update the key, assuming that
   // it encodes the kmer in forward orientation.
@@ -124,6 +182,9 @@ public:
     }
   }
 
+  /// Encode a string of size k to a key.
+  static Key64 encode(const std::string& sequence);
+
   /// Decode the key back to a string, given the kmer size used.
   std::string decode(size_t k) const;
 
@@ -134,9 +195,11 @@ public:
   constexpr static std::size_t KMER_MAX_LENGTH = 31;
 
 private:
-  // Specific key values.
+  // Specific key values. Note that the highest bit is not a part of the key.
   constexpr static key_type EMPTY_KEY = 0;
   constexpr static key_type NO_KEY = std::numeric_limits<key_type>::max();
+  constexpr static key_type KEY_MASK = NO_KEY >> 1;
+  constexpr static key_type IS_POINTER = NO_KEY ^ KEY_MASK; // High bit.
 
   // Constants for the encoding between std::string and the key.
   constexpr static size_t   PACK_WIDTH = 2;
@@ -157,7 +220,8 @@ std::ostream& operator<<(std::ostream& out, Key64 value);
 
 /*
   A kmer encoded using 2 bits/character in a pair of 64-bit integers. The encoding is
-  only defined if all characters in the kmer are valid.
+  only defined if all characters in the kmer are valid. The highest bit indicates
+  whether the corresponding value in the hash table is a position or a pointer.
 */
 
 struct Key128
@@ -165,6 +229,7 @@ struct Key128
 public:
   // Internal representation.
   typedef std::uint64_t key_type;
+  typedef std::pair<key_type, key_type> value_type;
   constexpr static std::size_t FIELD_BITS = sizeof(key_type) * gbwt::BYTE_BITS;
   key_type high, low;
 
@@ -172,7 +237,7 @@ public:
   constexpr Key128() : high(EMPTY_KEY), low(EMPTY_KEY) {}
 
   // No key.
-  constexpr static Key128 no_key() { return Key128(NO_KEY, NO_KEY); }
+  constexpr static Key128 no_key() { return Key128(NO_KEY & KEY_MASK, NO_KEY); }
 
   // Implicit conversion for testing.
   constexpr Key128(key_type key) : high(EMPTY_KEY), low(key) {}
@@ -180,20 +245,24 @@ public:
   // For testing.
   constexpr Key128(key_type high, key_type low) : high(high), low(low) {}
 
+  // Get a representation of the actual key.
+  value_type get_key() const { return value_type(this->high & KEY_MASK, this->low); }
+
   // Comparisons.
-  bool operator==(Key128 another) const { return (this->high == another.high && this->low == another.low); }
-  bool operator!=(Key128 another) const { return (this->high != another.high || this->low != another.low); }
-  bool operator<(Key128 another) const
-  {
-    if(this->high != another.high) { return (this->high < another.high); }
-    return (this->low < another.low);
-  }
+  bool operator==(Key128 another) const { return (this->get_key() == another.get_key()); }
+  bool operator!=(Key128 another) const { return (this->get_key() != another.get_key()); }
+  bool operator<(Key128 another) const { return (this->get_key() < another.get_key()); }
+
+  // Is the corresponding value a pointer?
+  bool is_pointer() const { return (this->high & IS_POINTER); }
+  void clear_pointer() { this->high &= ~IS_POINTER; }
+  void set_pointer() { this->high |= IS_POINTER; }
 
   // Hash of the key. Essentially boost::hash_combine.
   size_t hash() const
   {
-    size_t result = gbwt::wang_hash_64(this->high);
-    result ^= gbwt::wang_hash_64(this->low) + 0x9e3779b9 + (result << 6) + (result >> 2);
+    size_t result = wang_hash_64(this->high & KEY_MASK);
+    result ^= wang_hash_64(this->low) + 0x9e3779b9 + (result << 6) + (result >> 2);
     return result;
   }
 
@@ -232,6 +301,9 @@ public:
     }
   }
 
+  /// Encode a string of size k to a key.
+  static Key128 encode(const std::string& sequence);
+
   /// Decode the key back to a string, given the kmer size used.
   std::string decode(size_t k) const;
 
@@ -242,9 +314,11 @@ public:
   constexpr static std::size_t KMER_MAX_LENGTH = 63;
 
 private:
-  // Specific key values.
+  // Specific key values. Note that the highest bit is not a part of the key.
   constexpr static key_type EMPTY_KEY = 0;
   constexpr static key_type NO_KEY = std::numeric_limits<key_type>::max();
+  constexpr static key_type KEY_MASK = NO_KEY >> 1;
+  constexpr static key_type IS_POINTER = NO_KEY ^ KEY_MASK; // High bit.
 
   // Constants for the encoding between std::string and the key.
   constexpr static size_t   PACK_WIDTH    = 2;
@@ -265,6 +339,7 @@ std::ostream& operator<<(std::ostream& out, Key128 value);
 
 /*
   A class that implements the minimizer index as a hash table mapping kmers to sets of pos_t.
+  For each stored position, we also store 64 bits of payload for external purposes.
   The hash table uses quadratic probing with power-of-two size.
   We encode kmers using 2 bits/character and hash the encoding. A minimizer is the kmer with
   the smallest hash in a window of w consecutive kmers and their reverse complements.
@@ -282,36 +357,41 @@ std::ostream& operator<<(std::ostream& out, Key128 value);
     4  Use SDSL data structures where appropriate. Not compatible with version 3.
 
     5  An option to use 128-bit keys. Compatible with version 4.
+
+    6  Store the position/pointer bit in the key instead of a separate bit vector. Store
+       64 bits of payload for each position.
+       Not compatible with version 5.
 */
 
-template<class KeyType = Key64>
+template<class KeyType>
 class MinimizerIndex
 {
 public:
   typedef KeyType key_type;
-  typedef std::uint64_t code_type;
   typedef std::uint32_t offset_type;
 
   // Public constants.
-  constexpr static size_t    INITIAL_CAPACITY = 1024;
-  constexpr static double    MAX_LOAD_FACTOR  = 0.77;
-  constexpr static code_type NO_VALUE         = 0;
+  constexpr static size_t       INITIAL_CAPACITY = 1024;
+  constexpr static double       MAX_LOAD_FACTOR  = 0.77;
+  constexpr static code_type    NO_VALUE         = 0;
+  constexpr static payload_type DEFAULT_PAYLOAD  = 0;
 
   // Serialize the hash table in blocks of this many cells.
-  constexpr static size_t    BLOCK_SIZE       = 4 * gbwt::MEGABYTE;
+  constexpr static size_t BLOCK_SIZE = 4 * gbwt::MEGABYTE;
 
   const static std::string EXTENSION; // ".min"
 
   union value_type
   {
-    code_type               value;
-    std::vector<code_type>* pointer;
+    hit_type value;
+    std::vector<hit_type>* pointer;
   };
 
   typedef std::pair<key_type, value_type> cell_type;
 
-  // TODO: This should be constexpr in C++14.
-  static cell_type empty_cell() { return cell_type(key_type::no_key(), { NO_VALUE }); }
+  // TODO: These should be constexpr in C++14.
+  static hit_type empty_hit() { return { NO_VALUE, DEFAULT_PAYLOAD }; }
+  static cell_type empty_cell() { return cell_type(key_type::no_key(), { empty_hit() }); }
 
   /*
     The sequence offset of a minimizer is the base that corresponds to the start of the
@@ -352,16 +432,14 @@ public:
 
   MinimizerIndex() :
     header(KeyType::KMER_LENGTH, KeyType::WINDOW_LENGTH, INITIAL_CAPACITY, MAX_LOAD_FACTOR, KeyType::KEY_BITS),
-    hash_table(this->header.capacity, empty_cell()),
-    is_pointer(this->header.capacity, 0)
+    hash_table(this->header.capacity, empty_cell())
   {
     this->header.sanitize(KeyType::KMER_MAX_LENGTH);
   }
 
   MinimizerIndex(size_t kmer_length, size_t window_length) :
     header(kmer_length, window_length, INITIAL_CAPACITY, MAX_LOAD_FACTOR, KeyType::KEY_BITS),
-    hash_table(this->header.capacity, empty_cell()),
-    is_pointer(this->header.capacity, 0)
+    hash_table(this->header.capacity, empty_cell())
   {
     this->header.sanitize(KeyType::KMER_MAX_LENGTH);
   }
@@ -386,7 +464,6 @@ public:
     if(&another == this) { return; }
     std::swap(this->header, another.header);
     this->hash_table.swap(another.hash_table);
-    this->is_pointer.swap(another.is_pointer);
   }
 
   MinimizerIndex& operator=(const MinimizerIndex& source)
@@ -401,7 +478,6 @@ public:
     {
       this->header = std::move(source.header);
       this->hash_table = std::move(source.hash_table);
-      this->is_pointer = std::move(source.is_pointer);
     }
     return *this;
   }
@@ -414,13 +490,15 @@ public:
     bool ok = true;
 
     bytes += io::serialize(out, this->header, ok);
-    bytes += io::serialize_hash_table(out, this->hash_table, this->is_pointer, NO_VALUE, ok);
-    bytes += this->is_pointer.serialize(out); // We do not know if this was successful.
+    bytes += io::serialize_hash_table(out, this->hash_table, empty_hit(), ok);
 
     // Serialize the occurrence lists.
     for(size_t i = 0; i < this->capacity(); i++)
     {
-      if(this->is_pointer[i]) { bytes += io::serialize_vector(out, *(this->hash_table[i].second.pointer), ok); }
+      if(this->hash_table[i].first.is_pointer())
+      {
+        bytes += io::serialize_vector(out, *(this->hash_table[i].second.pointer), ok);
+      }
     }
 
     if(!ok)
@@ -444,16 +522,7 @@ public:
       std::cerr << "MinimizerIndex::deserialize(): Index version is " << this->header.version << "; expected " << MinimizerHeader::VERSION << std::endl;
       return false;
     }
-    // TODO: This depends on index version.
-    if(this->header.version == MinimizerHeader::FIRST_VERSION)
-    {
-      if(KeyType::KEY_BITS != Key64::KEY_BITS)
-      {
-        std::cerr << "MinimizerIndex::deserialize(): Cannot load version " << this->header.version << " into a " << KeyType::KEY_BITS << "-bit index" << std::endl;
-        return false;
-      }
-    }
-    else if(this->header.key_bits() != KeyType::KEY_BITS)
+    if(this->header.key_bits() != KeyType::KEY_BITS)
     {
       std::cerr << "MinimizerIndex::deserialize(): Expected " << KeyType::KEY_BITS << "-bit keys, got " << this->header.key_bits() << "-bit keys" << std::endl;
       return false;
@@ -462,16 +531,15 @@ public:
 
     // Load the hash table.
     if(ok) { ok &= io::load_vector(in, this->hash_table); }
-    if(ok) { this->is_pointer.load(in); }
 
     // Load the occurrence lists.
     if(ok)
     {
       for(size_t i = 0; i < this->capacity(); i++)
       {
-        if(this->is_pointer[i])
+        if(this->hash_table[i].first.is_pointer())
         {
-          this->hash_table[i].second.pointer = new std::vector<code_type>();
+          this->hash_table[i].second.pointer = new std::vector<hit_type>();
           ok &= io::load_vector(in, *(this->hash_table[i].second.pointer));
         }
       }
@@ -488,13 +556,14 @@ public:
   // For testing.
   bool operator==(const MinimizerIndex& another) const
   {
-    if(this->header != another.header || this->is_pointer != another.is_pointer) { return false; }
+    if(this->header != another.header) { return false; }
 
     for(size_t i = 0; i < this->capacity(); i++)
     {
       cell_type a = this->hash_table[i], b = another.hash_table[i];
       if(a.first != b.first) { return false; }
-      if(this->is_pointer[i])
+      if(a.first.is_pointer() != b.first.is_pointer()) { return false; }
+      if(a.first.is_pointer())
       {
         if(*(a.second.pointer) != *(b.second.pointer)) { return false; }
       }
@@ -553,7 +622,7 @@ public:
   /*
     A circular buffer of size 2^i for all minimizer candidates. The candidates are sorted
     by both key and sequence offset. The candidate at offset j is removed when we reach
-    offset j + w. Candidates at the tail are removed when we advance with a smaller key.
+    offset j + w. Candidates at the tail are removed when we advance with a smaller hash.
   */
   struct CircularBuffer
   {
@@ -602,7 +671,8 @@ public:
   /*
     Returns all minimizers in the string specified by the iterators. The return
     value is a vector of minimizers sorted by their offsets. If there are multiple
-    occurrences of a minimizer in a window, return all of them.
+    occurrences of one or more minimizer keys with the same hash in a window,
+    return all of them.
   */
   std::vector<minimizer_type> minimizers(std::string::const_iterator begin, std::string::const_iterator end) const
   {
@@ -631,10 +701,10 @@ public:
         // 1) this is the first minimizer we encounter;
         // 2) the last reported minimizer had the same key (we may have new occurrences); or
         // 3) the first candidate is located after the last reported minimizer.
-        if(result.empty() || result.back().key == buffer.front().key || result.back().offset < buffer.front().offset)
+        if(result.empty() || result.back().hash == buffer.front().hash || result.back().offset < buffer.front().offset)
         {
           // Insert all new occurrences of the minimizer in the window.
-          for(size_t i = buffer.begin(); i < buffer.end() && buffer.at(i).key == buffer.front().key; i++)
+          for(size_t i = buffer.begin(); i < buffer.end() && buffer.at(i).hash == buffer.front().hash; i++)
           {
             if(buffer.at(i).offset >= next_read_offset)
             {
@@ -668,22 +738,25 @@ public:
   
   /*
     Returns all minimizers in the string specified by the iterators, together
-    with the weight of how many windows they arise from. The return value is a
-    vector of pairs of minimizers and window counts sorted by their offsets. If
-    there are multiple occurrences of a minimizer in a window, they are all
-    returned, but the window's weight is all assigned to an arbitrary
-    minimizer that it contains.
+    with the start and length of the run of windows they arise from. The return
+    value is a vector of tuples of minimizers, starts, and lengths, sorted by
+    minimizer offset.
   */
-  std::vector<std::pair<minimizer_type, size_t>> weighted_minimizers(std::string::const_iterator begin, std::string::const_iterator end) const
+  std::vector<std::tuple<minimizer_type, size_t, size_t>> minimizer_regions(std::string::const_iterator begin, std::string::const_iterator end) const
   {
-    std::vector<std::pair<minimizer_type, size_t>> result;
+    std::vector<std::tuple<minimizer_type, size_t, size_t>> result;
     size_t window_length = this->k() + this->w() - 1, total_length = end - begin;
     if(total_length < window_length) { return result; }
     
     // Find the minimizers.
     CircularBuffer buffer(this->w());
+    // Note that start_pos isn't meaningfully the start of the window we are
+    // looking at.
     size_t valid_chars = 0, start_pos = 0;
     size_t next_read_offset = 0;  // The first read offset that may contain a new minimizer.
+    // All results before this are finished and have their lengths filled in.
+    // All results after are current winning minimizers of the current window.
+    size_t finished_through = 0; 
     key_type forward_key, reverse_key;
     std::string::const_iterator iter = begin;
     while(iter != end)
@@ -696,39 +769,79 @@ public:
       if(valid_chars >= this->k()) { buffer.advance(start_pos, forward_key, reverse_key); }
       else                         { buffer.advance(start_pos); }
       ++iter;
-      // If we have at least k valid characters, we can advance the starting position of the next kmer.
       if(static_cast<size_t>(iter - begin) >= this->k()) { start_pos++; }
-      // We have a full window with a minimizer.
-      if(static_cast<size_t>(iter - begin) >= window_length && !buffer.empty())
+      
+      // We have a full window.
+      if(static_cast<size_t>(iter - begin) >= window_length)
       {
-        // Insert the candidates if:
-        // 1) this is the first minimizer we encounter;
-        // 2) the last reported minimizer had the same key (we may have new occurrences); or
-        // 3) the first candidate is located after the last reported minimizer.
-        if(result.empty() || result.back().first.key == buffer.front().key || result.back().first.offset < buffer.front().offset)
+        // Work out where the window we are minimizing in began
+        size_t window_start = static_cast<size_t>(iter - begin) - window_length;
+        
+        // Work out the past-the-end index of the window we have just finished (not the current window)
+        size_t prev_past_end_pos = window_start + window_length - 1;
+      
+        // Finish off end positions for results that weren't replaced but are going out of range
+        while(finished_through < result.size() &&
+          std::get<0>(result[finished_through]).offset < window_start)
         {
-          // Insert all new occurrences of the minimizer in the window.
-          for(size_t i = buffer.begin(); i < buffer.end() && buffer.at(i).key == buffer.front().key; i++)
+          // Compute region length based on it stopping at the previous step
+          std::get<2>(result[finished_through]) = prev_past_end_pos - std::get<1>(result[finished_through]);
+          finished_through++;
+        }
+      
+        // Our full window has a minimizer in it
+        if (!buffer.empty())
+        {
+        
+          // Insert the candidates if:
+          // 1) this is the first minimizer we encounter;
+          // 2) the last reported minimizer had the same hash (we may have new occurrences); or
+          // 3) the first candidate is located after the last reported minimizer.
+          if(result.empty() ||
+            std::get<0>(result.back()).hash == buffer.front().hash ||
+            std::get<0>(result.back()).offset < buffer.front().offset)
           {
-            if(buffer.at(i).offset >= next_read_offset)
+            // Insert all new occurrences of the minimizer in the window.
+            for(size_t i = buffer.begin(); i < buffer.end() && buffer.at(i).hash == buffer.front().hash; i++)
             {
-              result.emplace_back(buffer.at(i), 0);
-              next_read_offset = buffer.at(i).offset + 1;
+              if(buffer.at(i).offset >= next_read_offset)
+              {
+                // Insert the minimizer instance, with its region starting
+                // where the window covered by the buffer starts.
+                result.emplace_back(buffer.at(i), window_start, 0);
+                // There can only ever really be one minimizer at a given start
+                // position. So look for the next one 1 base to the right.
+                next_read_offset = buffer.at(i).offset + 1;
+              }
+            }
+            
+            // If new minimizers beat out old ones, finish off the old ones.
+            while(!result.empty() &&
+              finished_through < result.size() &&
+              std::get<0>(result.back()).hash != std::get<0>(result[finished_through]).hash)
+            {
+              // The window before the one we are looking at was the last one for this minimizer.
+              std::get<2>(result[finished_through]) = prev_past_end_pos - std::get<1>(result[finished_through]);
+              finished_through++;
             }
           }
         }
-        
-        // Assign the window's weight to an arbitrary minimizer that occured in it.
-        // Whatever is last in result right now will work.
-        result.back().second++;
       }
     }
 
+    // Now close off the minimizers left active when we hit the end of the string.
+    while(finished_through < result.size())
+    {
+      // The region length is from the region start to the string end
+      std::get<2>(result[finished_through]) = total_length - std::get<1>(result[finished_through]);
+      finished_through++;
+    }
+
     // It was more convenient to use the first offset of the kmer, regardless of the orientation.
     // If the minimizer is a reverse complement, we must return the last offset instead.
-    for(auto& weighted_minimizer : result)
+    for(auto& record : result)
     {
-      if(weighted_minimizer.first.is_reverse) { weighted_minimizer.first.offset += this->k() - 1; }
+      if(std::get<0>(record).is_reverse) { std::get<0>(record).offset += this->k() - 1; }
     }
     std::sort(result.begin(), result.end());
 
@@ -737,61 +850,63 @@ public:
   
   /*
     Returns all minimizers in the string. The return value is a vector of
-    minimizers sorted by their offsets.
+    minimizers, region starts, and region lengths sorted by their offsets.
   */
-  std::vector<std::pair<minimizer_type, size_t>> weighted_minimizers(const std::string& str) const
+  std::vector<std::tuple<minimizer_type, size_t, size_t>> minimizer_regions(const std::string& str) const
   {
-    return this->weighted_minimizers(str.begin(), str.end());
+    return this->minimizer_regions(str.begin(), str.end());
   }
 
 //------------------------------------------------------------------------------
 
   /*
-    Inserts the position into the index, using minimizer.key as the key and
-    minimizer.hash as its hash. Does not insert empty minimizers or positions.
+    Inserts the position and the payload into the index, using minimizer.key as
+    the key and minimizer.hash as its hash. Does not insert empty minimizers or
+    positions. Does not update the payload if the position has already been
+    inserted with the same key.
     The offset of the position will be truncated to fit in OFFSET_BITS bits.
-    Use minimizer() or minimizers() to get the minimizer and valid_offset() to check
-    if the offset fits in the available space.
+    Use minimizer() or minimizers() to get the minimizer and
+    Position::valid_offset() to check if the offset fits in the available space.
     The position should match the orientation of the minimizer: a path label
     starting from the position should have the minimizer as its prefix.
   */
-  void insert(const minimizer_type& minimizer, const pos_t& pos)
+  void insert(const minimizer_type& minimizer, const pos_t& pos, payload_type payload = DEFAULT_PAYLOAD)
   {
     if(minimizer.empty() || is_empty(pos)) { return; }
 
     size_t offset = this->find_offset(minimizer.key, minimizer.hash);
-    code_type code = encode(pos);
+    code_type code = Position::encode(pos);
     if(this->hash_table[offset].first == key_type::no_key())
     {
-      this->insert(minimizer.key, code, offset);
+      this->insert(minimizer.key, { code, payload }, offset);
     }
     else if(this->hash_table[offset].first == minimizer.key)
     {
-      this->append(code, offset);
+      this->append({ code, payload }, offset);
     }
   }
 
   /*
-    Returns the sorted set of occurrences of the minimizer.
+    Returns the sorted set of occurrences of the minimizer with their payloads.
     Use minimizer() or minimizers() to get the minimizer.
     If the minimizer is in reverse orientation, use reverse_base_pos() to reverse
     the reported occurrences.
   */
-  std::vector<pos_t> find(const minimizer_type& minimizer) const
+  std::vector<std::pair<pos_t, payload_type>> find(const minimizer_type& minimizer) const
   {
-    std::vector<pos_t> result;
+    std::vector<std::pair<pos_t, payload_type>> result;
     if(minimizer.empty()) { return result; }
 
     size_t offset = this->find_offset(minimizer.key, minimizer.hash);
-    cell_type cell = this->hash_table[offset];
+    const cell_type& cell = this->hash_table[offset];
     if(cell.first == minimizer.key)
     {
-      if(this->is_pointer[offset])
+      if(cell.first.is_pointer())
       {
         result.reserve(cell.second.pointer->size());
-        for(code_type pos : *(cell.second.pointer)) { result.emplace_back(decode(pos)); }
+        for(hit_type hit : *(cell.second.pointer)) { result.emplace_back(Position::decode(hit.pos), hit.payload); }
       }
-      else { result.emplace_back(decode(cell.second.value)); }
+      else { result.emplace_back(Position::decode(cell.second.value.pos), cell.second.value.payload); }
     }
 
     return result;
@@ -806,9 +921,10 @@ public:
     if(minimizer.empty()) { return 0; }
 
     size_t offset = this->find_offset(minimizer.key, minimizer.hash);
-    if(this->hash_table[offset].first == minimizer.key)
+    const cell_type& cell = this->hash_table[offset];
+    if(cell.first == minimizer.key)
     {
-      return (this->is_pointer[offset] ? this->hash_table[offset].second.pointer->size() : 1);
+      return (cell.first.is_pointer() ? cell.second.pointer->size() : 1);
     }
 
     return 0;
@@ -816,21 +932,23 @@ public:
 
   /*
     Returns the occurrence count of the minimizer and a pointer to the internal
-    representation of the occurrences (which are in sorted order). The pointer may be
-    invalidated if new positions are inserted into the index.
-    Use minimizer() or minimizers() to get the minimizer and decode() to decode the
-    occurrences.
+    representation of the occurrences (which are in sorted order) and their payloads.
+    The pointer may be invalidated if new positions are inserted into the index.
+    Use minimizer() or minimizers() to get the minimizer and Position::decode() to
+    decode the occurrences.
+    If the minimizer is in reverse orientation, use reverse_base_pos() to reverse
+    the reported occurrences.
   */
-  std::pair<size_t, const code_type*> count_and_find(const minimizer_type& minimizer) const
+  std::pair<size_t, const hit_type*> count_and_find(const minimizer_type& minimizer) const
   {
-    std::pair<size_t, const code_type*> result(0, nullptr);
+    std::pair<size_t, const hit_type*> result(0, nullptr);
     if(minimizer.empty()) { return result; }
 
     size_t offset = this->find_offset(minimizer.key, minimizer.hash);
     if(this->hash_table[offset].first == minimizer.key)
     {
       const cell_type& cell = this->hash_table[offset];
-      if(this->is_pointer[offset])
+      if(cell.first.is_pointer())
       {
         result.first = cell.second.pointer->size();
         result.second = cell.second.pointer->data();
@@ -877,31 +995,6 @@ public:
 private:
   MinimizerHeader        header;
   std::vector<cell_type> hash_table;
-  sdsl::bit_vector       is_pointer;
-
-//------------------------------------------------------------------------------
-
-public:
-
-  // Constants for the encoding between pos_t and code_type.
-  constexpr static size_t    OFFSET_BITS = 10;
-  constexpr static size_t    ID_OFFSET   = OFFSET_BITS + 1;
-  constexpr static code_type REV_MASK    = static_cast<code_type>(1) << OFFSET_BITS;
-  constexpr static code_type OFF_MASK    = REV_MASK - 1;
-
-  // Is the offset small enough to fit in the low-order bits of the encoding?
-  static bool valid_offset(const pos_t& pos) { return (offset(pos) <= OFF_MASK); }
-
-  // Encode pos_t as code_type.
-  static code_type encode(const pos_t& pos)
-  {
-    return (static_cast<code_type>(id(pos)) << ID_OFFSET) |
-           (static_cast<code_type>(is_rev(pos)) << OFFSET_BITS) |
-           (static_cast<code_type>(offset(pos)) & OFF_MASK);
-  }
-
-  // Decode code_type as pos_t.
-  static pos_t decode(code_type pos) { return make_pos_t(pos >> ID_OFFSET, pos & REV_MASK, pos & OFF_MASK); }
 
 //------------------------------------------------------------------------------
 
@@ -911,7 +1004,6 @@ private:
     this->clear();
     this->header = source.header;
     this->hash_table = source.hash_table;
-    this->is_pointer = source.is_pointer;
   }
 
   // Delete all pointers in the hash table.
@@ -919,11 +1011,12 @@ private:
   {
     for(size_t i = 0; i < this->hash_table.size(); i++)
     {
-      if(this->is_pointer[i])
+      cell_type& cell = this->hash_table[i];
+      if(cell.first.is_pointer())
       {
-        delete this->hash_table[i].second.pointer;
-        this->hash_table[i].second.value = NO_VALUE;
-        this->is_pointer[i] = false;
+        delete cell.second.pointer;
+        cell.second.value = empty_hit();
+        cell.first.clear_pointer();
       }
     }
   }
@@ -945,12 +1038,12 @@ private:
     return 0;
   }
 
-  // Insert (key, pos) to hash_table[offset], which is assumed to be empty.
+  // Insert (key, hit) to hash_table[offset], which is assumed to be empty.
   // Rehashing may be necessary.
-  void insert(key_type key, code_type pos, size_t offset)
+  void insert(key_type key, hit_type hit, size_t offset)
   {
     this->hash_table[offset].first = key;
-    this->hash_table[offset].second.value = pos;
+    this->hash_table[offset].second.value = hit;
     this->header.keys++;
     this->header.values++;
     this->header.unique++;
@@ -959,14 +1052,15 @@ private:
   }
 
   // Add pos to the list of occurrences of key at hash_table[offset].
-  void append(code_type pos, size_t offset)
+  void append(hit_type hit, size_t offset)
   {
-    if(this->contains(offset, pos)) { return; }
+    if(this->contains(offset, hit)) { return; }
 
-    if(this->is_pointer[offset])
+    cell_type& cell = this->hash_table[offset];
+    if(cell.first.is_pointer())
     {
-      std::vector<code_type>* occs = this->hash_table[offset].second.pointer;
-      occs->push_back(pos);
+      std::vector<hit_type>* occs = cell.second.pointer;
+      occs->push_back(hit);
       size_t offset = occs->size() - 1;
       while(offset > 0 && occs->at(offset - 1) > occs->at(offset))
       {
@@ -976,28 +1070,29 @@ private:
     }
     else
     {
-      std::vector<code_type>* occs = new std::vector<code_type>(2);
-      occs->at(0) = this->hash_table[offset].second.value;
-      occs->at(1) = pos;
+      std::vector<hit_type>* occs = new std::vector<hit_type>(2);
+      occs->at(0) = cell.second.value;
+      occs->at(1) = hit;
       if(occs->at(0) > occs->at(1)) { std::swap(occs->at(0), occs->at(1)); }
-      this->hash_table[offset].second.pointer = occs;
-      this->is_pointer[offset] = true;
+      cell.second.pointer = occs;
+      cell.first.set_pointer();
       this->header.unique--;
     }
     this->header.values++;
   }
 
-  // Does the list of occurrences at hash_table[offset] contain pos?
-  bool contains(size_t offset, code_type pos) const
+  // Does the list of occurrences at hash_table[offset] contain the hit?
+  bool contains(size_t offset, hit_type hit) const
   {
-    if(this->is_pointer[offset])
+    const cell_type& cell = this->hash_table[offset];
+    if(cell.first.is_pointer())
     {
-      const std::vector<code_type>* occs = this->hash_table[offset].second.pointer;
-      return std::binary_search(occs->begin(), occs->end(), pos);
+      const std::vector<hit_type>* occs = cell.second.pointer;
+      return std::binary_search(occs->begin(), occs->end(), hit);
     }
     else
     {
-      return (this->hash_table[offset].second.value == pos);
+      return (cell.second.value == hit);
     }
   }
 
@@ -1006,21 +1101,18 @@ private:
   {
     // Reinitialize with a larger hash table.
     std::vector<cell_type> old_hash_table(2 * this->capacity(), empty_cell());
-    sdsl::bit_vector old_is_pointer(2 * this->capacity(), 0);
     this->hash_table.swap(old_hash_table);
-    this->is_pointer.swap(old_is_pointer);
     this->header.capacity = this->hash_table.size();
     this->header.max_keys = this->capacity() * MAX_LOAD_FACTOR;
 
     // Move the keys to the new hash table.
     for(size_t i = 0; i < old_hash_table.size(); i++)
     {
-      key_type key = old_hash_table[i].first;
-      if(key == key_type::no_key()) { continue; }
+      const cell_type& source = old_hash_table[i];
+      if(source.first == key_type::no_key()) { continue; }
 
-      size_t offset = this->find_offset(key, key.hash());
-      this->hash_table[offset] = old_hash_table[i];
-      this->is_pointer[offset] = old_is_pointer[i];
+      size_t offset = this->find_offset(source.first, source.first.hash());
+      this->hash_table[offset] = source;
     }
   }
 };
@@ -1035,9 +1127,32 @@ operator<<(std::ostream& out, const typename MinimizerIndex<KeyType>::minimizer_
 
 //------------------------------------------------------------------------------
 
+/*
+  Decode the subset of minimizer hits and their payloads in the given subgraph induced
+  by node identifiers.
+  This version should only be used when the number of hits is small.
+  If the minimizer is in reverse orientation, use reverse_base_pos() to reverse
+  the reported occurrences.
+*/
+void hits_in_subgraph(size_t hit_count, const hit_type* hits, const std::unordered_set<nid_t>& subgraph,
+                      const std::function<void(pos_t, payload_type)>& report_hit);
+
+/*
+  Decode the subset of minimizer hits and their payloads in the given subgraph induced
+  by node identifiers. The set of node ids must be in sorted order.
+  This version uses exponential search on the larger list, so it should be efficient
+  regardless of the size of the subgraph and the number of hits.
+  If the minimizer is in reverse orientation, use reverse_base_pos() to reverse
+  the reported occurrences.
+*/
+void hits_in_subgraph(size_t hit_count, const hit_type* hits, const std::vector<nid_t>& subgraph,
+                      const std::function<void(pos_t, payload_type)>& report_hit);
+
+//------------------------------------------------------------------------------
+
 // Choose the default index type.
-//typedef MinimizerIndex<Key64> DefaultMinimizerIndex;
-typedef MinimizerIndex<Key128> DefaultMinimizerIndex;
+typedef MinimizerIndex<Key64> DefaultMinimizerIndex;
+//typedef MinimizerIndex<Key128> DefaultMinimizerIndex;
 
 //------------------------------------------------------------------------------
 
@@ -1045,12 +1160,8 @@ typedef MinimizerIndex<Key128> DefaultMinimizerIndex;
 
 template<class KeyType> constexpr size_t MinimizerIndex<KeyType>::INITIAL_CAPACITY;
 template<class KeyType> constexpr double MinimizerIndex<KeyType>::MAX_LOAD_FACTOR;
-template<class KeyType> constexpr typename MinimizerIndex<KeyType>::code_type MinimizerIndex<KeyType>::NO_VALUE;
-
-template<class KeyType> constexpr size_t MinimizerIndex<KeyType>::OFFSET_BITS;
-template<class KeyType> constexpr size_t MinimizerIndex<KeyType>::ID_OFFSET;
-template<class KeyType> constexpr typename MinimizerIndex<KeyType>::code_type MinimizerIndex<KeyType>::REV_MASK;
-template<class KeyType> constexpr typename MinimizerIndex<KeyType>::code_type MinimizerIndex<KeyType>::OFF_MASK;
+template<class KeyType> constexpr code_type MinimizerIndex<KeyType>::NO_VALUE;
+template<class KeyType> constexpr code_type MinimizerIndex<KeyType>::DEFAULT_PAYLOAD;
 
 // Other template class variables.
 
