@@ -4,6 +4,8 @@
 
 #include "bdsg/odgi.hpp"
 
+#include <unordered_set>
+
 namespace bdsg {
 
 void ODGI::serialize_members(std::ostream& out) const {
@@ -20,38 +22,131 @@ uint32_t ODGI::get_magic_number() const {
 
 /// Method to check if a node exists by ID
 bool ODGI::has_node(nid_t node_id) const {
-    uint64_t rank = get_node_rank(node_id);
+    if (graph_id_hidden_set.count(node_id)) {
+        // Hide this node.
+        return false;
+    }
+    uint64_t rank = id_to_rank(node_id);
     return (rank >= node_v.size() ? false : !deleted_node_bv.at(rank));
 }
 
 /// Look up the handle for the node with the given ID in the given orientation
 handle_t ODGI::get_handle(const nid_t& node_id, bool is_reverse) const {
-    return number_bool_packing::pack(get_node_rank(node_id), is_reverse);
+    return number_bool_packing::pack(id_to_rank(node_id), is_reverse);
 }
 
 /// Get the ID from a handle
 nid_t ODGI::get_id(const handle_t& handle) const {
-    return number_bool_packing::unpack_number(handle) + 1 + _id_increment;
+    return rank_to_id(number_bool_packing::unpack_number(handle));
 }
 
-/// get the backing node for a given node id
-uint64_t ODGI::get_node_rank(const nid_t& node_id) const {
-    return node_id - _id_increment - 1;
+/// get the backing node rank for a given node id
+uint64_t ODGI::id_to_rank(const nid_t& node_id) const {
+    return node_id - _id_increment;
+}
+
+/// get the node id for the node with the given backing rank
+nid_t ODGI::rank_to_id(const uint64_t& rank) const {
+    return rank + _id_increment;
 }
 
 /// set the id increment, used when the graph starts at a high id to reduce loading costs
 void ODGI::set_id_increment(const nid_t& min_id) {
+    if (min_id < 1) {
+        // Clamp at the low end to 1, because rank 0 needs at least ID 1.
+        set_id_increment(1);
+        return;
+    }
+    
+    if ((get_node_count() + graph_id_hidden_set.size()) > 0) {
+        increment_node_ids(min_id - _id_increment);
+    }
     _id_increment = min_id;
 }
 
 /// Add the given value to all node IDs
 void ODGI::increment_node_ids(nid_t increment) {
-    throw runtime_error("Not implemented");
+    if (deleted_node_bv.size() == _deleted_node_count) {
+        // Nothing is in this graph! Nothing to do!
+        return;
+    }
+
+    if (increment > 0) {
+        // Shift right
+        
+        // Make sure node and deletion vectors are big enough
+        size_t old_size = node_v.size();
+        size_t new_size = old_size + increment;
+        node_v.resize(new_size);
+        while (deleted_node_bv.size() < new_size + 1) {
+            // Put a bunch of new 0s before the final delimiting 1.
+            deleted_node_bv.insert(old_size, 0);
+        }
+        // Shift all the nodes and deletion flags up
+        for (size_t i = new_size - 1; i >= increment; i--) {
+            node_v[i] = node_v[i - increment];
+            deleted_node_bv[i] = deleted_node_bv[i - increment];
+        }
+        // Clear out the low end (everyone is deleted)
+        for (size_t i = 0; i < increment; i++) {
+            deleted_node_bv[i] = 1;
+        }
+        _deleted_node_count += increment;
+    } else if (increment < 0) {
+        // Shift left
+        
+        if (-increment >= node_v.size()) {
+            throw std::runtime_error("Cannot shift IDs left by " + std::to_string(-increment) +
+                " because graph nodes would be destroyed.");
+        }
+        
+        for (size_t i = 0; i < -increment; i++) {
+            // Make sure all the slots we are clobbering are empty.
+            if (!deleted_node_bv[i]) {
+                throw std::runtime_error("Cannot shift IDs left by " + std::to_string(-increment) +
+                    " because graph nodes would be destroyed.");
+            }
+        }
+        
+        // Shift everything down
+        for (size_t i = 0; i < node_v.size() + increment; i++) {
+            node_v[i] = node_v[i - increment];
+            deleted_node_bv[i] = deleted_node_bv[i - increment];
+        }
+        // Clear out the high end (everyone is deleted).
+        // No need to touch the final delimiting 1.
+        for (size_t i = deleted_node_bv.size() + increment; i < deleted_node_bv.size() - 1; i++) {
+            deleted_node_bv[i] = 1;
+        }
+        
+        // Total number of deleted nodes is conserved.
+    } else {
+        // No shift. Nothing to do!
+        return;
+    }
+        
+    // Replace the hidden set
+    decltype(graph_id_hidden_set) new_hidden_set;
+    for (auto& item : graph_id_hidden_set) {
+        new_hidden_set.insert(item + increment);
+    }
+    graph_id_hidden_set = std::move(new_hidden_set);
+    
+    // Adjust the min and max node ranks
+    _max_node_rank += increment;
+    _min_node_rank += increment;
+
+    for (auto& path : path_metadata_v) {
+        // Increment all the ranks stored in the normal handles inside the step
+        // handles for path begin and end.
+        set_handle_of_step(path.first, add_to_number(get_handle_of_step(path.first), increment));
+        set_handle_of_step(path.last, add_to_number(get_handle_of_step(path.last), increment));
+    }
 }
 
 /// Reassign all node IDs as specified by the old->new mapping function.
 void ODGI::reassign_node_ids(const std::function<nid_t(const nid_t&)>& get_new_id) {
-    throw runtime_error("Not implemented");
+    throw runtime_error("Node IDs cannot yet be reassigned in the ODGI graph implementation.");
 }
     
 /// Get the orientation of a handle
@@ -123,7 +218,9 @@ bool ODGI::for_each_handle_impl(const std::function<bool(const handle_t&)>& iter
         for (uint64_t i = 0; i < node_v.size(); ++i) {
             if (deleted_node_bv.at(i) == 1) continue;
             if (!flag) continue;
-            bool result = iteratee(number_bool_packing::pack(i,false));
+            handle_t packed = number_bool_packing::pack(i,false);
+            if (graph_id_hidden_set.count(get_id(packed))) continue;
+            bool result = iteratee(packed);
 #pragma omp atomic
             flag &= result;
         }
@@ -131,28 +228,32 @@ bool ODGI::for_each_handle_impl(const std::function<bool(const handle_t&)>& iter
     } else {
         for (uint64_t i = 0; i < node_v.size(); ++i) {
             if (deleted_node_bv.at(i) == 1) continue;
-            if (!iteratee(number_bool_packing::pack(i,false))) return false;
+            handle_t packed = number_bool_packing::pack(i,false);
+            if (graph_id_hidden_set.count(get_id(packed))) continue;
+            if (!iteratee(packed)) return false;
         }
         return true;
     }
 }
 
-/// Return the number of nodes in the graph
+/// Return the number of nodes in the graph.
+/// Doesn't count deleted or hidden nodes.
 /// TODO: can't be node_count because XG has a field named node_count.
 size_t ODGI::get_node_count(void) const {
-    return node_v.size()-_deleted_node_count;
+    return node_v.size() - _deleted_node_count - graph_id_hidden_set.size();
 }
     
 /// Return the smallest ID in the graph, or some smaller number if the
 /// smallest ID is unavailable. Return value is unspecified if the graph is empty.
 nid_t ODGI::min_node_id(void) const {
-    return _min_node_id;
+    // May overflow for empty graphs.
+    return rank_to_id(_min_node_rank);
 }
     
 /// Return the largest ID in the graph, or some larger number if the
 /// largest ID is unavailable. Return value is unspecified if the graph is empty.
 nid_t ODGI::max_node_id(void) const {
-    return _max_node_id;
+    return rank_to_id(_max_node_rank);
 }
     
 ////////////////////////////////////////////////////////////////////////////
@@ -281,7 +382,7 @@ step_handle_t ODGI::path_back(const path_handle_t& path_handle) const {
 step_handle_t ODGI::path_front_end(const path_handle_t& path_handle) const {
     step_handle_t step;
     as_integers(step)[0] = as_integer(path_handle);
-    as_integers(step)[0] = std::numeric_limits<uint64_t>::max()-1;
+    as_integers(step)[1] = std::numeric_limits<uint64_t>::max()-1;
     return step;
 }
 
@@ -289,7 +390,7 @@ step_handle_t ODGI::path_front_end(const path_handle_t& path_handle) const {
 step_handle_t ODGI::path_end(const path_handle_t& path_handle) const {
     step_handle_t step;
     as_integers(step)[0] = as_integer(path_handle);
-    as_integers(step)[0] = std::numeric_limits<uint64_t>::max();
+    as_integers(step)[1] = std::numeric_limits<uint64_t>::max();
     return step;
 }
 
@@ -430,9 +531,11 @@ void ODGI::for_each_step_in_path(const path_handle_t& path, const std::function<
 handle_t ODGI::create_handle(const std::string& sequence) {
     // get first deleted node to recycle
     if (_deleted_node_count) {
-        return create_handle(sequence, deleted_node_bv.select1(0)+1);
+        uint64_t replaced_rank = deleted_node_bv.select1(0);
+        nid_t replaced_id = rank_to_id(replaced_rank);
+        return create_handle(sequence, replaced_id);
     } else {
-        return create_handle(sequence, node_v.size()+1);
+        return create_handle(sequence, rank_to_id(node_v.size()));
     }
 }
 
@@ -441,40 +544,55 @@ handle_t ODGI::create_hidden_handle(const std::string& sequence) {
     handle_t handle = create_handle(sequence);
     nid_t id = get_id(handle);
     graph_id_hidden_set.insert(id);
-    ++_hidden_count;
     return handle;
 }
 
 /// Create a new node with the given id and sequence, then return the handle.
 handle_t ODGI::create_handle(const std::string& sequence, const nid_t& id) {
-    assert(sequence.size());
-    assert(id > 0);
-    if (id > node_v.size()) {
-        uint64_t to_add = id - node_v.size();
+    
+    if (sequence.empty()) {
+        throw std::runtime_error("error:[ODGI] tried to create an empty node with ID " + std::to_string(id));
+    }
+    
+    if (id <= 0) {
+        throw std::runtime_error("error:[ODGI] tried to create a node with non-positive ID " + std::to_string(id));
+    }
+    
+    if (id < _id_increment) {
+        // We would need to change the ID increment to take this node, and
+        // invalidate existing handles, which create_handle can't do.
+        throw std::runtime_error("ODGI graph cannot accept a new node with ID " +
+            std::to_string(id) + " in a graph with ID increment " + 
+            std::to_string(_id_increment));
+    }
+    
+    // Give the node a rank, which is just how far above _id_increment it is.
+    uint64_t handle_rank = id_to_rank(id);
+    
+    if (handle_rank >= node_v.size()) {
         uint64_t old_size = node_v.size();
-        // realloc
-        node_v.resize((uint64_t)id);
+        // realloc to have a last entry handle_rank
+        node_v.resize(handle_rank + 1);
         _node_count = node_v.size();
-        // mark empty nodes
-        for (uint64_t i = 0; i < to_add; ++i) {
+        
+        // mark empty nodes (treat them as deleted)
+        uint64_t added = _node_count - old_size;
+        for (uint64_t i = 0; i < added; ++i) {
             // insert before final delimiter
             deleted_node_bv.insert(old_size, 1);
             ++_deleted_node_count;
         }
     }
-    // update min/max node ids
-    _max_node_id = max(id, _max_node_id);
-    if (_min_node_id) {
-        _min_node_id = (uint64_t)min(id, _min_node_id);
-    } else {
-        _min_node_id = id;
-    }
+    // update min/max node internal ranks
+    _max_node_rank = max(handle_rank, _max_node_rank);
+    _min_node_rank = min(handle_rank, _min_node_rank);
+    
     // add to node vector
-    uint64_t handle_rank = (uint64_t)id-1;
     // set its values
     auto& node = node_v[handle_rank];
     node.set_sequence(sequence);
     // it's not deleted
+    assert(deleted_node_bv[handle_rank]);
     deleted_node_bv[handle_rank] = 0;
     --_deleted_node_count;
     // return handle
@@ -482,11 +600,7 @@ handle_t ODGI::create_handle(const std::string& sequence, const nid_t& id) {
 }
     
 /// Remove the node belonging to the given handle and all of its edges.
-/// Does not update any stored paths.
-/// Invalidates the destroyed handle.
-/// May be called during serial for_each_handle iteration **ONLY** on the node being iterated.
-/// May **NOT** be called during parallel for_each_handle iteration.
-/// May **NOT** be called on the node from which edges are being followed during follow_edges.
+/// If any paths visit it, it becomes a "hidden" node accessible only via the paths.
 void ODGI::destroy_handle(const handle_t& handle) {
     handle_t fwd_handle = get_is_reverse(handle) ? flip(handle) : handle;
     uint64_t id = get_id(handle);
@@ -501,19 +615,29 @@ void ODGI::destroy_handle(const handle_t& handle) {
     for (auto& edge : edges_to_destroy) {
         destroy_edge(edge);
     }
-    // clear the node storage
+   
+    // Work out if it has any paths on it
     auto& node = node_v[number_bool_packing::unpack_number(handle)];
-    node.clear();
-    // remove from the graph by hiding it (compaction later)
-    deleted_node_bv[number_bool_packing::unpack_number(handle)] = 1;
-    // and from the set of hidden nodes, if it's a member
-    if (graph_id_hidden_set.count(id)) {
-        graph_id_hidden_set.erase(id);
-        --_hidden_count;
+    if (node.path_count() > 0) {
+        // Paths visit here, so hide the handle.
+        // We will get called again when the last one leaves.
+        graph_id_hidden_set.insert(id);
+    } else {
+        // No paths visit here. Destroy the handle and make sure it isn't hidden if it was.
+        
+        // clear the node storage
+        node.clear();
+        // remove from the graph by hiding it (compaction later)
+        deleted_node_bv[number_bool_packing::unpack_number(handle)] = 1;
+        // and from the set of hidden nodes, if it's a member
+        if (graph_id_hidden_set.count(id)) {
+            graph_id_hidden_set.erase(id);
+        }
+        ++_deleted_node_count;
+        // TODO: check if we should compact our deleted nodes storage
     }
-    //--_node_count;
-    ++_deleted_node_count;
-    // check if we should compact our deleted nodes storage
+    
+    
 }
 
 /*
@@ -655,16 +779,19 @@ void ODGI::destroy_edge(const handle_t& left_h, const handle_t& right_h) {
     _edge_count -= found_edge;
 }
         
-/// Remove all nodes and edges. Does not update any stored paths.
+/// Remove all nodes, edges, and paths.
 void ODGI::clear(void) {
     suc_bv null_bv;
-    _max_node_id = 0;
-    _min_node_id = 0;
+    _max_node_rank = 0;
+    _min_node_rank = std::numeric_limits<decltype(_min_node_rank)>::max();
+    _id_increment = 1;
     _node_count = 0;
     _edge_count = 0;
     _path_count = 0;
     _path_handle_next = 0;
     deleted_node_bv = null_bv;
+    // Deleted node BV needs a final delimiter.
+    deleted_node_bv.push_back(1);
     node_v.clear();
     path_metadata_v.clear();
     path_name_map.clear();
@@ -1158,6 +1285,12 @@ void ODGI::destroy_step(const step_handle_t& step_handle) {
         });
     node_t& curr_node = node_v.at(number_bool_packing::unpack_number(get_handle_of_step(step_handle)));
     curr_node.remove_path_step(as_integers(step_handle)[1]);
+    
+    if (curr_node.path_count() == 0 && graph_id_hidden_set.count(get_id(handle))) {
+        // We have freed up a hidden handle from its last reference. Clean it
+        // up and get rid of the now-useless node.
+        destroy_handle(handle);
+    }
 }
 
 step_handle_t ODGI::prepend_step(const path_handle_t& path, const handle_t& to_append) {
@@ -1231,6 +1364,18 @@ void ODGI::decrement_rank(const step_handle_t& step_handle) {
     }
 }
 
+/// Modify the given step handle to point to the given handle.
+void ODGI::set_handle_of_step(step_handle_t& step_handle, const handle_t& handle) const {
+    as_integers(step_handle)[0] = as_integer(handle);
+}
+    
+/// Add the offset to the number packed in the given handle, and return a
+/// new modified handle.
+handle_t ODGI::add_to_number(const handle_t& handle, int64_t offset) const {
+    return number_bool_packing::pack(number_bool_packing::unpack_number(handle) + offset,
+        number_bool_packing::unpack_bit(handle));
+}
+
 /// reassign the given step to the new handle
 step_handle_t ODGI::set_step(const step_handle_t& step_handle, const handle_t& assign_to) {
     return rewrite_segment(step_handle, step_handle, { assign_to }).first;
@@ -1302,15 +1447,16 @@ std::pair<step_handle_t, step_handle_t> ODGI::rewrite_segment(const step_handle_
 void ODGI::display(void) const {
     std::cerr << "------ graph state ------" << std::endl;
 
-    std::cerr << "_max_node_id = " << _max_node_id << std::endl;
-    std::cerr << "_min_node_id = " << _min_node_id << std::endl;
+    std::cerr << "_id_increment = " << _id_increment << std::endl;
+    std::cerr << "_max_node_rank = " << _max_node_rank << std::endl;
+    std::cerr << "_min_node_rank = " << _min_node_rank << std::endl;
 
     //std::cerr << "graph_id_map" << "\t";
     //for (auto& k : graph_id_map) std::cerr << k.first << "->" << k.second << " "; std::cerr << std::endl;
     std::cerr << "node_v" << "\t";
     for (uint64_t i = 0; i < node_v.size(); ++i) {
         auto& node = node_v.at(i);
-        nid_t node_id = i+1;
+        nid_t node_id = rank_to_id(i);
         std::cerr << node_id << ":" << node.sequence() << " ";
         const std::vector<uint64_t> node_edges = node.edges();
         for (uint64_t j = 0; j < node_edges.size(); ++j) {
@@ -1337,8 +1483,8 @@ void ODGI::display(void) const {
     for (uint64_t q = 0; q < path_metadata_v.size(); ++q) {
         auto& p = path_metadata_v.at(q);
         std::cerr << q << ":" << p.name << ":"
-                  << as_integers(p.first)[0] << "/" << as_integers(p.first)[1] << "->"
-                  << as_integers(p.last)[0] << "/" << as_integers(p.last)[1] << " ";
+                  << as_integers(p.first)[0] << "(" <<  get_id(as_handle(as_integers(p.first)[0])) << ")/" << as_integers(p.first)[1] << "->"
+                  << as_integers(p.last)[0] << "(" <<  get_id(as_handle(as_integers(p.last)[0])) << ")/" << as_integers(p.last)[1] << " ";
     } std::cerr << std::endl;
 
     // not dumped...
@@ -1395,13 +1541,14 @@ void ODGI::to_gfa(std::ostream& out) const {
         });
 }
 
-uint64_t ODGI::serialize_and_measure(std::ostream& out) const {
+long long int ODGI::serialize_and_measure(std::ostream& out) const {
     //rebuild_id_handle_mapping();
-    uint64_t written = 0;
-    out.write((char*)&_max_node_id,sizeof(_max_node_id));
-    written += sizeof(_max_node_id);
-    out.write((char*)&_min_node_id,sizeof(_min_node_id));
-    written += sizeof(_min_node_id);
+    // TODO: every write here needs an endianness-converting function!
+    long long int written = 0;
+    out.write((char*)&_max_node_rank,sizeof(_max_node_rank));
+    written += sizeof(_max_node_rank);
+    out.write((char*)&_min_node_rank,sizeof(_min_node_rank));
+    written += sizeof(_min_node_rank);
     out.write((char*)&_node_count,sizeof(_node_count));
     written += sizeof(_node_count);
     out.write((char*)&_edge_count,sizeof(_edge_count));
@@ -1414,6 +1561,13 @@ uint64_t ODGI::serialize_and_measure(std::ostream& out) const {
     written += sizeof(_deleted_node_count);
     out.write((char*)&_id_increment,sizeof(_id_increment));
     written += sizeof(_id_increment);
+    size_t hidden_count = graph_id_hidden_set.size();
+    out.write((char*)&hidden_count,sizeof(hidden_count));
+    written += sizeof(hidden_count);
+    for (const nid_t& hidden : graph_id_hidden_set) {
+        out.write((const char*)&hidden,sizeof(hidden));
+        written += sizeof(hidden);
+    }
     assert(_node_count == node_v.size());
     for (auto& node : node_v) {
         written += node.serialize(out);
@@ -1451,15 +1605,22 @@ uint64_t ODGI::serialize_and_measure(std::ostream& out) const {
 }
 
 void ODGI::load(std::istream& in) {
-    //uint64_t written = 0;
-    in.read((char*)&_max_node_id,sizeof(_max_node_id));
-    in.read((char*)&_min_node_id,sizeof(_min_node_id));
+    // TODO: every read here needs an endianness-converting function!
+    in.read((char*)&_max_node_rank,sizeof(_max_node_rank));
+    in.read((char*)&_min_node_rank,sizeof(_min_node_rank));
     in.read((char*)&_node_count,sizeof(_node_count));
     in.read((char*)&_edge_count,sizeof(_edge_count));
     in.read((char*)&_path_count,sizeof(_path_count));
     in.read((char*)&_path_handle_next,sizeof(_path_handle_next));
     in.read((char*)&_deleted_node_count,sizeof(_deleted_node_count));
     in.read((char*)&_id_increment,sizeof(_id_increment));
+    size_t hidden_count;
+    in.read((char*)&hidden_count,sizeof(hidden_count));
+    for (size_t i = 0; i < hidden_count; i++) {
+        nid_t hidden;
+        in.read((char*)&hidden,sizeof(hidden));
+        graph_id_hidden_set.insert(hidden);
+    }
     node_v.resize(_node_count);
     for (size_t i = 0; i < _node_count; ++i) {
         auto& node = node_v[i];
@@ -1492,6 +1653,14 @@ void ODGI::load(std::istream& in) {
         in.read((char*)&v,sizeof(uint64_t));
         path_name_map[string(k)] = v;
     }
+    
+    if (_id_increment == 0) {
+        // Make sure _id_increment isn't 0. Otherwise, the rank-0 unused slot will
+        // try to have ID 0 and we can't use it. This upsets the create_handle
+        // logic that tries to re-use the first empty slot.
+        set_id_increment(1);
+    }
+    
 }
 
 }
