@@ -17,28 +17,30 @@
 #include "utility.hpp"
 #include "statistics.hpp"
 #include "banded_global_aligner.hpp"
-#include "xdrop_aligner.hpp"
+#include "dozeu_interface.hpp"
 #include "handle.hpp"
 #include "reverse_graph.hpp"
 #include "null_masking_graph.hpp"
+#include "dozeu_pinning_overlay.hpp"
 #include "algorithms/distance_to_tail.hpp"
+
 // #define BENCH
 // #include "bench.h"
 
 namespace vg {
 
-    static const int8_t default_match = 1;
-    static const int8_t default_mismatch = 4;
-    static const int8_t default_gap_open = 6;
-    static const int8_t default_gap_extension = 1;
-    static const int8_t default_full_length_bonus = 5;
-    static const int8_t default_max_scaled_score = 32;
-    static const uint8_t default_max_qual_score = 255;
-    static const double default_gc_content = 0.5;
-    static const uint32_t default_xdrop_max_gap_length = 40;
-
-    
-    class VG; // forward declaration
+    static constexpr int8_t default_match = 1;
+    static constexpr int8_t default_mismatch = 4;
+    static constexpr int8_t default_score_matrix[16] = {
+         default_match,    -default_mismatch, -default_mismatch, -default_mismatch,
+        -default_mismatch,  default_match,    -default_mismatch, -default_mismatch,
+        -default_mismatch, -default_mismatch,  default_match,    -default_mismatch,
+        -default_mismatch, -default_mismatch, -default_mismatch,  default_match
+    };
+    static constexpr int8_t default_gap_open = 6;
+    static constexpr int8_t default_gap_extension = 1;
+    static constexpr int8_t default_full_length_bonus = 5;
+    static constexpr double default_gc_content = 0.5;
 
     /**
      * The abstract interface that any Aligner should implement.
@@ -48,27 +50,23 @@ namespace vg {
         
         /// Store optimal local alignment against a graph in the Alignment object.
         /// Gives the full length bonus separately on each end of the alignment.
-        virtual void align(Alignment& alignment, const HandleGraph& g, bool traceback_aln, bool print_score_matrices) const = 0;
-        
-        /// Same as previous, but takes advantage of a pre-computed topological order
-        virtual void align(Alignment& alignment, const HandleGraph& g, const vector<handle_t>& topological_order,
-                           bool traceback_aln, bool print_score_matrices) const = 0;
+        virtual void align(Alignment& alignment, const HandleGraph& g, bool traceback_aln) const = 0;
     };
 
     /**
      * The basic GSSW-based core aligner implementation, which can then be quality-adjusted or not.
      */
     class GSSWAligner : public BaseAligner {
+        
     protected:
+        
         GSSWAligner() = default;
         ~GSSWAligner();
         
         // for construction
         // needed when constructing an alignable graph from the nodes
         gssw_graph* create_gssw_graph(const HandleGraph& g) const;
-        // for constructing an alignable graph when the topological order is already known
-        gssw_graph* create_gssw_graph(const HandleGraph& g, const vector<handle_t>& topological_order) const;
-        
+
         // identify the IDs of nodes that should be used as pinning points in GSSW for pinned
         // alignment ((i.e. non-empty nodes as close as possible to sinks))
         unordered_set<id_t> identify_pinning_points(const HandleGraph& graph) const;
@@ -83,27 +81,31 @@ namespace vg {
                                        gssw_graph_mapping* gm,
                                        Alignment& alignment,
                                        bool pinned,
-                                       bool pin_left,
-                                       bool print_score_matrices = false) const;
+                                       bool pin_left) const;
         string graph_cigar(gssw_graph_mapping* gm) const;
         
     public:
         /// Given a nonempty vector of nonnegative scaled alignment scores,
         /// compute the mapping quality of the maximal score in the vector.
-        /// Sets max_idx_out to the index of that score in the vector. May
-        /// modify the input vector.
-        static double maximum_mapping_quality_exact(vector<double>& scaled_scores, size_t* max_idx_out); 
+        /// Sets max_idx_out to the index of that score in the vector.
+        /// Optionally includes a vector of implicit counts >= 1 for the scores, but
+        /// the mapping quality is always calculated as if its multiplicity is 1.
+        static double maximum_mapping_quality_exact(const vector<double>& scaled_scores, size_t* max_idx_out,
+                                                    const vector<double>* multiplicities = nullptr);
         /// Given a nonempty vector of nonnegative scaled alignment scores,
         /// approximate the mapping quality of the maximal score in the vector.
-        /// Sets max_idx_out to the index of that score in the vector. May
-        /// modify the input vector.
-        static double maximum_mapping_quality_approx(vector<double>& scaled_scores, size_t* max_idx_out);
+        /// Sets max_idx_out to the index of that score in the vector.
+        /// Optionally includes a vector of implicit counts >= 1 for the scores, but
+        /// the mapping quality is always calculated as if its multiplicity is 1.
+        static double maximum_mapping_quality_approx(const vector<double>& scaled_scores, size_t* max_idx_out,
+                                                     const vector<double>* multiplicities = nullptr);
     protected:
-        double group_mapping_quality_exact(vector<double>& scaled_scores, vector<size_t>& group) const;
+        double group_mapping_quality_exact(const vector<double>& scaled_scores, const vector<size_t>& group,
+                                           const vector<double>* multiplicities = nullptr) const;
         double estimate_next_best_score(int length, double min_diffs) const;
         
-        // must be called before querying mapping_quality
-        void init_mapping_quality(double gc_content);
+        // for calling in constructors so that mapping quality can be queried
+        void init_mapping_quality(const int8_t* score_matrix, double gc_content);
         
         // TODO: this algorithm has numerical problems, just removing it for now
         //vector<double> all_mapping_qualities_exact(vector<double> scaled_scores);
@@ -113,59 +115,47 @@ namespace vg {
         double max_possible_mapping_quality(int length) const;
         double estimate_max_possible_mapping_quality(int length, double min_diffs, double next_min_diffs) const;
         
-        // store optimal alignment against a graph in the Alignment object with one end of the sequence
-        // guaranteed to align to a source/sink node
-        //
-        // pinning left means that that the alignment starts with the first base of the read sequence and
-        // the first base of a source node sequence, pinning right means that the alignment starts with
-        // the final base of the read sequence and the final base of a sink node sequence
-        //
-        // Gives the full length bonus only on the non-pinned end of the alignment.
-        //
-        // assumes that graph is topologically sorted by node index
-        virtual void align_pinned(Alignment& alignment, const HandleGraph& g, bool pin_left) const = 0;
+        /// store optimal alignment against a graph in the Alignment object with one end of the sequence
+        /// guaranteed to align to a source/sink node. if xdrop is selected, use the xdrop heuristic, which
+        /// does not guarantee an optimal alignment.
+        ///
+        /// pinning left means that that the alignment starts with the first base of the read sequence and
+        /// the first base of a source node sequence, pinning right means that the alignment starts with
+        /// the final base of the read sequence and the final base of a sink node sequence
+        ///
+        /// Gives the full length bonus only on the non-pinned end of the alignment.
+        virtual void align_pinned(Alignment& alignment, const HandleGraph& g, bool pin_left, bool xdrop = false,
+                                  uint16_t xdrop_max_gap_length = default_xdrop_max_gap_length) const = 0;
         
-        /// Same as previous, but takes advantage of a pre-computed topological order.
-        /// The topological order MUST be left to right, no matter whether you are pinning left or right.
-        /// If alignment needs to proceed backward, it will be reversed internally.
-        virtual void align_pinned(Alignment& alignment, const HandleGraph& g,
-                                  const vector<handle_t>& topological_order, bool pin_left) const = 0;
-        
-        // store the top scoring pinned alignments in the vector in descending score order up to a maximum
-        // number of alignments (including the optimal one). if there are fewer than the maximum number in
-        // the return value, then it includes all alignments with a positive score. the optimal alignment
-        // will be stored in both the vector and in the main alignment object
-        //
-        // assumes that graph is topologically sorted by node index
+        /// store the top scoring pinned alignments in the vector in descending score order up to a maximum
+        /// number of alignments (including the optimal one). if there are fewer than the maximum number in
+        /// the return value, then it includes all alignments with a positive score. the optimal alignment
+        /// will be stored in both the vector and in the main alignment object
         virtual void align_pinned_multi(Alignment& alignment, vector<Alignment>& alt_alignments, const HandleGraph& g,
                                         bool pin_left, int32_t max_alt_alns) const = 0;
         
-        
-        /// Same as previous, but takes advantage of a pre-computed topological order.
-        /// The topological order MUST be left to right, no matter whether you are pinning left or right.
-        /// If alignment needs to proceed backward, it will be reversed internally.
-        virtual void align_pinned_multi(Alignment& alignment, vector<Alignment>& alt_alignments, const HandleGraph& g,
-                                        const vector<handle_t>& topological_order, bool pin_left, int32_t max_alt_alns) const = 0;
-        
-        // store optimal global alignment against a graph within a specified band in the Alignment object
-        // permissive banding auto detects the width of band needed so that paths can travel
-        // through every node in the graph
+        /// store optimal global alignment against a graph within a specified band in the Alignment object
+        /// permissive banding auto detects the width of band needed so that paths can travel
+        /// through every node in the graph
         virtual void align_global_banded(Alignment& alignment, const HandleGraph& g,
                                          int32_t band_padding = 0, bool permissive_banding = true) const = 0;
         
-        // store top scoring global alignments in the vector in descending score order up to a maximum number
-        // of alternate alignments (including the optimal alignment). if there are fewer than the maximum
-        // number of alignments in the return value, then the vector contains all possible alignments. the
-        // optimal alignment will be stored in both the vector and the original alignment object
+        /// store top scoring global alignments in the vector in descending score order up to a maximum number
+        /// of alternate alignments (including the optimal alignment). if there are fewer than the maximum
+        /// number of alignments in the return value, then the vector contains all possible alignments. the
+        /// optimal alignment will be stored in both the vector and the original alignment object
         virtual void align_global_banded_multi(Alignment& alignment, vector<Alignment>& alt_alignments,
                                                const HandleGraph& g, int32_t max_alt_alns, int32_t band_padding = 0,
                                                bool permissive_banding = true) const = 0;
-        // xdrop aligner
-        virtual void align_xdrop(Alignment& alignment, Graph& g, const vector<MaximalExactMatch>& mems, bool reverse_complemented) const = 0;
-        virtual void align_xdrop_multi(Alignment& alignment, Graph& g, const vector<MaximalExactMatch>& mems, bool reverse_complemented, int32_t max_alt_alns) const = 0;
-        /// Get a fresh XdropAligner instance to align with.
-        /// TODO: make XdropAligner thread safe, and make it a thing you can get from GetAligner.
-        virtual unique_ptr<XdropAligner> get_xdrop() const = 0;
+        /// xdrop aligner
+        virtual void align_xdrop(Alignment& alignment, const HandleGraph& g, const vector<MaximalExactMatch>& mems,
+                                 bool reverse_complemented, uint16_t max_gap_length = default_xdrop_max_gap_length) const = 0;
+        
+        /// xdrop aligner, but with a precomputed topological order on the graph, which need not include
+        /// all of the graph's handles and which may contain both orientations of a handle
+        virtual void align_xdrop(Alignment& alignment, const HandleGraph& g, const vector<handle_t>& order,
+                                 const vector<MaximalExactMatch>& mems, bool reverse_complemented,
+                                 uint16_t max_gap_length = default_xdrop_max_gap_length) const = 0;
 
         /// Compute the score of an exact match in the given alignment, from the
         /// given offset, of the given length.
@@ -215,11 +205,17 @@ namespace vg {
                                             double maybe_mq_threshold,
                                             double identity_weight) const;
         
-        /// Computes mapping quality for the optimal score in a vector of scores
-        int32_t compute_mapping_quality(vector<double>& scores, bool fast_approximation) const;
+        /// Computes mapping quality for the optimal score in a vector of scores.
+        /// Optionally includes a vector of implicit counts >= 1 for the scores, but
+        /// the mapping quality is always calculated as if it multiplicity is 1.
+        int32_t compute_mapping_quality(const vector<double>& scores, bool fast_approximation,
+                                        const vector<double>* multiplicities = nullptr) const;
         
-        /// Computes mapping quality for a group of scores in a vector of scores (group given by indexes)
-        int32_t compute_group_mapping_quality(vector<double>& scores, vector<size_t>& group) const;
+        /// Computes mapping quality for a group of scores in a vector of scores (group given by indexes).
+        /// Optionally includes a vector of implicit counts >= 1 for the score, but the mapping quality is always
+        /// calculated as if each member of the group has multiplicity is 1.
+        int32_t compute_group_mapping_quality(const vector<double>& scores, const vector<size_t>& group,
+                                              const vector<double>* multiplicities = nullptr) const;
         
         /// Returns the  difference between an optimal and second-best alignment scores that would
         /// result in this mapping quality using the fast mapping quality approximation
@@ -254,10 +250,6 @@ namespace vg {
         virtual int32_t score_ungapped_alignment(const Alignment& aln,
                                                  bool strip_bonuses = false) const;
 
-        /// Reads a 5x5 substitution scoring matrix from an input stream (can be an ifstream)
-        /// expecting 5 whitespace-separated 8-bit integers per line
-        virtual void load_scoring_matrix(std::istream& matrix_stream);
-
         /// Without necessarily rescoring the entire alignment, return the score
         /// of the given alignment with bonuses removed. Assumes that bonuses
         /// are actually included in the score.
@@ -275,76 +267,51 @@ namespace vg {
         
         // log of the base of the logarithm underlying the log-odds interpretation of the scores
         double log_base = 0.0;
-        
     };
     
     /**
      * An ordinary aligner.
      */
     class Aligner : public GSSWAligner {
-    private:
-        
-        // internal function interacting with gssw for pinned and local alignment
-        void align_internal(Alignment& alignment, vector<Alignment>* multi_alignments, const HandleGraph& g,
-                            const vector<handle_t>* topological_order,
-                            bool pinned, bool pin_left, int32_t max_alt_alns,
-                            bool traceback_aln,
-                            bool print_score_matrices) const;
 
-        // members
-        XdropAligner xdrop;
-        // bench_t bench;
     public:
-        Aligner(int8_t _match = default_match,
-                int8_t _mismatch = default_mismatch,
+        
+        Aligner(const int8_t* _score_matrix = default_score_matrix,
                 int8_t _gap_open = default_gap_open,
                 int8_t _gap_extension = default_gap_extension,
                 int8_t _full_length_bonus = default_full_length_bonus,
                 double _gc_content = default_gc_content);
-        // xdrop_aligner wrapper, omit default values to call the one above for Aligner();
-        Aligner(int8_t _match,
-                int8_t _mismatch,
-                int8_t _gap_open,
-                int8_t _gap_extension,
-                int8_t _full_length_bonus,
-                double _gc_content,
-                uint32_t _xdrop_max_gap_length);
         ~Aligner(void) = default;
         
         /// Store optimal local alignment against a graph in the Alignment object.
         /// Gives the full length bonus separately on each end of the alignment.
-        /// Assumes that graph is topologically sorted by node index.
-        void align(Alignment& alignment, const HandleGraph& g, bool traceback_aln, bool print_score_matrices) const;
-        void align(Alignment& alignment, const HandleGraph& g, const vector<handle_t>& topological_order,
-                   bool traceback_aln, bool print_score_matrices) const;
-        
+        void align(Alignment& alignment, const HandleGraph& g, bool traceback_aln) const;
+
+        /// Align against a subgraph induced by a subset of nodes. The topological
+        /// order of the handles in the subgraph must be provided.
+        /// Store optimal local alignment in the Alignment object.
+        /// Gives the full length bonus separately on each end of the alignment.
+        void align(Alignment& alignment, const HandleGraph& g,
+                   const std::vector<handle_t>& topological_order) const;
+
         /// store optimal alignment against a graph in the Alignment object with one end of the sequence
-        /// guaranteed to align to a source/sink node
+        /// guaranteed to align to a source/sink node. if xdrop is selected, use the xdrop heuristic, which
+        /// does not guarantee an optimal alignment.
         ///
         /// pinning left means that that the alignment starts with the first base of the read sequence and
         /// the first base of a source node sequence, pinning right means that the alignment starts with
         /// the final base of the read sequence and the final base of a sink node sequence
         ///
         /// Gives the full length bonus only on the non-pinned end of the alignment.
-        ///
-        void align_pinned(Alignment& alignment, const HandleGraph& g, bool pin_left) const;
-        /// The topological order MUST be left to right, no matter whether you are pinning left or right.
-        /// If alignment needs to proceed backward, it will be reversed internally.
-        void align_pinned(Alignment& alignment, const HandleGraph& g, const vector<handle_t>& topological_order,
-                          bool pin_left) const;
+        void align_pinned(Alignment& alignment, const HandleGraph& g, bool pin_left, bool xdrop = false,
+                          uint16_t xdrop_max_gap_length = default_xdrop_max_gap_length) const;
                 
         /// store the top scoring pinned alignments in the vector in descending score order up to a maximum
         /// number of alignments (including the optimal one). if there are fewer than the maximum number in
         /// the return value, then it includes all alignments with a positive score. the optimal alignment
         /// will be stored in both the vector and in the main alignment object
-        ///
-        /// assumes that graph is topologically sorted by node index
         void align_pinned_multi(Alignment& alignment, vector<Alignment>& alt_alignments, const HandleGraph& g,
                                 bool pin_left, int32_t max_alt_alns) const;
-        /// The topological order MUST be left to right, no matter whether you are pinning left or right.
-        /// If alignment needs to proceed backward, it will be reversed internally.
-        void align_pinned_multi(Alignment& alignment, vector<Alignment>& alt_alignments, const HandleGraph& g,
-                                const vector<handle_t>& topological_order, bool pin_left, int32_t max_alt_alns) const;
         
         /// store optimal global alignment against a graph within a specified band in the Alignment object
         /// permissive banding auto detects the width of band needed so that paths can travel
@@ -359,10 +326,15 @@ namespace vg {
         void align_global_banded_multi(Alignment& alignment, vector<Alignment>& alt_alignments, const HandleGraph& g,
                                        int32_t max_alt_alns, int32_t band_padding = 0, bool permissive_banding = true) const;
 
-        // xdrop aligner
-        void align_xdrop(Alignment& alignment, Graph& g, const vector<MaximalExactMatch>& mems, bool reverse_complemented) const;
-        void align_xdrop_multi(Alignment& alignment, Graph& g, const vector<MaximalExactMatch>& mems, bool reverse_complemented, int32_t max_alt_alns) const;
-        unique_ptr<XdropAligner> get_xdrop() const;
+        /// xdrop aligner
+        void align_xdrop(Alignment& alignment, const HandleGraph& g, const vector<MaximalExactMatch>& mems,
+                         bool reverse_complemented, uint16_t max_gap_length = default_xdrop_max_gap_length) const;
+        
+        /// xdrop aligner, but with a precomputed topological order on the graph, which need not include
+        /// all of the graph's handles and which may contain both orientations of a handle
+        void align_xdrop(Alignment& alignment, const HandleGraph& g, const vector<handle_t>& order,
+                         const vector<MaximalExactMatch>& mems, bool reverse_complemented,
+                         uint16_t max_gap_length = default_xdrop_max_gap_length) const;
 
         int32_t score_exact_match(const Alignment& aln, size_t read_offset, size_t length) const;
         int32_t score_exact_match(const string& sequence, const string& base_quality) const;
@@ -376,6 +348,16 @@ namespace vg {
 
         int32_t score_partial_alignment(const Alignment& alignment, const HandleGraph& graph, const Path& path,
                                         string::const_iterator seq_begin) const;
+        
+    private:
+        
+        // internal function interacting with gssw for pinned and local alignment
+        void align_internal(Alignment& alignment, vector<Alignment>* multi_alignments, const HandleGraph& g,
+                            bool pinned, bool pin_left, int32_t max_alt_alns,
+                            bool traceback_aln) const;
+        
+        // members
+        vector<XdropAligner> xdrops;
     };
 
     /**
@@ -384,44 +366,31 @@ namespace vg {
     class QualAdjAligner : public GSSWAligner {
     public:
         
-        QualAdjAligner(int8_t _match = default_match,
-                       int8_t _mismatch = default_mismatch,
+        QualAdjAligner(const int8_t* _score_matrix = default_score_matrix,
                        int8_t _gap_open = default_gap_open,
                        int8_t _gap_extension = default_gap_extension,
                        int8_t _full_length_bonus = default_full_length_bonus,
-                       int8_t _max_scaled_score = default_max_scaled_score,
-                       uint8_t _max_qual_score = default_max_qual_score,
-                       double gc_content = default_gc_content);
+                       double _gc_content = default_gc_content);
 
         ~QualAdjAligner(void) = default;
-
+        
         // base quality adjusted counterparts to functions of same name from Aligner
-        void align(Alignment& alignment, const HandleGraph& g, bool traceback_aln, bool print_score_matrices) const;
-        void align(Alignment& alignment, const HandleGraph& g, const vector<handle_t>& topological_order,
-                   bool traceback_aln, bool print_score_matrices) const;
+        
+        void align(Alignment& alignment, const HandleGraph& g, bool traceback_aln) const;
         void align_global_banded(Alignment& alignment, const HandleGraph& g,
                                  int32_t band_padding = 0, bool permissive_banding = true) const;
-        void align_pinned(Alignment& alignment, const HandleGraph& g, bool pin_left) const;
-        /// The topological order MUST be left to right, no matter whether you are pinning left or right.
-        /// If alignment needs to proceed backward, it will be reversed internally.
-        void align_pinned(Alignment& alignment, const HandleGraph& g, const vector<handle_t>& topological_order,
-                          bool pin_left) const;
+        void align_pinned(Alignment& alignment, const HandleGraph& g, bool pin_left, bool xdrop = false,
+                          uint16_t xdrop_max_gap_length = default_xdrop_max_gap_length) const;
         void align_global_banded_multi(Alignment& alignment, vector<Alignment>& alt_alignments, const HandleGraph& g,
                                        int32_t max_alt_alns, int32_t band_padding = 0, bool permissive_banding = true) const;
         void align_pinned_multi(Alignment& alignment, vector<Alignment>& alt_alignments, const HandleGraph& g,
                                 bool pin_left, int32_t max_alt_alns) const;
-        /// The topological order MUST be left to right, no matter whether you are pinning left or right.
-        /// If alignment needs to proceed backward, it will be reversed internally.
-        void align_pinned_multi(Alignment& alignment, vector<Alignment>& alt_alignments, const HandleGraph& g,
-                                const vector<handle_t>& topological_order, bool pin_left, int32_t max_alt_alns) const;
                                 
-        // TODO: xdrop isn't actually possible with the quality adjusted aligner (yet).
-        // xdrop aligner
-        void align_xdrop(Alignment& alignment, Graph& g, const vector<MaximalExactMatch>& mems, bool reverse_complemented) const;
-        void align_xdrop_multi(Alignment& alignment, Graph& g, const vector<MaximalExactMatch>& mems, bool reverse_complemented, int32_t max_alt_alns) const;
-        unique_ptr<XdropAligner> get_xdrop() const;
-
-        void init_mapping_quality(double gc_content);
+        void align_xdrop(Alignment& alignment, const HandleGraph& g, const vector<MaximalExactMatch>& mems,
+                         bool reverse_complemented, uint16_t max_gap_length = default_xdrop_max_gap_length) const;
+        void align_xdrop(Alignment& alignment, const HandleGraph& g, const vector<handle_t>& order,
+                         const vector<MaximalExactMatch>& mems, bool reverse_complemented,
+                         uint16_t max_gap_length = default_xdrop_max_gap_length) const;
         
         int32_t score_exact_match(const Alignment& aln, size_t read_offset, size_t length) const;
         int32_t score_exact_match(const string& sequence, const string& base_quality) const;
@@ -431,18 +400,17 @@ namespace vg {
         int32_t score_partial_alignment(const Alignment& alignment, const HandleGraph& graph, const Path& path,
                                         string::const_iterator seq_begin) const;
         
-        uint8_t max_qual_score;
-        int8_t scale_factor;
         
     private:
 
+        // internal function interacting with gssw for pinned and local alignment
         void align_internal(Alignment& alignment, vector<Alignment>* multi_alignments, const HandleGraph& g,
-                            const vector<handle_t>* topological_order,
                             bool pinned, bool pin_left, int32_t max_alt_alns,
-                            bool traceback_aln,
-                            bool print_score_matrices) const;
+                            bool traceback_aln) const;
         
 
+        // members
+        vector<QualAdjXdropAligner> xdrops;
     };
     
     
@@ -470,16 +438,26 @@ namespace vg {
         const Aligner* get_regular_aligner() const;
         
     public:
+        
         /// Set all the aligner scoring parameters and create the stored aligner instances.
-        void set_alignment_scores(int8_t match, int8_t mismatch, int8_t gap_open, int8_t gap_extend, int8_t full_length_bonus,
-                                  uint32_t xdrop_max_gap_length = default_xdrop_max_gap_length);
-                                  
-        /// Load a scoring amtrix from a file to set scores
-        void load_scoring_matrix(std::ifstream& matrix_stream);
+        void set_alignment_scores(int8_t match, int8_t mismatch, int8_t gap_open, int8_t gap_extend, int8_t full_length_bonus);
+        
+        /// Set the algner scoring parameters and create the stored aligner instances. The
+        /// stream should contain a 4 x 4 whitespace-separated substitution matrix (in the
+        /// order ACGT)
+        void set_alignment_scores(std::istream& matrix_stream, int8_t gap_open, int8_t gap_extend, int8_t full_length_bonus);
+        
+        /// Set the algner scoring parameters and create the stored aligner instances. The
+        /// score matrix should by a 4 x 4 array in the order (ACGT)
+        void set_alignment_scores(const int8_t* score_matrix, int8_t gap_open, int8_t gap_extend, int8_t full_length_bonus);
+        
+        /// Allocates an array to hold a 4x4 substitution matrix and returns it
+        static int8_t* parse_matrix(std::istream& matrix_stream);
         
         bool adjust_alignments_for_base_quality = false; // use base quality adjusted alignments
 
     private:
+        
         // GSSW aligners
         unique_ptr<QualAdjAligner> qual_adj_aligner;
         unique_ptr<Aligner> regular_aligner;
