@@ -12,6 +12,8 @@
 
 #include <sstream>
 
+//#define debug
+
 namespace vg {
 using namespace std;
 
@@ -38,16 +40,20 @@ void AlignmentEmitter::emit_mapped_pair(vector<Alignment>&& alns1, vector<Alignm
 }
 
 unique_ptr<AlignmentEmitter> get_alignment_emitter(const string& filename, const string& format,
-    const map<string, int64_t>& path_length, size_t max_threads, const PathPositionHandleGraph* splicing_graph) {
+    const map<string, int64_t>& path_length, size_t max_threads, const HandleGraph* graph) {
 
     // Make the backing, non-buffered emitter
     AlignmentEmitter* backing = nullptr;
     if (format == "GAM" || format == "JSON") {
         // Make an emitter that supports VG formats
         backing = new VGAlignmentEmitter(filename, format, max_threads);
+    } else if (format == "GAF") {
+        backing = new GafAlignmentEmitter(filename, format, *graph, max_threads);
     } else if (format == "SAM" || format == "BAM" || format == "CRAM") {
         // Make an emitter that supports HTSlib formats
-        if (splicing_graph) {
+        if (graph) {
+            const PathPositionHandleGraph* splicing_graph = dynamic_cast<const PathPositionHandleGraph*>(graph);
+            assert(splicing_graph != nullptr);
             // Use the graph to look for spliced alignments
             backing = new SplicedHTSAlignmentEmitter(filename, format, path_length, *splicing_graph, max_threads);
         }
@@ -685,7 +691,7 @@ vector<pair<int, char>> SplicedHTSAlignmentEmitter::spliced_cigar_against_path(c
                 handle_t next_handle = graph.get_handle_of_step(next_step);
                 const Position& next_pos = path.mapping(rev ? path.mapping_size() - 2 - i : i + 1).position();
                 if (graph.get_id(next_handle) != next_pos.node_id()
-                    || (graph.get_is_reverse(next_handle) == next_pos.is_reverse()) != rev) {
+                    || (graph.get_is_reverse(next_handle) != next_pos.is_reverse()) != rev) {
                     
                     // the next mapping in the alignment is not the next mapping on the path, so we must have
                     // taken a deletion
@@ -989,4 +995,114 @@ void VGAlignmentEmitter::emit_mapped_pairs(vector<vector<Alignment>>&& alns1_bat
     }
 }
 
+GafAlignmentEmitter::GafAlignmentEmitter(const string& filename,
+                                         const string& format,
+                                         const HandleGraph& _graph,
+                                         size_t max_threads):
+    out_file(filename == "-" ? nullptr : new ofstream(filename)),
+    multiplexer(out_file.get() != nullptr ? *out_file : cout, max_threads),
+    graph(_graph) {
+    
+    // We only support GAF format
+    assert(format == "GAF");
+    
+#ifdef debug
+    cerr << "Creating GafAlignmentEmitter for " << format << " format to output file " << filename << " @ " << out_file.get() << endl;
+    if (out_file.get() != nullptr) {
+        cerr << "Output stream is at " << out_file->tellp() << endl;
+    } else {
+        cerr << "Output stream is at " << cout.tellp() << endl;
+    }
+#endif
+    
+    if (filename != "-") {
+        // Check the file
+        if (!*out_file) {
+            // We couldn't get it open
+            cerr << "[vg::GafAlignmentEmitter] failed to open " << filename << " for writing " << format << " output" << endl;
+            exit(1);
+        }
+    }
+    
+    // We later infer our format and output destination from out_file and proto being empty/set.
+}
+
+GafAlignmentEmitter::~GafAlignmentEmitter() {
+#ifdef debug
+    cerr << "Destroying GafAlignmentEmitter" << endl;
+    if (out_file.get() != nullptr) {
+        cerr << "Output stream is at " << out_file->tellp() << endl;
+    } else {
+        cerr << "Output stream is at " << cout.tellp() << endl;
+    }
+#endif
+
+#ifdef debug
+    cerr << "Destroyed GafAlignmentEmitter" << endl;
+#endif
+}
+
+void GafAlignmentEmitter::emit_singles(vector<Alignment>&& aln_batch) {
+    size_t thread_number = omp_get_thread_num();
+    // Serialize to a string in our thread
+    for (auto& aln : aln_batch) {
+        multiplexer.get_thread_stream(thread_number) << alignment_to_gaf(graph, aln) << endl;
+    }
+    // No need to flush, we can always register a breakpoint.
+    multiplexer.register_breakpoint(thread_number);
+}
+
+void GafAlignmentEmitter::emit_mapped_singles(vector<vector<Alignment>>&& alns_batch) {
+    size_t thread_number = omp_get_thread_num();
+    // Serialize to a string in our thread
+    for (auto& alns : alns_batch) {
+        for (auto& aln : alns) {
+            multiplexer.get_thread_stream(thread_number) << alignment_to_gaf(graph, aln) << endl;
+        }
+    }
+    // No need to flush, we can always register a breakpoint.
+    multiplexer.register_breakpoint(thread_number);
+#ifdef debug
+    cerr << "Sent " << alns_batch.size() << " batches from thread " << thread_number << " followed by a breakpoint" << endl;
+#endif
+}
+
+void GafAlignmentEmitter::emit_pairs(vector<Alignment>&& aln1_batch,
+                                    vector<Alignment>&& aln2_batch,
+                                    vector<int64_t>&& tlen_limit_batch) {
+    // Sizes need to match up
+    assert(aln1_batch.size() == aln2_batch.size());
+    assert(aln1_batch.size() == tlen_limit_batch.size());
+    
+    size_t thread_number = omp_get_thread_num();
+    
+    // Serialize to a string in our thread in collated order
+    for (size_t i = 0; i < aln1_batch.size(); i++) {
+        multiplexer.get_thread_stream(thread_number) << alignment_to_gaf(graph, aln1_batch[i]) << endl
+                                                     << alignment_to_gaf(graph, aln2_batch[i]) << endl;
+    }
+    // No need to flush, we can always register a breakpoint.
+    multiplexer.register_breakpoint(thread_number);
+}
+
+void GafAlignmentEmitter::emit_mapped_pairs(vector<vector<Alignment>>&& alns1_batch,
+                                           vector<vector<Alignment>>&& alns2_batch,
+                                           vector<int64_t>&& tlen_limit_batch) {
+    // Sizes need to match up
+    assert(alns1_batch.size() == alns2_batch.size());
+    assert(alns1_batch.size() == tlen_limit_batch.size());
+    
+    size_t thread_number = omp_get_thread_num();
+    // Serialize to an interleaved string in our thread
+    for (size_t i = 0; i < alns1_batch.size(); i++) {
+        assert(alns1_batch[i].size() == alns1_batch[i].size());
+        for (size_t j = 0; j < alns1_batch[i].size(); j++) {
+            multiplexer.get_thread_stream(thread_number) << alignment_to_gaf(graph, alns1_batch[i][j]) << endl
+                                                         << alignment_to_gaf(graph, alns2_batch[i][j]) << endl;
+        }
+    }
+    // No need to flush, we can always register a breakpoint.
+    multiplexer.register_breakpoint(thread_number);
+}
+
 }
