@@ -16,6 +16,7 @@
 #include "register_loader_saver_xg.hpp"
 #include "register_loader_saver_packed_graph.hpp"
 #include "register_loader_saver_hash_graph.hpp"
+#include "register_loader_saver_odgi.hpp"
 
 #include "register_libvg_io.hpp"
 
@@ -37,7 +38,8 @@ bool register_libvg_io() {
     register_loader_saver_vg();
     register_loader_saver_xg();
     register_loader_saver_packed_graph();
-    register_loader_saver_hash_graph();    
+    register_loader_saver_hash_graph();
+    register_loader_saver_odgi(); 
     return true;
 }
     
