@@ -17,12 +17,20 @@ using namespace std;
 using namespace vg::io;
 
 void register_loader_saver_odgi() {
-    Registry::register_bare_loader_saver<bdsg::ODGI, MutablePathDeletableHandleGraph, MutablePathMutableHandleGraph, MutableHandleGraph, PathHandleGraph, HandleGraph>("ODGI", [](istream& input) -> void* {
+
+    // Convert the ODGI SerializableHandleGraph magic number to a string
+    bdsg::ODGI empty;
+    // Make sure it is in network byte order
+    uint32_t new_magic_number = htonl(empty.get_magic_number());
+    // Load all 4 characters of it into a string
+    string new_magic((char*)&new_magic_number, 4);
+
+    Registry::register_bare_loader_saver_with_magic<bdsg::ODGI, MutablePathDeletableHandleGraph, MutablePathMutableHandleGraph, MutableHandleGraph, PathHandleGraph, HandleGraph>("ODGI", new_magic, [](istream& input) -> void* {
         // Allocate an ODGI graph
         bdsg::ODGI* odgi = new bdsg::ODGI();
         
         // Load it
-        odgi->load(input);
+        odgi->deserialize(input);
         
         // Return it so the caller owns it.
         return (void*) odgi;
