@@ -22,7 +22,6 @@
 #include "mem.hpp"
 #include "cluster.hpp"
 #include "graph.hpp"
-#include "proto_handle_graph.hpp"
 #include "translator.hpp"
 // TODO: pull out ScoreProvider into its own file
 #include "haplotypes.hpp"
@@ -209,7 +208,11 @@ public:
     
     /// Override alignment score setting to support haplotype consistency exponent
     void set_alignment_scores(int8_t match, int8_t mismatch, int8_t gap_open, int8_t gap_extend, int8_t full_length_bonus,
-                              uint32_t xdrop_max_gap_length = default_xdrop_max_gap_length, double haplotype_consistency_exponent = 1);
+                              double haplotype_consistency_exponent = 1);
+    
+    /// Same, but loading a 4x4 substitution score matrix from a stream
+    void set_alignment_scores(istream& matrix_stream, int8_t gap_open, int8_t gap_extend, int8_t full_length_bonus,
+                              double haplotype_consistency_exponent = 1);
     
     void set_cache_size(int new_cache_size);
     
@@ -218,8 +221,7 @@ public:
     // These are SMEMs by definition when shorter than the max_mem_length or GCSA2 order.
     // Designating reseed_length returns minimally-more-frequent sub-MEMs in addition to SMEMs when SMEM is >= reseed_length.
     // Minimally-more-frequent sub-MEMs are MEMs contained in an SMEM that have occurrences outside of the SMEM.
-    // SMEMs and sub-MEMs will be automatically filled with the nodes they contain, which the occurrences of the sub-MEMs
-    // that are inside SMEM hits filtered out. (filling sub-MEMs currently requires an PathPositionHandleGraph index)
+    // SMEMs and sub-MEMs will be automatically filled with the nodes they contain
     
     vector<MaximalExactMatch>
     find_mems_deep(string::const_iterator seq_begin,
@@ -243,6 +245,25 @@ public:
                      int min_mem_length = 1,
                      int reseed_length = 0);
     
+    vector<MaximalExactMatch>
+    find_stripped_matches(string::const_iterator seq_begin,
+                          string::const_iterator seq_end,
+                          size_t strip_length,
+                          size_t max_match_length,
+                          size_t target_count);
+    
+    vector<MaximalExactMatch>
+    find_fanout_mems(string::const_iterator seq_begin,
+                     string::const_iterator seq_end,
+                     string::const_iterator qual_begin,
+                     int max_fans_out,
+                     char max_fanout_base_quality);
+    
+    vector<pos_t> walk_fanout_path(string::const_iterator begin,
+                                   string::const_iterator end,
+                                   const list<pair<string::const_iterator, char>>& fanout_breaks,
+                                   gcsa::node_type pos);
+    
     /// identifies tracts of order-length MEMs that were unfilled because their hit count was above the max
     /// and fills one MEM in the tract (the one with the smallest hit count), assumes MEMs are lexicographically
     /// ordered by read index
@@ -267,10 +288,16 @@ public:
     double fast_reseed_length_diff = 0.45; // how much smaller than its parent a sub-MEM can be in the fast reseed algorithm
     bool adaptive_reseed_diff = true; // use an adaptive length difference algorithm in reseed algorithm
     double adaptive_diff_exponent = 0.065; // exponent that describes limiting behavior of adaptive diff algorithm
-    int hit_max = 0;       // ignore or MEMs with more than this many hits
+    int hit_max = 0;       // only query at most this many hits for a MEM (0 for no limit)
+    int hard_hit_max = 0; // don't query any hits for MEMs with this many occurrences or more (0 for no limit)
     bool use_approx_sub_mem_count = false;
     bool prefilter_redundant_hits = true;
     int max_sub_mem_recursion_depth = 2;
+    bool use_greedy_mem_restarts = false;
+    int greedy_restart_min_length = 40;
+    int greedy_restart_max_count = 2;
+    int greedy_restart_max_lcp = 0; // 0 for no max
+    bool greedy_restart_assume_substitution = false;
     int unpaired_penalty = 17;
     bool precollapse_order_length_hits = true;
     double avg_node_length = 0;
@@ -619,6 +646,7 @@ public:
 
     bool always_rescue; // Should rescue be attempted for all imperfect alignments?
     bool include_full_length_bonuses;
+    int max_xdrop_gap_length;
     
     bool simultaneous_pair_alignment;
     int max_band_jump; // the maximum length edit we can detect via banded alignment
