@@ -25,10 +25,10 @@ size_t Packer::estimate_bin_count(size_t num_threads) {
     return pow(2, log2(num_threads) + 14);
 }
 
-Packer::Packer(void) : graph(nullptr), data_width(8), cov_bin_size(0), edge_cov_bin_size(0), num_bases_dynamic(0), base_locks(nullptr), num_edges_dynamic(0), edge_locks(nullptr), tmpfstream_locks(nullptr) { }
+Packer::Packer(void) : graph(nullptr), data_width(8), cov_bin_size(0), edge_cov_bin_size(0), num_bases_dynamic(0), base_locks(nullptr), num_edges_dynamic(0), edge_locks(nullptr), node_quality_locks(nullptr), tmpfstream_locks(nullptr) { }
 
-Packer::Packer(const HandleGraph* graph, size_t bin_size, size_t coverage_bins, size_t data_width, bool record_bases, bool record_edges, bool record_edits) :
-    graph(graph), data_width(data_width), bin_size(bin_size), record_bases(record_bases), record_edges(record_edges), record_edits(record_edits) {
+Packer::Packer(const HandleGraph* graph, size_t bin_size, size_t coverage_bins, size_t data_width, bool record_bases, bool record_edges, bool record_edits, bool record_qualities) :
+    graph(graph), data_width(data_width), bin_size(bin_size), record_bases(record_bases), record_edges(record_edges), record_edits(record_edits), record_qualities(record_qualities) {
     // get the size of the base coverage counter
     num_bases_dynamic = 0;
     if (record_bases) {
@@ -48,26 +48,40 @@ Packer::Packer(const HandleGraph* graph, size_t bin_size, size_t coverage_bins,
             });
         ++num_edges_dynamic; // add one so our size is greater than the max element
     }
-
+    // get the size of the node qualitiy counter
+    num_nodes_dynamic = 0;
+    size_t qual_coverage_bins = 0;
+    if (record_qualities) {
+        num_nodes_dynamic = graph->get_node_count() + 1; // add one for 1-based ranks
+        if (num_bases_dynamic > 0) {
+            // scale down the quality bins to be proportional in size with the base coverage bins if we can
+            qual_coverage_bins = coverage_bins * ((double)num_nodes_dynamic / (double)num_bases_dynamic);
+            qual_coverage_bins = max((size_t)1, qual_coverage_bins);
+        }
+    }
+    
     // only bin if we need to
     if (num_edges_dynamic <= coverage_bins || num_bases_dynamic <= coverage_bins) {
         coverage_bins = 1;
+        qual_coverage_bins = 1;
     }
     assert(coverage_bins > 0);
-    coverage_dynamic.reserve(coverage_bins);
-    edge_coverage_dynamic.reserve(coverage_bins);
 
     // coverage counter for each bin (totally independent from the edit coverage bins)
     // they are initialized on-demand to better support sparse use-cases
     coverage_dynamic.resize(coverage_bins, nullptr);
     edge_coverage_dynamic.resize(coverage_bins, nullptr);
+    node_quality_dynamic.resize(qual_coverage_bins, nullptr);
+    
     // need this for every lookup, so store here
     cov_bin_size = coverage_dynamic.size() > 0 ? num_bases_dynamic / coverage_dynamic.size() : 0;
     edge_cov_bin_size = edge_coverage_dynamic.size() > 0 ? num_edges_dynamic / edge_coverage_dynamic.size() : 0;
+    node_qual_bin_size = node_quality_dynamic.size() > 0 ? num_nodes_dynamic / node_quality_dynamic.size() : 0;
 
     // mutexes for coverage
     base_locks = new std::mutex[coverage_dynamic.size()];
     edge_locks = new std::mutex[edge_coverage_dynamic.size()];
+    node_quality_locks = new std::mutex[node_quality_dynamic.size()];
     tmpfstream_locks = nullptr;
     
     // count the bins if binning
@@ -99,10 +113,16 @@ void Packer::clear() {
         delete counter;
         counter = nullptr;
     }
+    for (auto& counter : node_quality_dynamic) {
+        delete counter;
+        counter = nullptr;
+    }
     delete [] base_locks;
     base_locks = nullptr;
     delete [] edge_locks;
     edge_locks = nullptr;
+    delete [] node_quality_locks;
+    node_quality_locks = nullptr;
     delete [] tmpfstream_locks;
     tmpfstream_locks = nullptr;
     close_edit_tmpfiles();
@@ -141,6 +161,7 @@ void Packer::load(istream& in) {
     for (size_t i = 0; i < n_bins; ++i) {
         edit_csas[i].load(in);
     }
+    node_quality_civ.load(in);
     // We can only load compacted.
     is_compacted = true;
 }
@@ -229,9 +250,9 @@ void Packer::write_edits(ostream& out, size_t bin) const {
 void Packer::collect_coverage(const vector<Packer*>& packers) {
     // assume the same basis vector
     assert(!is_compacted);
+    if (record_bases) {
 #pragma omp parallel for
-    for (size_t i = 0; i < coverage_dynamic.size(); ++i) {
-        if (record_bases) {
+        for (size_t i = 0; i < coverage_dynamic.size(); ++i) {
             size_t base_offset = i * cov_bin_size;
             size_t bin_size = coverage_bin_size(i);
             for (size_t j = 0; j < bin_size; ++j) {
@@ -242,7 +263,10 @@ void Packer::collect_coverage(const vector<Packer*>& packers) {
                 increment_coverage(j + base_offset, inc_cov);
             }
         }
-        if (record_edges) {
+    }
+    if (record_edges) {
+#pragma omp parallel for
+        for (size_t i = 0; i < edge_coverage_dynamic.size(); ++i) {
             size_t edge_base_offset = i * edge_cov_bin_size;
             size_t edge_bin_size = edge_coverage_bin_size(i);
             for (size_t j = 0; j < edge_bin_size; ++j) {
@@ -254,6 +278,20 @@ void Packer::collect_coverage(const vector<Packer*>& packers) {
             }
         }
     }
+    if (record_qualities) {
+#pragma omp parallel for
+        for (size_t i = 1; i < node_quality_dynamic.size(); ++i) {
+            size_t qual_base_offset = i * node_qual_bin_size;
+            size_t qual_bin_size = node_quality_bin_size(i);
+            for (size_t j = 0; j < qual_bin_size; ++j) {
+                size_t inc_qual_cov = 0;
+                for (size_t k = 0; k < packers.size(); ++k) {
+                    inc_qual_cov += packers[k]->total_node_quality(j + qual_base_offset);
+                }
+                increment_node_quality(j + qual_base_offset, inc_qual_cov);
+            }
+        }            
+    }
 }
 
 size_t Packer::serialize(std::ostream& out,
@@ -265,10 +303,11 @@ size_t Packer::serialize(std::ostream& out,
     written += sdsl::write_member(bin_size, out, child, "bin_size_" + name);
     written += sdsl::write_member(edit_csas.size(), out, child, "n_bins_" + name);
     written += coverage_civ.serialize(out, child, "graph_coverage_" + name);
-    written += edge_coverage_civ.serialize(out, child, "edge_coverate_" +name);
+    written += edge_coverage_civ.serialize(out, child, "edge_coverage_" +name);
     for (auto& edit_csa : edit_csas) {
         written += edit_csa.serialize(out, child, "edit_csa_" + name);
     }
+    written += node_quality_civ.serialize(out, child, "node_quality_" + name);
     sdsl::structure_tree::add_size(child, written);
     return written;
 }
@@ -293,6 +332,8 @@ void Packer::make_compact(void) {
     int_vector<> coverage_iv;
     size_t edge_coverage_length = edge_vector_size();
     int_vector<> edge_coverage_iv;
+    size_t node_quality_length = node_quality_vector_size();
+    int_vector<> node_quality_iv;
     
 #ifdef debug
     cerr << "Concatenating entries for " << basis_length << " bases and " << edge_coverage_length << " edges" << endl;
@@ -310,6 +351,10 @@ void Packer::make_compact(void) {
             {
                 util::assign(edge_coverage_iv, int_vector<>(edge_coverage_length));
             }
+#pragma omp task
+            {
+                util::assign(node_quality_iv, int_vector<>(node_quality_length));
+            }
         }
     }
 #pragma omp parallel for
@@ -320,6 +365,10 @@ void Packer::make_compact(void) {
     for (size_t i = 0; i < edge_coverage_length; ++i) {
         edge_coverage_iv[i] = edge_coverage(i);
     }
+#pragma omp parallel for
+    for (size_t i = 1; i < node_quality_length; ++i) {
+        node_quality_iv[i] = average_node_quality(i);
+    }
 
     #pragma omp parallel
     {
@@ -333,6 +382,10 @@ void Packer::make_compact(void) {
             {
                 util::assign(edge_coverage_civ, edge_coverage_iv);
             }
+#pragma omp task
+            {
+                util::assign(node_quality_civ, node_quality_iv);
+            }
         }
     }
     
@@ -399,9 +452,10 @@ void Packer::remove_edit_tmpfiles(void) {
     }
 }
 
-void Packer::add(const Alignment& aln, int min_mapq, int min_baseq , bool qual_adjust) {
+void Packer::add(const Alignment& aln, int min_mapq, int min_baseq) {
     // mapping quality threshold filter
-    if (aln.mapping_quality() < min_mapq) {
+    int mapping_quality = aln.mapping_quality();
+    if (mapping_quality < min_mapq) {
         return;
     }
     // count the nodes, edges, and edits
@@ -412,7 +466,6 @@ void Packer::add(const Alignment& aln, int min_mapq, int min_baseq , bool qual_a
     size_t position_in_read = 0;
     for (size_t mi = 0; mi < aln.path().mapping_size(); ++mi) {
         auto& mapping = aln.path().mapping(mi);
-        int mapping_quality = aln.mapping_quality();
         if (!mapping.has_position()) {
 #ifdef debug
             cerr << "Mapping has no position" << endl;
@@ -426,48 +479,54 @@ void Packer::add(const Alignment& aln, int min_mapq, int min_baseq , bool qual_a
             continue;
         }
         size_t i = position_in_basis(mapping.position());
+        size_t node_quality_index = node_index(mapping.position().node_id());
+        size_t total_node_quality = 0;
         // keep track of average base quality in the mapping
         int bq_total = 0;
         int bq_count = 0;
-        for (auto& edit : mapping.edit()) {
-            if (edit_is_match(edit)) {
+        if (record_bases or record_qualities) {
+            for (auto& edit : mapping.edit()) {
+                if (edit_is_match(edit)) {
 #ifdef debug
-                cerr << "Recording a match" << endl;
+                    cerr << "Recording a match" << endl;
 #endif
-                int direction = mapping.position().is_reverse() ? -1 : 1;
-                for (size_t j = 0; j < edit.from_length(); ++j, ++position_in_read) {
-                    int64_t coverage_idx = i + direction * j;
-                    int base_quality = compute_quality(aln, position_in_read);
-                    bq_total += base_quality;
-                    ++bq_count;
-                    // base quality threshold filter (only if we found some kind of quality)
-                    if (base_quality < 0 || base_quality >= min_baseq) {
-                        if (!qual_adjust) {
+                    int direction = mapping.position().is_reverse() ? -1 : 1;
+                    for (size_t j = 0; j < edit.from_length(); ++j, ++position_in_read) {
+                        int64_t coverage_idx = i + direction * j;
+                        int base_quality = compute_quality(aln, position_in_read);
+                        bq_total += base_quality;
+                        ++bq_count;
+                        // base quality threshold filter (only if we found some kind of quality)
+                        if (base_quality < 0 || base_quality >= min_baseq) {
                             increment_coverage(coverage_idx);
-                        } else {
-                            increment_coverage(coverage_idx, base_quality);
+                            if (record_qualities && mapping_quality > 0) {
+                                total_node_quality += mapping_quality;
+                            }
                         }
-                    }
-                }         
-            } else if (record_edits) {
-                // we represent things on the forward strand
-                string pos_repr = pos_key(i);
-                string edit_repr = edit_value(edit, mapping.position().is_reverse());
-                size_t bin = bin_for_position(i);
-                std::lock_guard<std::mutex> guard(tmpfstream_locks[bin]);
-                *tmpfstreams[bin] << pos_repr << edit_repr;
-            } 
-            if (mapping.position().is_reverse()) {
-                i -= edit.from_length();
-            } else {
-                i += edit.from_length();
+                    }         
+                } else if (record_edits) {
+                    // we represent things on the forward strand
+                    string pos_repr = pos_key(i);
+                    string edit_repr = edit_value(edit, mapping.position().is_reverse());
+                    size_t bin = bin_for_position(i);
+                    std::lock_guard<std::mutex> guard(tmpfstream_locks[bin]);
+                    *tmpfstreams[bin] << pos_repr << edit_repr;
+                } 
+                if (mapping.position().is_reverse()) {
+                    i -= edit.from_length();
+                } else {
+                    i += edit.from_length();
+                }
+                if (!edit_is_match(edit)) {
+                    position_in_read += edit.to_length();
+                }
             }
-            if (!edit_is_match(edit)) {
-                position_in_read += edit.to_length();
+            if (total_node_quality > 0) {
+                increment_node_quality(node_quality_index, total_node_quality);
             }
         }
-
-        if (has_prev_mapping && prev_mapping.position().node_id() != mapping.position().node_id()) {
+        
+        if (record_edges && has_prev_mapping && prev_mapping.position().node_id() != mapping.position().node_id()) {
             // Note: we are effectively ignoring edits here.  So an edge is covered even
             // if there's a sub or indel at either of its ends in the path.  
             Edge e;
@@ -491,14 +550,10 @@ void Packer::add(const Alignment& aln, int min_mapq, int min_baseq , bool qual_a
                 }
                 // base quality threshold filter (only if we found some kind of quality)
                 if (avg_base_quality < 0 || avg_base_quality >= min_baseq) {
-                    if (!qual_adjust) {
-                        increment_edge_coverage(edge_idx);
-                    } else {
-                        increment_edge_coverage(edge_idx, combine_qualities(aln.mapping_quality(), avg_base_quality));
-                    }
+                    increment_edge_coverage(edge_idx);
                 }
             }
-        }            
+        }
 
         prev_mapping = mapping;
         has_prev_mapping = true;
@@ -591,6 +646,14 @@ size_t Packer::edge_vector_size(void) const{
     }
 }
 
+size_t Packer::node_quality_vector_size(void) const {
+    if (is_compacted) {
+        return node_quality_civ.size();
+    } else {
+        return num_nodes_dynamic;
+    }
+}
+
 pair<size_t, size_t> Packer::coverage_bin_offset(size_t i) const {
     size_t bin = min((size_t)(i / cov_bin_size), (size_t)(coverage_dynamic.size() - 1));
     // last bin can have different size so we don't use mod
@@ -605,6 +668,13 @@ pair<size_t, size_t> Packer::edge_coverage_bin_offset(size_t i) const {
     return make_pair(bin, offset);
 }
 
+pair<size_t, size_t> Packer::node_quality_bin_offset(size_t i) const {
+    size_t bin = min((size_t)(i / node_qual_bin_size), (size_t)(node_quality_dynamic.size() - 1));
+    // last bin can have different size so we don't use mod
+    size_t offset = i - bin * node_qual_bin_size;
+    return make_pair(bin, offset);
+}
+
 size_t Packer::coverage_bin_size(size_t i) const {
     size_t bin_size = cov_bin_size;
     if (i == coverage_dynamic.size() - 1) {
@@ -621,6 +691,14 @@ size_t Packer::edge_coverage_bin_size(size_t i) const {
     return bin_size;
 }
 
+size_t Packer::node_quality_bin_size(size_t i) const {
+    size_t bin_size = node_qual_bin_size;
+    if (i == node_quality_dynamic.size() - 1) {
+        bin_size += num_nodes_dynamic % node_quality_dynamic.size();
+    }
+    return bin_size;
+}
+
 void Packer::init_coverage_bin(size_t i) {
     if (coverage_dynamic[i] == nullptr) {
         coverage_dynamic[i] = new gcsa::CounterArray(coverage_bin_size(i), data_width);
@@ -633,6 +711,13 @@ void Packer::init_edge_coverage_bin(size_t i) {
     }
 }
 
+void Packer::init_node_quality_bin(size_t i) {
+    if (node_quality_dynamic[i] == nullptr) {
+        // add some bits to the data width because we have to add a quality and not 1 each time
+        node_quality_dynamic[i] = new gcsa::CounterArray(node_quality_bin_size(i), data_width + 12);
+    }
+}
+
 void Packer::increment_coverage(size_t i) {
     pair<size_t, size_t> bin_offset = coverage_bin_offset(i);
     std::lock_guard<std::mutex> guard(base_locks[bin_offset.first]);
@@ -671,6 +756,39 @@ void Packer::increment_edge_coverage(size_t i, size_t v) {
     }
 }
 
+void Packer::increment_node_quality(size_t i, size_t v) {
+    if (v > 0) {
+        pair<size_t, size_t> bin_offset = node_quality_bin_offset(i);
+        std::lock_guard<std::mutex> guard(node_quality_locks[bin_offset.first]);
+        init_node_quality_bin(bin_offset.first);
+        node_quality_dynamic.at(bin_offset.first)->increment(bin_offset.second, v);
+#ifdef debug
+        cerr << "Set quality of node " << i << " at " << bin_offset.first << "/" << bin_offset.second << " to " << (*node_quality_dynamic.at(bin_offset.first))[bin_offset.second] << endl;
+#endif
+    }        
+}
+
+bool Packer::has_qualities() const {
+    if (is_compacted) {
+        for (size_t i = 0; i < node_quality_civ.size(); ++i) {
+            if (node_quality_civ[i] > 0) {
+                return true;
+            }
+        }
+    } else {
+        for (size_t i = 0; i < node_quality_dynamic.size(); ++i) {
+            if (node_quality_dynamic[i] != nullptr) {
+                for (size_t j = 0; j < node_quality_dynamic[i]->size(); ++j) {
+                    if ((*node_quality_dynamic.at(i))[j] > 0) {
+                        return true;
+                    }
+                }
+            }
+        }
+    }
+    return false;
+}
+
 size_t Packer::coverage_at_position(size_t i) const {
     if (is_compacted) {
         return coverage_civ[i];
@@ -697,12 +815,59 @@ size_t Packer::edge_coverage(size_t i) const {
         }
     }
 }
-
+            
 size_t Packer::edge_coverage(Edge& e) const {
     size_t pos = edge_index(e);
     return edge_coverage(pos);
 }
 
+size_t Packer::total_node_quality(size_t i) const {
+    if (is_compacted) {
+        size_t avg_qual = average_node_quality(i);
+        Position pos;
+        pos.set_node_id(index_to_node(i));
+        pos.set_is_reverse(false);
+        size_t cov_pos = position_in_basis(pos);
+        size_t node_len = graph->get_length(graph->get_handle(pos.node_id()));
+        size_t base = position_in_basis(pos);
+        size_t coverage = 0;
+        for (size_t i = 0; i < node_len; ++i) {
+            coverage += coverage_at_position(base + i);
+        }
+        return avg_qual * coverage;
+    } else {
+        pair<size_t, size_t> bin_offset = node_quality_bin_offset(i);
+        if (node_quality_dynamic[bin_offset.first] == nullptr) {
+            return 0;
+        } else {
+            return (*node_quality_dynamic.at(bin_offset.first))[bin_offset.second];
+        }
+    }
+}
+
+size_t Packer::average_node_quality(size_t i) const {
+    if (is_compacted) {
+        return node_quality_civ[i];
+    } else {
+        Position pos;
+        pos.set_node_id(index_to_node(i));
+        pos.set_is_reverse(false);
+        size_t cov_pos = position_in_basis(pos);
+        size_t node_len = graph->get_length(graph->get_handle(pos.node_id()));
+        size_t base = position_in_basis(pos);
+        size_t total_coverage = 0;
+        for (size_t i = 0; i < node_len; ++i) {
+            total_coverage += coverage_at_position(base + i);
+        }
+        if (total_coverage == 0) {
+            assert(total_node_quality(i) == 0);
+            return 0;
+        } else {
+            return (size_t)std::lround(total_node_quality(i) / total_coverage);
+        }
+    }
+}
+
 vector<Edit> Packer::edits_at_position(size_t i) const {
     vector<Edit> edits;
     if (i == 0) return edits;
@@ -747,6 +912,17 @@ size_t Packer::edge_index(const Edge& e) const {
     return dynamic_cast<const VectorizableHandleGraph*>(graph)->edge_index(edge);
 }
 
+size_t Packer::node_index(nid_t node_id) const {
+    if (!graph->has_node(node_id)) {
+        return 0;
+    }
+    return dynamic_cast<const VectorizableHandleGraph*>(graph)->id_to_rank(node_id);
+}
+
+nid_t Packer::index_to_node(size_t i) const {
+    return dynamic_cast<const VectorizableHandleGraph*>(graph)->rank_to_id(i);
+}
+
 ostream& Packer::as_table(ostream& out, bool show_edits, vector<vg::id_t> node_ids) {
 #ifdef debug
     cerr << "Packer table of " << coverage_civ.size() << " rows:" << endl;
@@ -833,6 +1009,24 @@ ostream& Packer::as_edge_table(ostream& out, vector<vg::id_t> node_ids) {
     return out;
 }
     
+ostream& Packer::as_quality_table(ostream& out, vector<vg::id_t> node_ids) {
+#ifdef debug
+    cerr << "Packer quality table of " << node_quality_civ.size() << " rows:" << endl;
+#endif
+
+    out << "node.rank" << "\t"
+        << "node.id" << "\t"
+        << "avg-mapq";
+    out << endl;
+    for (size_t i = 1; i < node_quality_civ.size(); ++i) {
+        nid_t node_id = index_to_node(i);
+        if (!node_ids.empty() && find(node_ids.begin(), node_ids.end(), node_id) == node_ids.end()) {
+            continue;
+        }
+        out << i << "\t" << node_id << "\t" << node_quality_civ[i] << endl;
+    }
+    return out;
+}
 
 ostream& Packer::show_structure(ostream& out) {
     out << coverage_civ << endl; // graph coverage (compacted coverage_dynamic)
