@@ -22,7 +22,8 @@ void Sampler::set_source_paths(const vector<string>& source_paths,
         if (haplotype_transcripts.empty()) {
             for (const pair<string, double>& transcript_expression : transcript_expressions) {
                 this->source_paths.push_back(transcript_expression.first);
-                expression_values.push_back(transcript_expression.second);
+                size_t tx_len = xgidx->get_path_length(xgidx->get_path_handle(transcript_expression.first));
+                expression_values.push_back(transcript_expression.second * tx_len);
             }
         }
         else {
@@ -37,7 +38,8 @@ void Sampler::set_source_paths(const vector<string>& source_paths,
                 }
                 for (size_t i : haplotypes_of_transcript[transcript_expression.first]) {
                     double haplotype_expression = (transcript_expression.second * get<2>(haplotype_transcripts[i])) / total_haplotypes;
-                    expression_values.push_back(haplotype_expression);
+                    size_t hp_tx_len = xgidx->get_path_length(xgidx->get_path_handle(get<0>(haplotype_transcripts[i])));
+                    expression_values.push_back(haplotype_expression * hp_tx_len);
                     this->source_paths.push_back(get<0>(haplotype_transcripts[i]));
                 }
             }
@@ -600,29 +602,29 @@ NGSSimulator::NGSSimulator(PathPositionHandleGraph& graph,
                            double substition_polymorphism_rate,
                            double indel_polymorphism_rate,
                            double indel_error_proportion,
-                           double insert_length_mean,
-                           double insert_length_stdev,
+                           double fragment_length_mean,
+                           double fragment_length_stdev,
                            double error_multiplier,
                            bool retry_on_Ns,
+                           bool sample_unsheared_paths,
                            size_t seed) :
       graph(graph)
-    , node_cache(100)
-    , edge_cache(100)
     , sub_poly_rate(substition_polymorphism_rate)
     , indel_poly_rate(indel_polymorphism_rate)
     , indel_error_prop(indel_error_proportion)
-    , insert_mean(insert_length_mean)
-    , insert_sd(insert_length_stdev)
+    , fragment_mean(fragment_length_mean)
+    , fragment_sd(fragment_length_stdev)
     , retry_on_Ns(retry_on_Ns)
     , prng(seed ? seed : random_device()())
     , strand_sampler(0, 1)
     , background_sampler(0, alphabet.size() - 1)
     , mut_sampler(0, alphabet.size() - 2)
     , prob_sampler(0.0, 1.0)
-    , insert_sampler(insert_length_mean, insert_length_stdev)
+    , fragment_sampler(fragment_length_mean, fragment_length_stdev)
     , seed(seed)
     , source_paths(source_paths_input)
     , joint_initial_distr(seed - 1)
+    , sample_unsheared_paths(sample_unsheared_paths)
 {
     if (!ngs_paired_fastq_file.empty() && interleaved_fastq) {
         cerr << "error:[NGSSimulator] cannot indicate interleaved FASTQ and paired FASTQs simultaneously" << endl;
@@ -651,34 +653,37 @@ NGSSimulator::NGSSimulator(PathPositionHandleGraph& graph,
         exit(1);
     }
     
-    if (insert_length_mean <= 0.0) {
-        cerr << "error:[NGSSimulator] Mean insert length must be positive" << endl;
+    if (fragment_length_mean <= 0.0) {
+        cerr << "error:[NGSSimulator] Mean fragment length must be positive" << endl;
         exit(1);
     }
     
-    if (insert_length_stdev < 0.0) {
-        cerr << "error:[NGSSimulator] Insert length standard deviation must be positive" << endl;
+    if (fragment_length_stdev < 0.0) {
+        cerr << "error:[NGSSimulator] Fragment length standard deviation must be positive" << endl;
         exit(1);
     }
     
-    if (insert_length_mean < 5.0 * insert_length_stdev) {
-        cerr << "warning:[NGSSimulator] Recommended that insert length mean (" << insert_length_mean
-            << ") > 5 * insert length standard deviation (" << insert_length_stdev << ")" << endl;
-    }
     
-    graph.for_each_handle([&](const handle_t& handle) {
-        total_seq_length += graph.get_length(handle);
-    });
     
     if (source_paths_input.empty() && transcript_expressions.empty()) {
         // we are sampling from all positions
+        graph.for_each_handle([&](const handle_t& handle) {
+            total_seq_length += graph.get_length(handle);
+        });
         start_pos_samplers.emplace_back(1, total_seq_length);
     }
     else if (!source_paths_input.empty()) {
         // we are sampling from a given set of source paths
         vector<size_t> path_sizes;
         for (const auto& source_path : source_paths_input) {
-            path_sizes.push_back(graph.get_path_length(graph.get_path_handle(source_path)));
+            if (sample_unsheared_paths) {
+                // sample uniformly between paths
+                path_sizes.push_back(1);
+            }
+            else {
+                // sample proportional to length
+                path_sizes.push_back(graph.get_path_length(graph.get_path_handle(source_path)));
+            }
             start_pos_samplers.emplace_back(0, path_sizes.back() - 1);
         }
         path_sampler = vg::discrete_distribution<>(path_sizes.begin(), path_sizes.end());
@@ -691,9 +696,13 @@ NGSSimulator::NGSSimulator(PathPositionHandleGraph& graph,
             // no transcript name file provided, path names should match transcript names in the
             // expression file
             for (const pair<string, double>& transcript_expression : transcript_expressions) {
+                size_t tx_len = graph.get_path_length(graph.get_path_handle(transcript_expression.first));
+                if (tx_len == 0) {
+                    continue;
+                }
                 source_paths.push_back(transcript_expression.first);
-                start_pos_samplers.emplace_back(0, graph.get_path_length(graph.get_path_handle(transcript_expression.first)) - 1);
-                expression_values.push_back(transcript_expression.second);
+                start_pos_samplers.emplace_back(0, tx_len - 1);
+                expression_values.push_back(transcript_expression.second * (sample_unsheared_paths ? 1 : tx_len));
             }
         }
         else {
@@ -709,14 +718,14 @@ NGSSimulator::NGSSimulator(PathPositionHandleGraph& graph,
                     total_haplotypes += get<2>(haplotype_transcripts[i]);
                 }
                 for (size_t i : haplotypes_of_transcript[transcript_expression.first]) {
-                    size_t path_length = graph.get_path_length(graph.get_path_handle(get<0>(haplotype_transcripts[i])));
-                    if (path_length == 0) {
+                    size_t hp_tx_len = graph.get_path_length(graph.get_path_handle(get<0>(haplotype_transcripts[i])));
+                    if (hp_tx_len == 0) {
                         continue;
                     }
                     double haplotype_expression = (transcript_expression.second * get<2>(haplotype_transcripts[i])) / total_haplotypes;
-                    expression_values.push_back(haplotype_expression);
+                    expression_values.push_back(haplotype_expression  * (sample_unsheared_paths ? 1 : hp_tx_len));
                     source_paths.push_back(get<0>(haplotype_transcripts[i]));
-                    start_pos_samplers.emplace_back(0, path_length - 1);
+                    start_pos_samplers.emplace_back(0, hp_tx_len - 1);
                 }
             }
         }
@@ -779,8 +788,8 @@ NGSSimulator::NGSSimulator(PathPositionHandleGraph& graph,
         cerr << "warning:[NGSSimulator] Auto-detected read length of " << modal_length << " encompasses less than half of training reads, NGSSimulator is optimized for training data in which most reads are the same length" << endl;
     }
     
-    if (modal_length > insert_length_mean - 2.0 * insert_length_stdev) {
-        cerr << "warning:[NGSSimulator] Auto-detected read length of " << modal_length << " is long compared to mean insert length " << insert_length_mean << " and standard deviation " << insert_length_stdev << ", sampling may take additional time and statistical properties of insert length distribution may not reflect input parameters" << endl;
+    if (modal_length > fragment_length_mean - 2.0 * fragment_length_stdev) {
+        cerr << "warning:[NGSSimulator] Auto-detected read length of " << modal_length << " is long compared to mean fragment length " << fragment_length_mean << " and standard deviation " << fragment_length_stdev << ". Sampling may take additional time and the statistical properties of the fragment length distribution may not reflect input parameters." << endl;
     }
     
     // shorten the quality string samplers until they are the modal length (this determines read length later)
@@ -805,8 +814,10 @@ NGSSimulator::NGSSimulator(PathPositionHandleGraph& graph,
 
 Alignment NGSSimulator::sample_read() {
     
-    
     Alignment aln;
+    
+    aln.set_name(get_read_name());
+    
     // sample a quality string based on the trained distribution
     pair<string, vector<bool>> qual_and_masks = sample_read_quality();
     
@@ -823,6 +834,13 @@ Alignment NGSSimulator::sample_read() {
     
     aln.set_quality(qual_and_masks.first);
     
+    // Sample our path (if dealing with source_paths)
+    size_t source_path_idx = sample_path();
+    string source_path;
+    if (source_path_idx != numeric_limits<size_t>::max()) {
+        source_path = source_paths[source_path_idx];
+    }
+    
     // attempt samples until we get one that succeeds without walking
     // off the end of the graph
     while (!aln.has_path()) {
@@ -832,10 +850,8 @@ Alignment NGSSimulator::sample_read() {
         bool is_reverse;
         // And our position in the graph, which we use whether there's a source path or not.
         pos_t pos;
-        // And our path (if dealing with source_paths)
-        string source_path;
         // Populate them
-        sample_start_pos(offset, is_reverse, pos, source_path);
+        sample_start_pos(source_path_idx, -1, offset, is_reverse, pos);
         // align the first end at this position on the source path or graph
         sample_read_internal(aln, offset, is_reverse, pos, source_path);
         
@@ -851,13 +867,17 @@ Alignment NGSSimulator::sample_read() {
     // mask out any of the sequence that we sampled to be an 'N'
     apply_N_mask(*aln.mutable_sequence(), qual_and_masks.second);
     
-    aln.set_name(get_read_name());
     algorithms::annotate_with_initial_path_positions(graph, aln);
     return aln;
 }
 
 pair<Alignment, Alignment> NGSSimulator::sample_read_pair() {
     pair<Alignment, Alignment> aln_pair;
+    
+    string name = get_read_name();
+    aln_pair.first.set_name(name + "_1");
+    aln_pair.second.set_name(name + "_2");
+        
     pair<pair<string, vector<bool>>, pair<string, vector<bool>>> qual_and_mask_pair = sample_read_quality_pair();
     
 #ifdef debug_ngs_sim
@@ -878,37 +898,64 @@ pair<Alignment, Alignment> NGSSimulator::sample_read_pair() {
     assert(qual_and_mask_pair.first.first.size() == qual_and_mask_pair.first.second.size());
     assert(qual_and_mask_pair.second.first.size() == qual_and_mask_pair.second.second.size());
     
+    int64_t fragment_length = -1;
+    while (fragment_length <= 0) {
+        fragment_length = (int64_t) round(fragment_sampler(prng));
+    }
+    
+    // Sample our path (if dealing with source_paths)
+    size_t source_path_idx = sample_path();
+    string source_path;
+    if (source_path_idx != numeric_limits<size_t>::max()) {
+        source_path = source_paths[source_path_idx];
+#ifdef debug_ngs_sim
+        cerr << "sampling from path " << source_path << " with length " << graph.get_path_length(graph.get_path_handle(source_path)) << endl;
+#endif
+        fragment_length = min<int64_t>(fragment_length, graph.get_path_length(graph.get_path_handle(source_path)));
+    }
+    
+    if (fragment_length < transition_distrs_1.size()) {
+        // the fragment is shorter than the sequencing length
+        qual_and_mask_pair.first.first.resize(fragment_length);
+        qual_and_mask_pair.first.second.resize(fragment_length);
+        qual_and_mask_pair.second.first.resize(fragment_length);
+        qual_and_mask_pair.second.second.resize(fragment_length);
+    }
+    
     aln_pair.first.set_quality(qual_and_mask_pair.first.first);
     aln_pair.second.set_quality(qual_and_mask_pair.second.first);
     
+#ifdef debug_ngs_sim
+    cerr << "sampled fragment length " << fragment_length << endl;
+#endif
+    
     // reverse the quality string so that it acts like it's reading from the opposite end
     // when we walk forward from the beginning of the first read
     std::reverse(aln_pair.second.mutable_quality()->begin(),
                  aln_pair.second.mutable_quality()->end());
     
-    
     while (!aln_pair.first.has_path() || !aln_pair.second.has_path()) {
-        int64_t insert_length = (int64_t) round(insert_sampler(prng));
-        if (insert_length < (int64_t) transition_distrs_1.size()) {
-            // don't make reads where the insert length is shorter than one end of the read
-            continue;
-        }
-        
         // This is our offset along the source path, if in use
         size_t offset;
         // And our direction to go along the source path, if in use
         bool is_reverse;
         // And our position in the graph, which we use whether there's a source path or not.
         pos_t pos;
-        // And our path (if dealing with source_paths)
-        string source_path;
         // Populate them
-        sample_start_pos(offset, is_reverse, pos, source_path);
+        sample_start_pos(source_path_idx, fragment_length, offset, is_reverse, pos);
+        
+#ifdef debug_ngs_sim
+        cerr << "sampled start pos " << pos << ", is reverse " << is_reverse << ", offset " << offset << endl;
+#endif
+        
         // align the first end at this position on the source path or graph
         sample_read_internal(aln_pair.first, offset, is_reverse, pos, source_path);
         
         if (retry_on_Ns) {
             if (aln_pair.first.sequence().find('N') != string::npos) {
+#ifdef debug_ngs_sim
+                cerr << "rejecting sample because of an N" << endl;
+#endif
                 aln_pair.first.clear_path();
                 aln_pair.first.clear_sequence();
             }
@@ -918,26 +965,39 @@ pair<Alignment, Alignment> NGSSimulator::sample_read_pair() {
             continue;
         }
         
-        // walk out the unsequenced part of the insert in the graph
-        int64_t remaining_length = insert_length - 2 * transition_distrs_1.size();
+        // walk out the unsequenced part of the fragment in the graph
+        int64_t remaining_length = fragment_length - (aln_pair.first.quality().size() +
+                                                      aln_pair.second.quality().size());
+#ifdef debug_ngs_sim
+        cerr << "walking " << remaining_length << " to start of next read in pair" << endl;
+#endif
         if (remaining_length >= 0) {
             // we need to move forward from the end of the first read
             if (advance_by_distance(offset, is_reverse, pos, remaining_length, source_path)) {
+#ifdef debug_ngs_sim
+                cerr << "rejecting sample because insert is off of path" << endl;
+#endif
                 // we hit the end of the graph trying to walk
                 continue;
             }
         } else {
             // we need to walk backwards from the end of the first read
-            pos = walk_backwards(aln_pair.first.path(), -remaining_length);
+            int64_t walk_length = min<int64_t>(-remaining_length, path_from_length(aln_pair.first.path()));
+            pos = walk_backwards(aln_pair.first.path(), walk_length);
             // Make sure to update the offset along the path as well. If offset
             // and is_reverse aren't being used (becasue we aren't in path
             // mode), the result won't be used either.
-            offset += is_reverse ? -remaining_length : remaining_length;
+            offset += is_reverse ? walk_length : -walk_length;
         }
         // guard against running off the end of nodes
         // XXX this should not be happening
         // it seems to occur in some graphs due to the behavior of advance_by_distance
-        if (vg::offset(pos) >= graph.get_length(graph.get_handle(id(pos)))) continue;
+        if (vg::offset(pos) >= graph.get_length(graph.get_handle(id(pos)))) {
+#ifdef debug_ngs_sim
+            cerr << "rejecting sample because of invalid walked location" << endl;
+#endif
+            continue;
+        }
 
         // align the second end starting at the walked position
         sample_read_internal(aln_pair.second, offset, is_reverse, pos, source_path); 
@@ -952,16 +1012,13 @@ pair<Alignment, Alignment> NGSSimulator::sample_read_pair() {
     
     // unreverse the second read in the pair
     aln_pair.second = reverse_complement_alignment(aln_pair.second, [&](id_t node_id) {
-            return graph.get_length(graph.get_handle(node_id));
+        return graph.get_length(graph.get_handle(node_id));
     });
     
     // mask out any of the sequence that we sampled to be an 'N'
     apply_N_mask(*aln_pair.first.mutable_sequence(), qual_and_mask_pair.first.second);
     apply_N_mask(*aln_pair.second.mutable_sequence(), qual_and_mask_pair.second.second);
-    
-    string name = get_read_name();
-    aln_pair.first.set_name(name + "_1");
-    aln_pair.second.set_name(name + "_2");
+        
     algorithms::annotate_with_initial_path_positions(graph, aln_pair.first);
     algorithms::annotate_with_initial_path_positions(graph, aln_pair.second);
     return aln_pair;
@@ -969,12 +1026,19 @@ pair<Alignment, Alignment> NGSSimulator::sample_read_pair() {
 
 void NGSSimulator::sample_read_internal(Alignment& aln, size_t& offset, bool& is_reverse, pos_t& curr_pos,
                                         const string& source_path) {
-   
+    
+    // we will accept a read that cannot be extended to the full read length if we're simulating from
+    // a path that's too small or if we are sampling unsheared paths
+    bool accept_partial = sample_unsheared_paths;
+    if (!accept_partial && !source_path.empty()) {
+        accept_partial = graph.get_path_length(graph.get_path_handle(source_path)) < transition_distrs_1.size();
+    }
+    
     // Make sure we are starting inside the node
     // XXX this is broken
     auto first_node_length = graph.get_length(graph.get_handle(id(curr_pos)));
     if (vg::offset(curr_pos) >= first_node_length) {
-        cerr << "somithgn wrong << " << vg::offset(curr_pos) << " " << first_node_length << endl;
+        cerr << "something wrong " << vg::offset(curr_pos) << " " << first_node_length << endl;
         cerr << vg::id(curr_pos) << ":" << vg::is_rev(curr_pos) << ":" << vg::offset(curr_pos) << endl;
     }
     assert(vg::offset(curr_pos) < first_node_length);
@@ -998,6 +1062,9 @@ void NGSSimulator::sample_read_internal(Alignment& aln, size_t& offset, bool& is
             apply_insertion(aln, curr_pos);
         }
         if (aln.sequence().size() >= aln.quality().size() || hit_end) {
+#ifdef debug_ngs_sim
+            cerr << "break 1: ending sample with seq len " << aln.sequence().size() << ", qual len " << aln.quality().size() << ", hit end? " << hit_end << endl;
+#endif
             break;
         }
         
@@ -1014,6 +1081,9 @@ void NGSSimulator::sample_read_internal(Alignment& aln, size_t& offset, bool& is
                 apply_insertion(aln, curr_pos);
                 
                 if (aln.sequence().size() >= aln.quality().size() || hit_end) {
+#ifdef debug_ngs_sim
+                    cerr << "break 2: ending sample with seq len " << aln.sequence().size() << ", qual len " << aln.quality().size() << ", hit end? " << hit_end << endl;
+#endif
                     break;
                 }
             }
@@ -1030,6 +1100,9 @@ void NGSSimulator::sample_read_internal(Alignment& aln, size_t& offset, bool& is
             err_prob = phred_prob[aln.quality()[aln.sequence().size()]];
         }
         if (aln.sequence().size() >= aln.quality().size() || hit_end) {
+#ifdef debug_ngs_sim
+            cerr << "break 3: ending sample with seq len " << aln.sequence().size() << ", qual len " << aln.quality().size() << ", hit end? " << hit_end << endl;
+#endif
             break;
         }
         
@@ -1057,6 +1130,9 @@ void NGSSimulator::sample_read_internal(Alignment& aln, size_t& offset, bool& is
         hit_end = advance(offset, is_reverse, curr_pos, graph_char, source_path);
         
         if (aln.sequence().size() >= aln.quality().size() || hit_end) {
+#ifdef debug_ngs_sim
+            cerr << "break 4: ending sample with seq len " << aln.sequence().size() << ", qual len " << aln.quality().size() << ", hit end? " << hit_end << endl;
+#endif
             break;
         }
         
@@ -1074,8 +1150,14 @@ void NGSSimulator::sample_read_internal(Alignment& aln, size_t& offset, bool& is
     // remove the sequence and path if we hit the end the graph before finishing
     // the alignment
     if (aln.sequence().size() != aln.quality().size()) {
-        aln.clear_path();
-        aln.clear_sequence();
+        if (accept_partial) {
+            // we simulated the whole path, so we don't use the final quality values
+            aln.mutable_quality()->resize(aln.sequence().size());
+        }
+        else {
+            aln.clear_path();
+            aln.clear_sequence();
+        }
     }
     
 #ifdef debug_ngs_sim
@@ -1180,22 +1262,27 @@ bool NGSSimulator::advance_on_path_by_distance(size_t& offset, bool& is_reverse,
     if (is_reverse) {
         // Go left on the path
         if (offset < distance) {
-            // We hit the end
-            return true;
+            // We hit the beginning
+            offset = 0;
+        }
+        else {
+            offset -= distance;
         }
-        offset -= distance;
     } else {
         // Go right on the path
         if (offset + distance >= path_length) {
             // We hit the end
-            return true;
+            offset = path_length - 1;
+        }
+        else {
+            offset += distance;
         }
-        offset += distance;
     }
     
     // Set position according to position on path
     pos = position_at(&graph, source_path, offset, is_reverse);
     
+    // TODO: the behavior has changed so that this never returns true anymore
     return false;
 }
 
@@ -1203,9 +1290,7 @@ pos_t NGSSimulator::walk_backwards(const Path& path, size_t distance) {
     // Starting at the past-the-end of the path, walk back to the given nonzero distance.
     // Walking back the whole path length puts you at the start of the path.
     
-    if (distance > path_to_length(path)) {
-        throw runtime_error("Cannot walk back " + to_string(distance) + " on path of length " + to_string(path_to_length(path)));
-    }
+    distance = min<size_t>(distance, path_from_length(path));
     assert(distance > 0);
 
     // walk backwards until we find the mapping it's on
@@ -1419,14 +1504,19 @@ void NGSSimulator::apply_insertion(Alignment& aln, const pos_t& pos) {
     }
 }
 
-void NGSSimulator::sample_start_pos(size_t& offset, bool& is_reverse, pos_t& pos, string& source_path) {
+size_t NGSSimulator::sample_path() {
+    return source_paths.empty() ? numeric_limits<size_t>::max() : path_sampler(prng);
+}
+
+void NGSSimulator::sample_start_pos(const size_t& source_path_idx, const int64_t& fragment_length,
+                                    size_t& offset, bool& is_reverse, pos_t& pos) {
     if (source_paths.empty()) {
         pos = sample_start_graph_pos();
         offset = 0;
         is_reverse = false;
-        source_path = "";
-    } else {
-        tie(offset, is_reverse, pos, source_path) = sample_start_path_pos();
+    }
+    else {
+        tie(offset, is_reverse, pos) = sample_start_path_pos(source_path_idx, fragment_length);
     }
 }
 
@@ -1442,22 +1532,35 @@ pos_t NGSSimulator::sample_start_graph_pos() {
     return make_pos_t(id, rev, node_offset);
 }
 
-tuple<size_t, bool, pos_t, string> NGSSimulator::sample_start_path_pos() {
-    // choose a path
-    size_t source_path_idx = path_sampler(prng);
-    string source_path = source_paths[source_path_idx];
-    // The start pos sampler hasd been set up in path space, 0-based
-    size_t offset = start_pos_samplers[source_path_idx](prng);
+tuple<size_t, bool, pos_t> NGSSimulator::sample_start_path_pos(const size_t& source_path_idx,
+                                                               const int64_t& fragment_length) {
+    size_t path_length = graph.get_path_length(graph.get_path_handle(source_paths[source_path_idx]));
     bool rev = strand_sampler(prng);
-    pos_t pos = position_at(&graph, source_path, offset, rev);
+    size_t offset;
+    if (sample_unsheared_paths || path_length < transition_distrs_1.size() ||
+        (fragment_length > 0 && fragment_length >= path_length)) {
+        if (rev) {
+            offset = path_length - 1;
+        }
+        else {
+            offset = 0;
+        }
+    }
+    else {
+        // The start pos sampler has been set up in path space, 0-based
+        offset = start_pos_samplers[source_path_idx](prng);
+    }
+    pos_t pos = position_at(&graph, source_paths[source_path_idx], offset, rev);
     
-    return make_tuple(offset, rev, pos, source_path);
+    return make_tuple(offset, rev, pos);
 }
 
 string NGSSimulator::get_read_name() {
     stringstream sstrm;
-    sstrm << "seed_" << seed << "_fragment_" << sample_counter;
-    sample_counter++;
+    size_t num;
+#pragma omp atomic capture
+    num = sample_counter++;
+    sstrm << "seed_" << seed << "_fragment_" << num;
     return sstrm.str();
 }
 
@@ -1541,8 +1644,7 @@ pair<string, vector<bool>> NGSSimulator::sample_read_quality_internal(pair<uint8
 }
                                               
 void NGSSimulator::apply_N_mask(string& sequence, const vector<bool>& n_mask) {
-    assert(sequence.size() == n_mask.size());
-    for (size_t i = 0; i < n_mask.size(); i++) {
+    for (size_t i = 0; i < sequence.size(); i++) {
         if (n_mask[i]) {
             sequence[i] = 'N';
         }
