@@ -22,9 +22,9 @@
 #include "../haplotype_extracter.hpp"
 #include "../algorithms/sorted_id_ranges.hpp"
 #include "../algorithms/weakly_connected_components.hpp"
-#include <bdsg/overlay_helper.hpp>
+#include <bdsg/overlays/overlay_helper.hpp>
 #include "../io/save_handle_graph.hpp"
-#include "convert_handle.hpp"
+#include "../algorithms/copy_graph.hpp"
 
 using namespace std;
 using namespace vg;
@@ -655,11 +655,11 @@ int main_chunk(int argc, char** argv) {
                 // our graph is not in vg format.  covert it, extend it, convert it back
                 // this can eventually be avoided by handlifying the haplotype tracer
                 VG vg;
-                convert_path_handle_graph(subgraph.get(), &vg);
+                algorithms::copy_path_handle_graph(subgraph.get(), &vg);
                 subgraph.reset();
                 vg.extend(g);
                 subgraph = vg::io::new_output_graph<MutablePathMutableHandleGraph>(output_format);
-                convert_path_handle_graph(&vg, subgraph.get());
+                algorithms::copy_path_handle_graph(&vg, subgraph.get());
             }
         }
 
