@@ -7,7 +7,7 @@
 #include "../alignment_emitter.hpp"
 #include <vg/io/stream.hpp>
 #include <vg/io/vpkg.hpp>
-#include <bdsg/overlay_helper.hpp>
+#include <bdsg/overlays/overlay_helper.hpp>
 
 #include <unistd.h>
 #include <getopt.h>
@@ -59,7 +59,7 @@ void help_map(char** argv) {
          << "scoring:" << endl
          << "    -q, --match INT               use this match score [1]" << endl
          << "    -z, --mismatch INT            use this mismatch penalty [4]" << endl
-         << "    --score-matrix FILE           read a 5x5 integer substitution scoring matrix from a file" << endl
+         << "    --score-matrix FILE           read a 4x4 integer substitution scoring matrix from a file" << endl
          << "    -o, --gap-open INT            use this gap open penalty [6]" << endl
          << "    -y, --gap-extend INT          use this gap extension penalty [1]" << endl
          << "    -L, --full-l-bonus INT        the full-length alignment bonus [5]" << endl
@@ -83,6 +83,7 @@ void help_map(char** argv) {
          << "    -R, --read-group NAME         for --reads input, add this read group" << endl
          << "output:" << endl
          << "    -j, --output-json             output JSON rather than an alignment stream (helpful for debugging)" << endl
+         << "    -%, --gaf                     output alignments in GAF format" << endl
          << "    --surject-to TYPE             surject the output into the graph's paths, writing TYPE := bam |sam | cram" << endl
          << "    --buffer-size INT             buffer this many alignments together before outputting in GAM [512]" << endl
          << "    -X, --compare                 realign GAM input (-G), writing alignment with \"correct\" field set to overlap with input" << endl
@@ -91,7 +92,6 @@ void help_map(char** argv) {
          << "    -M, --max-multimaps INT       produce up to INT alignments for each read [1]" << endl
          << "    -Q, --mq-max INT              cap the mapping quality at INT [60]" << endl
          << "    --exclude-unaligned           exclude reads with no alignment" << endl
-
          << "    -D, --debug                   print debugging information about alignment to stderr" << endl;
 
 }
@@ -258,11 +258,12 @@ int main_map(int argc, char** argv) {
                 {"unpaired-cost", required_argument, 0, 'S'},
                 {"max-gap-length", required_argument, 0, 1},
                 {"xdrop-alignment", no_argument, 0, 2},
+                {"gaf", no_argument, 0, '%'},
                 {0, 0, 0, 0}
             };
 
         int option_index = 0;
-        c = getopt_long (argc, argv, "s:J:Q:d:x:g:1:T:N:R:c:M:t:G:jb:Kf:iw:P:Dk:Y:r:W:6H:Z:q:z:o:y:Au:B:I:S:l:e:C:V:O:L:a:n:E:X:UpF:m:7:v5:824:3:9:0:",
+        c = getopt_long (argc, argv, "s:J:Q:d:x:g:1:T:N:R:c:M:t:G:jb:Kf:iw:P:Dk:Y:r:W:6H:Z:q:z:o:y:Au:B:I:S:l:e:C:V:O:L:a:n:E:X:UpF:m:7:v5:824:3:9:0:%",
                          long_options, &option_index);
 
 
@@ -415,6 +416,10 @@ int main_map(int argc, char** argv) {
             output_format = "JSON";
             break;
 
+        case '%':
+            output_format = "GAF";
+            break;
+
         case 'w':
             band_width = parse<int>(optarg);
             band_width = band_width == 0 ? INT_MAX : band_width;
@@ -732,7 +737,7 @@ int main_map(int argc, char** argv) {
         });
 
     // Set up output to an emitter that will handle serialization
-    unique_ptr<AlignmentEmitter> alignment_emitter = get_alignment_emitter("-", output_format, path_length, thread_count);
+    unique_ptr<AlignmentEmitter> alignment_emitter = get_alignment_emitter("-", output_format, path_length, thread_count, xgidx);
 
     // TODO: Refactor the surjection code out of surject_main and into somewhere where we can just use it here!
 
@@ -815,9 +820,16 @@ int main_map(int argc, char** argv) {
         m->fast_reseed = use_fast_reseed;
         m->max_sub_mem_recursion_depth = max_sub_mem_recursion_depth;
         m->max_target_factor = max_target_factor;
-        m->set_alignment_scores(match, mismatch, gap_open, gap_extend, full_length_bonus, max_gap_length, haplotype_consistency_exponent);
-        if(matrix_stream.is_open()) m->load_scoring_matrix(matrix_stream);
+        if (matrix_stream.is_open()) {
+            m->set_alignment_scores(matrix_stream, gap_open, gap_extend, full_length_bonus, haplotype_consistency_exponent);
+            // reset the stream for the next Mapper
+            matrix_stream.seekg(0);
+        }
+        else {
+            m->set_alignment_scores(match, mismatch, gap_open, gap_extend, full_length_bonus, haplotype_consistency_exponent);
+        }
         m->strip_bonuses = strip_bonuses;
+        m->max_xdrop_gap_length = max_gap_length;
         m->adjust_alignments_for_base_quality = qual_adjust_alignments;
         m->extra_multimaps = extra_multimaps;
         m->mapping_quality_method = mapping_quality_method;
