@@ -15,12 +15,15 @@
 #include "subcommand.hpp"
 
 #include "../vg.hpp"
-#include "../xg.hpp"
 #include "../aligner.hpp"
+#include "../gbwt_helper.hpp"
+#include "../alignment_emitter.hpp"
 #include "../sampler.hpp"
+#include "../algorithms/copy_graph.hpp"
 #include <vg/io/protobuf_emitter.hpp>
 #include <vg/io/vpkg.hpp>
-#include <bdsg/overlay_helper.hpp>
+#include <bdsg/hash_graph.hpp>
+#include <bdsg/overlays/overlay_helper.hpp>
 
 using namespace std;
 using namespace vg;
@@ -82,15 +85,17 @@ void help_sim(char** argv) {
     cerr << "usage: " << argv[0] << " sim [options]" << endl
          << "Samples sequences from the xg-indexed graph." << endl
          << endl
-         << "options:" << endl
-         << "    -x, --xg-name FILE          use the xg index (or graph) in FILE" << endl
+         << "basic options:" << endl
+         << "    -x, --xg-name FILE          use the graph in FILE (required)" << endl
+         << "    -n, --num-reads N           simulate N reads or read pairs" << endl
+         << "    -l, --read-length N         simulate reads of length N" << endl
+         << "    -r, --progress              show progress information" << endl
+         << "output options:" << endl
+         << "    -a, --align-out             generate true alignments on stdout rather than reads" << endl
+         << "    -J, --json-out              write alignments in json" << endl
+         << "simulation parameters:" << endl
          << "    -F, --fastq FILE            match the error profile of NGS reads in FILE, repeat for paired reads (ignores -l,-f)" << endl
          << "    -I, --interleaved           reads in FASTQ (-F) are interleaved read pairs" << endl
-         << "    -P, --path PATH             simulate from the given names path (multiple allowed, cannot also give -T)" << endl
-         << "    -T, --tx-expr-file FILE     simulate from an expression profile formatted as RSEM output (cannot also give -P)" << endl
-         << "    -H, --haplo-tx-file FILE    transcript origin info table from vg rna -i (required for -T on haplotype transcripts)" << endl
-         << "    -l, --read-length N         write reads of length N" << endl
-         << "    -n, --num-reads N           simulate N reads or read pairs" << endl
          << "    -s, --random-seed N         use this specific seed for the PRNG" << endl
          << "    -e, --sub-rate FLOAT        base substitution rate (default 0.0)" << endl
          << "    -i, --indel-rate FLOAT      indel rate (default 0.0)" << endl
@@ -100,8 +105,15 @@ void help_sim(char** argv) {
          << "    -p, --frag-len N            make paired end reads with given fragment length N" << endl
          << "    -v, --frag-std-dev FLOAT    use this standard deviation for fragment length estimation" << endl
          << "    -N, --allow-Ns              allow reads to be sampled from the graph with Ns in them" << endl
-         << "    -a, --align-out             generate true alignments on stdout rather than reads" << endl
-         << "    -J, --json-out              write alignments in json" << endl;
+         << "    -t, --threads               number of compute threads (only when using FASTQ with -F) [1]" << endl
+         << "simulate from paths:" << endl
+         << "    -P, --path PATH             simulate from this path (may repeat; cannot also give -T)" << endl
+         << "    -A, --any-path              simulate from any path (overrides -P)" << endl
+         << "    -m, --sample-name NAME      simulate from this sample (may repeat; requires -g)" << endl
+         << "    -g, --gbwt-name FILE        use samples from this GBWT index" << endl
+         << "    -T, --tx-expr-file FILE     simulate from an expression profile formatted as RSEM output (cannot also give -P)" << endl
+         << "    -H, --haplo-tx-file FILE    transcript origin info table from vg rna -i (required for -T on haplotype transcripts)" << endl
+         << "    -u, --unsheared             sample from unsheared fragments" << endl;
 }
 
 int main_sim(int argc, char** argv) {
@@ -111,8 +123,12 @@ int main_sim(int argc, char** argv) {
         return 1;
     }
 
-    int read_length = 100;
+    string xg_name;
     int num_reads = 1;
+    int read_length = 100;
+    bool progress = false;
+    int threads = 1;
+
     int seed_val = time(NULL);
     double base_error = 0;
     double indel_error = 0;
@@ -122,15 +138,22 @@ int main_sim(int argc, char** argv) {
     int fragment_length = 0;
     double fragment_std_dev = 0;
     bool reads_may_contain_Ns = false;
-    string xg_name;
     bool strip_bonuses = false;
     bool interleaved = false;
+    bool unsheared_fragments = false;
     double indel_prop = 0.0;
     double error_scale_factor = 1.0;
     string fastq_name;
     string fastq_2_name;
+
     // What path should we sample from? Empty string = the whole graph.
     vector<string> path_names;
+    bool any_path = false;
+
+    // Sample from GBWT threads.
+    std::vector<std::string> sample_names;
+    std::string gbwt_name;
+
     // Alternatively, which transcripts with how much expression?
     string rsem_file_name;
     vector<pair<string, double>> transcript_expressions;
@@ -146,9 +169,13 @@ int main_sim(int argc, char** argv) {
         {
             {"help", no_argument, 0, 'h'},
             {"xg-name", required_argument, 0, 'x'},
+            {"progress", no_argument, 0, 'r'},
             {"fastq", required_argument, 0, 'F'},
             {"interleaved", no_argument, 0, 'I'},
             {"path", required_argument, 0, 'P'},
+            {"any-path", no_argument, 0, 'A'},
+            {"sample-name", required_argument, 0, 'm'},
+            {"gbwt-name", required_argument, 0, 'g'},
             {"tx-expr-file", required_argument, 0, 'T'},
             {"read-length", required_argument, 0, 'l'},
             {"num-reads", required_argument, 0, 'n'},
@@ -157,17 +184,19 @@ int main_sim(int argc, char** argv) {
             {"align-out", no_argument, 0, 'a'},
             {"json-out", no_argument, 0, 'J'},
             {"allow-Ns", no_argument, 0, 'N'},
+            {"unsheared", no_argument, 0, 'u'},
             {"sub-rate", required_argument, 0, 'e'},
             {"indel-rate", required_argument, 0, 'i'},
             {"indel-err-prop", required_argument, 0, 'd'},
             {"scale-err", required_argument, 0, 'S'},
             {"frag-len", required_argument, 0, 'p'},
             {"frag-std-dev", required_argument, 0, 'v'},
+            {"threads", required_argument, 0, 't'},
             {0, 0, 0, 0}
         };
 
         int option_index = 0;
-        c = getopt_long (argc, argv, "hl:n:s:e:i:fax:Jp:v:Nd:F:P:T:H:S:I",
+        c = getopt_long (argc, argv, "hrl:n:s:e:i:fax:Jp:v:Nud:F:P:Am:g:T:H:S:It:",
                 long_options, &option_index);
 
         // Detect the end of the options.
@@ -181,6 +210,10 @@ int main_sim(int argc, char** argv) {
             xg_name = optarg;
             break;
             
+        case 'r':
+            progress = true;
+            break;
+
         case 'F':
             if (fastq_name.empty()) {
                 fastq_name = optarg;
@@ -201,7 +234,19 @@ int main_sim(int argc, char** argv) {
         case 'P':
             path_names.push_back(optarg);
             break;
-            
+
+        case 'A':
+            any_path = true;
+            break;
+
+        case 'm':
+            sample_names.push_back(optarg);
+            break;
+
+        case 'g':
+            gbwt_name = optarg;
+            break;
+
         case 'T':
             rsem_file_name = optarg;
             break;
@@ -259,6 +304,10 @@ int main_sim(int argc, char** argv) {
         case 'N':
             reads_may_contain_Ns = true;
             break;
+                
+        case 'u':
+            unsheared_fragments = true;
+            break;
 
         case 'p':
             fragment_length = parse<int>(optarg);
@@ -267,6 +316,10 @@ int main_sim(int argc, char** argv) {
         case 'v':
             fragment_std_dev = parse<double>(optarg);
             break;
+                
+        case 't':
+            threads = parse<int>(optarg);
+            break;
             
         case 'h':
         case '?':
@@ -278,13 +331,22 @@ int main_sim(int argc, char** argv) {
             abort ();
         }
     }
+    
+    omp_set_num_threads(threads);
 
     if (xg_name.empty()) {
-        cerr << "[vg sim] error: we need an xg index to sample reads from" << endl;
+        cerr << "[vg sim] error: we need a graph to sample reads from" << endl;
         return 1;
     }
-    
+    if (gbwt_name.empty() != sample_names.empty()) {
+        cerr << "[vg sim] error: --gbwt-name and --sample-name must be used together" << endl;
+        return 1;
+    }
+
     if (!rsem_file_name.empty()) {
+        if (progress) {
+            std::cerr << "Reading transcription profile from " << rsem_file_name << std::endl;
+        }
         ifstream rsem_in(rsem_file_name);
         if (!rsem_in) {
             cerr << "[vg sim] error: could not open transcription profile file " << rsem_file_name << endl;
@@ -294,6 +356,9 @@ int main_sim(int argc, char** argv) {
     }
     
     if (!haplotype_transcript_file_name.empty()) {
+        if (progress) {
+            std::cerr << "Reading haplotype transcript file " << haplotype_transcript_file_name << std::endl;
+        }
         ifstream haplo_tx_in(haplotype_transcript_file_name);
         if (!haplo_tx_in) {
             cerr << "[vg sim] error: could not open haplotype transcript file " << haplotype_transcript_file_name << endl;
@@ -302,23 +367,96 @@ int main_sim(int argc, char** argv) {
         haplotype_transcripts = parse_haplotype_transcript_file(haplo_tx_in);
     }
 
-    unique_ptr<PathHandleGraph> path_handle_graph;
-    bdsg::PathPositionVectorizableOverlayHelper overlay_helper;
-    PathPositionHandleGraph* xgidx = nullptr;
-
-    if (!xg_name.empty()) {
-        path_handle_graph = vg::io::VPKG::load_one<PathHandleGraph>(xg_name);
-        xgidx = dynamic_cast<PathPositionHandleGraph*>(overlay_helper.apply(path_handle_graph.get()));
+    if (progress) {
+        std::cerr << "Loading graph " << xg_name << std::endl;
     }
-    
-    for (auto& path_name : path_names) {
-        if (xgidx->has_path(path_name) == false) {
-            cerr << "[vg sim] error: path \""<< path_name << "\" not found in index" << endl;
+    unique_ptr<PathHandleGraph> path_handle_graph = vg::io::VPKG::load_one<PathHandleGraph>(xg_name);
+
+    // Deal with GBWT threads
+    if (!gbwt_name.empty()) {
+        if (progress) {
+            std::cerr << "Loading GBWT index " << gbwt_name << std::endl;
+        }
+        std::unique_ptr<gbwt::GBWT> gbwt_index = vg::io::VPKG::load_one<gbwt::GBWT>(gbwt_name);
+        if (!(gbwt_index->hasMetadata()) || !(gbwt_index->metadata.hasSampleNames()) || !(gbwt_index->metadata.hasPathNames())) {
+            std::cerr << "[vg sim] error: GBWT index does not contain sufficient metadata" << std::endl;
             return 1;
         }
+        if (progress) {
+            std::cerr << "Checking " << sample_names.size() << " samples" << std::endl;
+        }
+        hash_set<gbwt::size_type> sample_ids;
+        for (std::string& sample_name : sample_names) {
+            gbwt::size_type id = gbwt_index->metadata.sample(sample_name);
+            if (id >= gbwt_index->metadata.samples()) {
+                std::cerr << "[vg sim] error: sample \"" << sample_name << "\" not found in the GBWT index" << std::endl;
+                return 1;
+            }
+            sample_ids.insert(id);
+        }
+        MutablePathMutableHandleGraph* mutable_graph = dynamic_cast<MutablePathMutableHandleGraph*>(path_handle_graph.get());
+        if (mutable_graph == nullptr) {
+            if (progress) {
+                std::cerr << "Converting the graph into HashGraph" << std::endl;
+            }
+            mutable_graph = new bdsg::HashGraph();
+            algorithms::copy_path_handle_graph(path_handle_graph.get(), mutable_graph);
+            path_handle_graph.reset(mutable_graph);
+        }
+        if (progress) {
+            std::cerr << "Inserting " << sample_ids.size() << " samples into the graph" << std::endl;
+        }
+        size_t inserted = 0;
+        for (gbwt::size_type i = 0; i < gbwt_index->metadata.paths(); i++) {
+            auto& path = gbwt_index->metadata.path(i);
+            if (sample_ids.find(path.sample) != sample_ids.end()) {
+                std::string path_name = insert_gbwt_path(*mutable_graph, *gbwt_index, i);
+                if (!path_name.empty()) {
+                    path_names.push_back(path_name);
+                    inserted++;
+                }
+            }
+        }
+        if (progress) {
+            std::cerr << "Inserted " << inserted << " paths" << std::endl;
+        }
+    }
+
+    if (progress) {
+        std::cerr << "Creating path position overlay" << std::endl;
+    }
+    bdsg::PathPositionVectorizableOverlayHelper overlay_helper;
+    PathPositionHandleGraph* xgidx = dynamic_cast<PathPositionHandleGraph*>(overlay_helper.apply(path_handle_graph.get()));
+
+    // Deal with path names.
+    if (any_path) {
+        if (progress) {
+            std::cerr << "Selecting all " << xgidx->get_path_count() << " paths" << std::endl;
+        }
+        if (xgidx->get_path_count() == 0) {
+            cerr << "[vg sim] error: the graph does not contain paths" << endl;
+            return 1;
+        }
+        path_names.clear();
+        xgidx->for_each_path_handle([&](const path_handle_t& handle) {
+            path_names.push_back(xgidx->get_path_name(handle));
+        });
+    } else if (!path_names.empty()) {
+        if (progress) {
+            std::cerr << "Checking " << path_names.size() << " selected paths" << std::endl;
+        }
+        for (auto& path_name : path_names) {
+            if (xgidx->has_path(path_name) == false) {
+                cerr << "[vg sim] error: path \""<< path_name << "\" not found in index" << endl;
+                return 1;
+            }
+        }
     }
     
     if (haplotype_transcript_file_name.empty()) {
+        if (progress && !transcript_expressions.empty()) {
+            std::cerr << "Checking " << transcript_expressions.size() << " transcripts" << std::endl;
+        }
         for (auto& transcript_expression : transcript_expressions) {
             if (!xgidx->has_path(transcript_expression.first)) {
                 cerr << "[vg sim] error: transcript path for \""<< transcript_expression.first << "\" not found in index" << endl;
@@ -328,6 +466,9 @@ int main_sim(int argc, char** argv) {
         }
     }
     else {
+        if (progress) {
+            std::cerr << "Checking " << haplotype_transcripts.size() << " haplotype transcripts" << std::endl;
+        }
         for (auto& haplotype_transcript : haplotype_transcripts) {
             if (!xgidx->has_path(get<0>(haplotype_transcript))) {
                 cerr << "[vg sim] error: transcript path for \""<< get<0>(haplotype_transcript) << "\" not found in index" << endl;
@@ -342,14 +483,68 @@ int main_sim(int argc, char** argv) {
         // Make an emitter to emit Alignments
         aln_emitter = unique_ptr<vg::io::ProtobufEmitter<Alignment>>(new vg::io::ProtobufEmitter<Alignment>(cout));
     }
-    
+
+    if (progress) {
+        std::cerr << "Simulating " << (fragment_length > 0 ? "read pairs" : "reads") << std::endl;
+        std::cerr << "--num-reads " << num_reads << std::endl;
+        std::cerr << "--read-length " << read_length << std::endl;
+        if (align_out) {
+            std::cerr << "--align-out" << std::endl;
+        }
+        if (json_out) {
+            std::cerr << "--json-out" << std::endl;
+        }
+        if (!fastq_name.empty()) {
+            std::cerr << "--fastq " << fastq_name << std::endl;
+            if (!fastq_2_name.empty()) {
+                std::cerr << "--fastq " << fastq_2_name << std::endl;
+            }
+            if (interleaved) {
+                std::cerr << "--interleaved" << std::endl;
+            }
+        } else {
+            if (base_error > 0.0) {
+                std::cerr << "--sub-rate " << base_error << std::endl;
+            }
+        }
+        if (indel_error > 0.0) {
+            std::cerr << "--indel-rate " << indel_error << std::endl;
+        }
+        if (!fastq_name.empty()) {
+            if (indel_prop > 0.0) {
+                std::cerr << "--indel-err-prop " << indel_prop << std::endl;
+            }
+            if (error_scale_factor != 1.0) {
+                std::cerr << "--scale-err " << error_scale_factor << std::endl;
+            }
+        }
+        if (forward_only) {
+            std::cerr << "--forward-only" << std::endl;
+        }
+        if (fragment_length > 0) {
+            std::cerr << "--frag-len " << fragment_length << std::endl;
+            if (fragment_std_dev > 0.0) {
+                std::cerr << "--frag-std-dev " << fragment_std_dev << std::endl;
+            }
+        }
+        if (reads_may_contain_Ns) {
+            std::cerr << "--allow-Ns" << std::endl;
+        }
+    }
+
     if (fastq_name.empty()) {
         // Use the fixed error rate sampler
         
+        if (unsheared_fragments) {
+            cerr << "Unsheared fragment option only available when simulating from FASTQ-trained errors" << endl;
+        }
+        
         // Make a sample to sample reads with
         Sampler sampler(xgidx, seed_val, forward_only, reads_may_contain_Ns, path_names, transcript_expressions, haplotype_transcripts);
         
-        Aligner rescorer(default_match, default_mismatch, default_gap_open, default_gap_extension, default_full_length_bonus);
+        // initialize an aligner
+        Aligner rescorer(default_score_matrix, default_gap_open, default_gap_extension,
+                         default_full_length_bonus, vg::default_gc_content);
 
         // We define a function to score a using the aligner
         auto rescore = [&] (Alignment& aln) {
@@ -433,7 +628,8 @@ int main_sim(int argc, char** argv) {
     else {
         // Use the trained error rate
         
-        Aligner aligner(default_match, default_mismatch, default_gap_open, default_gap_extension, 5);
+        Aligner aligner(default_score_matrix, default_gap_open, default_gap_extension,
+                         default_full_length_bonus, vg::default_gc_content);
         
         NGSSimulator sampler(*xgidx,
                              fastq_name,
@@ -449,43 +645,36 @@ int main_sim(int argc, char** argv) {
                              fragment_std_dev ? fragment_std_dev : 0.000001, // eliminates errors from having 0 as stddev without substantial difference
                              error_scale_factor,
                              !reads_may_contain_Ns,
+                             unsheared_fragments,
                              seed_val);
         
-        if (fragment_length) {
-            for (size_t i = 0; i < num_reads; i++) {
+        unique_ptr<AlignmentEmitter> alignment_emitter = get_alignment_emitter("-", json_out ? "JSON" : "GAM",
+                                                                               map<string, int64_t>(), get_thread_count());
+        
+#pragma omp parallel for
+        for (size_t i = 0; i < num_reads; i++) {
+            if (fragment_length) {
                 pair<Alignment, Alignment> read_pair = sampler.sample_read_pair();
                 read_pair.first.set_score(aligner.score_ungapped_alignment(read_pair.first, strip_bonuses));
                 read_pair.second.set_score(aligner.score_ungapped_alignment(read_pair.second, strip_bonuses));
                 
                 if (align_out) {
-                    if (json_out) {
-                        cout << pb2json(read_pair.first) << endl;
-                        cout << pb2json(read_pair.second) << endl;
-                    }
-                    else {
-                        aln_emitter->write_copy(read_pair.first);
-                        aln_emitter->write_copy(read_pair.second);
-                    }
+                    alignment_emitter->emit_pair(std::move(read_pair.first), std::move(read_pair.second));
                 }
                 else {
+#pragma omp critical
                     cout << read_pair.first.sequence() << "\t" << read_pair.second.sequence() << endl;
                 }
             }
-        }
-        else {
-            for (size_t i = 0; i < num_reads; i++) {
+            else {
                 Alignment read = sampler.sample_read();
                 read.set_score(aligner.score_ungapped_alignment(read, strip_bonuses));
                 
                 if (align_out) {
-                    if (json_out) {
-                        cout << pb2json(read) << endl;
-                    }
-                    else {
-                        aln_emitter->write_copy(read);
-                    }
+                    alignment_emitter->emit_single(std::move(read));
                 }
                 else {
+#pragma omp critical
                     cout << read.sequence() << endl;
                 }
             }
