@@ -18,9 +18,13 @@
 #include "../gfa.hpp"
 #include "../io/json_stream_helper.hpp"
 #include "../handle.hpp"
-#include "../convert_handle.hpp"
+#include "../algorithms/copy_graph.hpp"
+#include "../algorithms/gfa_to_handle.hpp"
+
 #include <vg/io/message_iterator.hpp>
 #include <vg/io/vpkg.hpp>
+#include <bdsg/hash_graph.hpp>
+
 
 using namespace std;
 using namespace vg;
@@ -113,6 +117,7 @@ int main_view(int argc, char** argv) {
     // and json-gam -> gam
     //     json-pileup -> pileup
 
+    bool which_read_in_pair = true;
     string output_type;
     string input_type;
     string rdf_base_uri;
@@ -490,13 +495,22 @@ int main_view(int argc, char** argv) {
             });
         }
     } else if (input_type == "gfa") {
-        get_input_file(file_name, [&](istream& in) {
-            graph = make_unique<VG>();
-            if (!gfa_to_graph(in, dynamic_cast<vg::VG*>(graph.get()))) {
-                // GFA loading has failed because the file is invalid
-                exit(1);
-            }
-        });
+        graph = make_unique<bdsg::HashGraph>();
+       
+        try {
+            // Use the disk-backed GFA loader that `vg convert` also uses.
+            algorithms::gfa_to_path_handle_graph(file_name,
+                dynamic_cast<MutablePathMutableHandleGraph*>(graph.get()));
+        } catch (algorithms::GFAFormatError& e) {
+            cerr << "error:[vg view] Input GFA is not acceptable." << endl;
+            cerr << e.what() << endl;
+            exit(1);
+        } catch (std::ios_base::failure& e) {
+            cerr << "error:[vg view] IO error processing input GFA." << endl;
+            cerr << e.what() << endl;
+            exit(1);
+        }
+        
         // GFA can convert to any of the graph formats, so keep going
     } else if(input_type == "json") {
         assert(input_json);
@@ -552,8 +566,11 @@ int main_view(int argc, char** argv) {
             else if (output_type == "multipath") {
                 vector<MultipathAlignment> buf;
                 function<void(Alignment&)> lambda = [&buf](Alignment& aln) {
+                    multipath_alignment_t mp_aln;
+                    to_multipath_alignment(aln, mp_aln);
                     buf.emplace_back();
-                    to_multipath_alignment(aln, buf.back());
+                    to_proto_multipath_alignment(mp_aln, buf.back());
+                    transfer_proto_metadata(aln, buf.back());
                     vg::io::write_buffered(cout, buf, 1000);
                 };
                 get_input_file(file_name, [&](istream& in) {
@@ -575,8 +592,11 @@ int main_view(int argc, char** argv) {
                 vector<MultipathAlignment> buf;
                 Alignment aln;
                 while (json_helper.get_read_fn()(aln)) {
+                    multipath_alignment_t mp_aln;
+                    to_multipath_alignment(aln, mp_aln);
                     buf.emplace_back();
-                    to_multipath_alignment(aln, buf.back());
+                    to_proto_multipath_alignment(mp_aln, buf.back());
+                    transfer_proto_metadata(aln, buf.back());
                     vg::io::write_buffered(std::cout, buf, 1000);
                 }
                 vg::io::write_buffered(cout, buf, 0);
@@ -626,10 +646,23 @@ int main_view(int argc, char** argv) {
             }
             else if (output_type == "gam") {
                 vector<Alignment> buf;
-                MultipathAlignment mp_aln;
-                while (json_helper.get_read_fn()(mp_aln)) {
+                MultipathAlignment proto_mp_aln;
+                while (json_helper.get_read_fn()(proto_mp_aln)) {
+                    multipath_alignment_t mp_aln;
+                    from_proto_multipath_alignment(proto_mp_aln, mp_aln);
                     buf.emplace_back();
                     optimal_alignment(mp_aln, buf.back());
+                    transfer_proto_metadata(proto_mp_aln, buf.back());
+                    if (!proto_mp_aln.paired_read_name().empty()) {
+                        // alternate using next/prev
+                        if (which_read_in_pair) {
+                            buf.back().mutable_fragment_next()->set_name(proto_mp_aln.paired_read_name());
+                        }
+                        else {
+                            buf.back().mutable_fragment_prev()->set_name(proto_mp_aln.paired_read_name());
+                        }
+                    }
+                    which_read_in_pair = !which_read_in_pair;
                     vg::io::write_buffered(std::cout, buf, 1000);
                 }
                 vg::io::write_buffered(cout, buf, 0);
@@ -672,9 +705,12 @@ int main_view(int argc, char** argv) {
             }
             else if (output_type == "gam") {
                 vector<Alignment> buf;
-                function<void(MultipathAlignment&)> lambda = [&buf](MultipathAlignment& mp_aln) {
+                function<void(MultipathAlignment&)> lambda = [&buf](MultipathAlignment& proto_mp_aln) {
+                    multipath_alignment_t mp_aln;
+                    from_proto_multipath_alignment(proto_mp_aln, mp_aln);
                     buf.emplace_back();
                     optimal_alignment(mp_aln, buf.back());
+                    transfer_proto_metadata(proto_mp_aln, buf.back());
                     vg::io::write_buffered(cout, buf, 1000);
                 };
                 get_input_file(file_name, [&](istream& in) {
@@ -839,7 +875,7 @@ int main_view(int argc, char** argv) {
     if (vg_graph == nullptr) {
         // Copy instead. Should be fine because we on;y ever want to run this on small graphs anyway.
         vg_graph = new vg::VG();
-        convert_path_handle_graph(graph.get(), vg_graph);
+        algorithms::copy_path_handle_graph(graph.get(), vg_graph);
         
         // Make sure the new VG has its Proto right
         // TODO: if we didn't reach into vg.graph we wouldn't need to do this.
