@@ -446,7 +446,7 @@ using namespace std;
 #endif
             
             // perform a full length surjection within the section section
-            sections.push_back(realigning_surject(graph, section_source, path_handle, section_path_chunks, true, true));
+            sections.push_back(realigning_surject(graph, section_source, path_handle, section_path_chunks, true, true, true));
             read_ranges.push_back(read_range);
             ref_ranges.push_back(ref_range);
             
@@ -519,6 +519,10 @@ using namespace std;
                                (deletion_length > 1 ? (deletion_length - 1) * get_aligner()->gap_extension : 0));
             }
             section_edge_scores[i] = edge_score;
+            
+#ifdef debug_spliced_surject
+            cerr << "edge has score " << edge_score << endl;
+#endif
         }
         
         // now we find use dynamic programming to find the best alignment across chunks
@@ -530,7 +534,7 @@ using namespace std;
         for (size_t i = 0; i < sections.size(); ++i) {
             score_dp[i] = sections[i].score();
         }
-        if (!allow_negative_scores) {
+        if (allow_negative_scores) {
             // remove the initialization if it's not a source so we don't allow subpath local alignments
             for (size_t i = 0; i < constriction_targets.size(); ++i) {
                 score_dp[constriction_comps[constriction_targets[i]]] = numeric_limits<int32_t>::min();
@@ -545,6 +549,11 @@ using namespace std;
             is_sink[from] = false;
             
             int32_t extended_score = score_dp[from] + section_edge_scores[i] + sections[to].score();
+            
+#ifdef debug_spliced_surject
+            cerr << "extending from component " << from << " (DP score " << score_dp[from] << ") with score of " << extended_score << " to " << to << " (DP score " << score_dp[to] << ")" << endl;
+#endif
+            
             if (extended_score > score_dp[to]) {
                 score_dp[to] = extended_score;
                 backpointer[to] = from;
@@ -569,6 +578,9 @@ using namespace std;
         
 #ifdef debug_spliced_surject
         cerr << "combining " << traceback.size() << " sections into surjected alignment" << endl;
+        for (int64_t i = traceback.size() - 1; i >= 0; --i) {
+            cerr << "\t" << traceback[i] << endl;
+        }
 #endif
         
         // make an alignment to build out the path in
@@ -661,7 +673,8 @@ using namespace std;
 
     Alignment Surjector::realigning_surject(const PathPositionHandleGraph* path_position_graph, const Alignment& source,
                                             const path_handle_t& path_handle, const vector<path_chunk_t>& path_chunks,
-                                            bool allow_negative_scores, bool preserve_N_alignments) const {
+                                            bool allow_negative_scores, bool preserve_N_alignments,
+                                            bool preserve_tail_indel_anchors) const {
         
 #ifdef debug_anchored_surject
         cerr << "using overlap chunks on path " << graph->get_path_name(path_handle) << ", performing realigning surjection" << endl;
@@ -701,7 +714,8 @@ using namespace std;
 #endif
         
         // compute the connectivity between the path chunks
-        MultipathAlignmentGraph mp_aln_graph(split_path_graph, path_chunks, source, node_trans, !preserve_N_alignments);
+        MultipathAlignmentGraph mp_aln_graph(split_path_graph, path_chunks, source, node_trans, !preserve_N_alignments,
+                                             preserve_tail_indel_anchors);
         
         // we don't overlap this reference path at all or we filtered out all of the path chunks, so just make a sentinel
         if (mp_aln_graph.empty()) {
@@ -714,10 +728,17 @@ using namespace std;
         mp_aln_graph.remove_transitive_edges(topological_order);
         
         // align the intervening segments and store the result in a multipath alignment
-        MultipathAlignment mp_aln;
-        mp_aln_graph.align(source, split_path_graph, get_aligner(), false, 1, false, 1, mp_aln, allow_negative_scores);
+        multipath_alignment_t mp_aln;
+        mp_aln_graph.align(source, split_path_graph, get_aligner(), false, 1, false, numeric_limits<int64_t>::max(),
+                           false, 1, mp_aln, allow_negative_scores);
         topologically_order_subpaths(mp_aln);
         
+        if (preserve_tail_indel_anchors) {
+            // this code path sometimes produces subpaths that have no aligned bases, which
+            // sometimes play poorly with other parts of the code base
+            remove_empty_subpaths(mp_aln);
+        }
+        
         for (size_t i = 0; i < mp_aln.subpath_size(); i++) {
             // translate back into the original ID space
             translate_oriented_node_ids(*mp_aln.mutable_subpath(i)->mutable_path(), node_trans);
@@ -741,7 +762,10 @@ using namespace std;
         Alignment surjected;
         optimal_alignment(mp_aln, surjected, allow_negative_scores);
         
-        // transfer applicable metadata
+        // transfer applicable metadata (including data that doesn't transit through multipath_alignment_t)
+        surjected.set_name(source.name());
+        surjected.set_read_group(source.read_group());
+        surjected.set_sample_name(source.sample_name());
         surjected.set_mapping_quality(source.mapping_quality());
         if (source.has_fragment_next()) {
             *surjected.mutable_fragment_next() = source.fragment_next();
@@ -749,6 +773,9 @@ using namespace std;
         if (source.has_fragment_prev()) {
             *surjected.mutable_fragment_prev() = source.fragment_prev();
         }
+        if (source.has_annotation()) {
+            *surjected.mutable_annotation() = source.annotation();
+        }
         
 #ifdef debug_anchored_surject
         cerr << "concatenated and translated alignment " << pb2json(surjected) << endl;
