@@ -14,7 +14,7 @@ namespace vg {
 
 using namespace std;
 
-// #define transcriptome_debug
+//#define transcriptome_debug
 
 
 Transcriptome::Transcriptome(const string & graph_filename, const bool show_progress) {
@@ -32,7 +32,7 @@ Transcriptome::Transcriptome(const string & graph_filename, const bool show_prog
     }
 }
 
-int32_t Transcriptome::add_intron_splice_junctions(istream & intron_stream, gbwt::GBWT * haplotype_index) {
+int32_t Transcriptome::add_intron_splice_junctions(istream & intron_stream, unique_ptr<gbwt::GBWT> & haplotype_index) {
 
 #ifdef transcriptome_debug
     double time_parsing_1 = gcsa::readTimer();
@@ -82,7 +82,7 @@ int32_t Transcriptome::add_intron_splice_junctions(istream & intron_stream, gbwt
     return introns.size();
 }
 
-int32_t Transcriptome::add_transcript_splice_junctions(istream & transcript_stream, gbwt::GBWT * haplotype_index) {
+int32_t Transcriptome::add_transcript_splice_junctions(istream & transcript_stream, unique_ptr<gbwt::GBWT> & haplotype_index) {
 
 #ifdef transcriptome_debug
     double time_parsing_1 = gcsa::readTimer();
@@ -172,8 +172,6 @@ vector<Transcript> Transcriptome::parse_introns(istream & intron_stream) const {
 
     vector<Transcript> introns;
 
-    pair<string, PathIndex *> chrom_path_index("", nullptr);
-
     int32_t line_number = 0;
 
     string chrom;
@@ -181,6 +179,9 @@ vector<Transcript> Transcriptome::parse_introns(istream & intron_stream) const {
     string end;
     string strand;
 
+    // Create path position overlay of splice graph
+    bdsg::PositionOverlay graph_path_pos_overlay(_splice_graph.get());
+
     while (intron_stream.good()) {
 
         line_number += 1;
@@ -197,17 +198,7 @@ vector<Transcript> Transcriptome::parse_introns(istream & intron_stream) const {
         
             cerr << "[transcriptome] ERROR: Chromomsome path \"" << chrom << "\" not found in graph (line " << line_number << ")." << endl;
             exit(1);
-
-        } else if (chrom_path_index.first != chrom) {
-
-            delete chrom_path_index.second;
-            chrom_path_index.first = chrom;
-
-            // Construct path index for chromosome/contig.
-            chrom_path_index.second = new PathIndex(*_splice_graph, chrom_path_index.first);
-        }
-
-        assert(chrom_path_index.second);
+        } 
 
         // Parse start and end intron position and convert end to inclusive.
         assert(getline(intron_stream, pos, '\t'));
@@ -232,11 +223,11 @@ vector<Transcript> Transcriptome::parse_introns(istream & intron_stream) const {
         }
 
         // Create "intron" transcript.
-        introns.emplace_back(Transcript("", is_reverse, chrom));
+        introns.emplace_back(Transcript("", is_reverse, chrom, graph_path_pos_overlay.get_path_length(_splice_graph->get_path_handle(chrom))));
 
         // Add intron boundaries as flanking exons to current "intron" transcript.
-        add_exon(&(introns.back()), make_pair(spos - 1, spos - 1), *chrom_path_index.second);
-        add_exon(&(introns.back()), make_pair(epos + 1, epos + 1), *chrom_path_index.second);
+        add_exon(&(introns.back()), make_pair(spos - 1, spos - 1), graph_path_pos_overlay);
+        add_exon(&(introns.back()), make_pair(epos + 1, epos + 1), graph_path_pos_overlay);
     }
 
     if (introns.empty()) {
@@ -245,8 +236,6 @@ vector<Transcript> Transcriptome::parse_introns(istream & intron_stream) const {
         exit(1);        
     }
 
-    delete chrom_path_index.second;
-
     return introns;
 }
 
@@ -254,8 +243,6 @@ vector<Transcript> Transcriptome::parse_transcripts(istream & transcript_stream)
 
     vector<Transcript> transcripts;
 
-    pair<string, PathIndex *> chrom_path_index("", nullptr);
-
     int32_t line_number = 0;
 
     string chrom;
@@ -272,6 +259,9 @@ vector<Transcript> Transcriptome::parse_transcripts(istream & transcript_stream)
     // Regex used to extract transcript name/id from gff file.
     regex regex_id_exp_gff(transcript_tag + "={1}([^;]*);?");
 
+    // Create path position overlay of splice graph
+    bdsg::PositionOverlay graph_path_pos_overlay(_splice_graph.get());
+
     while (transcript_stream.good()) {
 
         line_number += 1;
@@ -288,17 +278,7 @@ vector<Transcript> Transcriptome::parse_transcripts(istream & transcript_stream)
         
             cerr << "[transcriptome] ERROR: Chromomsome path \"" << chrom << "\" not found in graph (line " << line_number << ")." << endl;
             exit(1);
-
-        } else if (chrom_path_index.first != chrom) {
-
-            delete chrom_path_index.second;
-            chrom_path_index.first = chrom;
-
-            // Construct path index for chromosome/contig.
-            chrom_path_index.second = new PathIndex(*_splice_graph, chrom_path_index.first);
-        }
-
-        assert(chrom_path_index.second);
+        } 
 
         transcript_stream.ignore(numeric_limits<streamsize>::max(), '\t');         
         assert(getline(transcript_stream, feature, '\t'));
@@ -356,20 +336,20 @@ vector<Transcript> Transcriptome::parse_transcripts(istream & transcript_stream)
         // Is this a new transcript.
         if (transcripts.empty()) {
 
-            transcripts.emplace_back(Transcript(transcript_id, is_reverse, chrom));
+            transcripts.emplace_back(Transcript(transcript_id, is_reverse, chrom, graph_path_pos_overlay.get_path_length(_splice_graph->get_path_handle(chrom))));
         
         // Is this a new transcript.
         } else if (transcripts.back().name != transcript_id) {
 
             // Reorder reversed order exons.
             reorder_exons(&transcripts.back());
-            transcripts.emplace_back(Transcript(transcript_id, is_reverse, chrom));
+            transcripts.emplace_back(Transcript(transcript_id, is_reverse, chrom, graph_path_pos_overlay.get_path_length(_splice_graph->get_path_handle(chrom))));
         }
 
         assert(transcripts.back().is_reverse == is_reverse);
 
         // Add exon to current transcript.
-        add_exon(&(transcripts.back()), make_pair(spos, epos), *chrom_path_index.second);
+        add_exon(&(transcripts.back()), make_pair(spos, epos), graph_path_pos_overlay);
     }
 
     if (transcripts.empty()) {
@@ -378,8 +358,6 @@ vector<Transcript> Transcriptome::parse_transcripts(istream & transcript_stream)
         exit(1);        
     }
 
-    delete chrom_path_index.second;
-
     reorder_exons(&transcripts.back());
 
     return transcripts;
@@ -390,34 +368,43 @@ float Transcriptome::mean_node_length() const {
     return static_cast<float>(_splice_graph->get_total_length()) / _splice_graph->get_node_count();
 }
 
-void Transcriptome::add_exon(Transcript * transcript, const pair<int32_t, int32_t> & exon_pos, const PathIndex & chrom_path_index) const {
+void Transcriptome::add_exon(Transcript * transcript, const pair<int32_t, int32_t> & exon_pos, const bdsg::PositionOverlay & graph_path_pos_overlay) const {
 
     transcript->exons.emplace_back(exon_pos);
 
+    assert(exon_pos.first >= 0);
+    assert(exon_pos.second < transcript->chrom_length);
+
     // Exon border positions (last position in upstream intron and 
-    // first position in downstream intron).
-    const pair<int32_t, int32_t> exon_border_pos = make_pair(exon_pos.first - 1, exon_pos.second + 1);
+    // first position in downstream intron). The positions are not 
+    // offset if it is the first or last on the path.
+    const pair<uint32_t, uint32_t> exon_border_pos = make_pair(max(0, exon_pos.first - 1), min(static_cast<int32_t>(transcript->chrom_length) - 1, exon_pos.second + 1));
 
-    // Find path positions (node start position and id) of exon node 
-    // borders (start - 1 and end + 1) using path index.
-    auto chrom_path_index_start_it = chrom_path_index.find_position(exon_border_pos.first);
-    auto chrom_path_index_end_it = chrom_path_index.find_position(exon_border_pos.second);
+    auto path_handle = _splice_graph->get_path_handle(transcript->chrom);
 
-    assert(chrom_path_index_start_it != chrom_path_index.end());
-    assert(chrom_path_index_end_it != chrom_path_index.end());
+    // Find path positions of exon node borders (start - 1 and end + 1).
+    auto chrom_path_start_step = graph_path_pos_overlay.get_step_at_position(path_handle, exon_border_pos.first);
+    auto chrom_path_end_step = graph_path_pos_overlay.get_step_at_position(path_handle, exon_border_pos.second);
 
-    assert(chrom_path_index_start_it->first <= exon_border_pos.first);
-    assert(chrom_path_index_end_it->first <= exon_border_pos.second);
+    assert(chrom_path_start_step != graph_path_pos_overlay.path_end(path_handle));
+    assert(chrom_path_end_step != graph_path_pos_overlay.path_end(path_handle));
+
+    // Find the start position of the exon border nodes.
+    auto chrom_path_start_node_pos = graph_path_pos_overlay.get_position_of_step(chrom_path_start_step);
+    auto chrom_path_end_node_pos = graph_path_pos_overlay.get_position_of_step(chrom_path_end_step);
+
+    assert(chrom_path_start_node_pos <= exon_border_pos.first);
+    assert(chrom_path_end_node_pos <= exon_border_pos.second);
 
     transcript->exon_border_nodes.emplace_back(Position(), Position());
 
     // Set node id of exon border boundaries.
-    transcript->exon_border_nodes.back().first.set_node_id(chrom_path_index_start_it->second.node);
-    transcript->exon_border_nodes.back().second.set_node_id(chrom_path_index_end_it->second.node);
+    transcript->exon_border_nodes.back().first.set_node_id(_splice_graph->get_id(_splice_graph->get_handle_of_step(chrom_path_start_step)));
+    transcript->exon_border_nodes.back().second.set_node_id(_splice_graph->get_id(_splice_graph->get_handle_of_step(chrom_path_end_step)));
 
     // Set node offset of exon border boundaries. 
-    transcript->exon_border_nodes.back().first.set_offset(exon_border_pos.first - chrom_path_index_start_it->first);
-    transcript->exon_border_nodes.back().second.set_offset(exon_border_pos.second  - chrom_path_index_end_it->first);
+    transcript->exon_border_nodes.back().first.set_offset(exon_border_pos.first - chrom_path_start_node_pos);
+    transcript->exon_border_nodes.back().second.set_offset(exon_border_pos.second  - chrom_path_end_node_pos);
 
     // Set whether exon node border boundaries are reverse.
     transcript->exon_border_nodes.back().first.set_is_reverse(transcript->is_reverse);
@@ -533,7 +520,7 @@ void Transcriptome::project_and_add_transcripts_callback(const int32_t thread_id
             completed_transcript_paths = construct_completed_transcript_paths(project_transcript_gbwt(transcript, haplotype_index, mean_node_length)); 
         }
 
-        if (use_embedded_paths || use_reference_paths) { 
+        if (use_all_paths || use_reference_paths) { 
 
             // Project transcript onto embedded paths.
             auto new_completed_transcript_paths = construct_completed_transcript_paths(project_transcript_embedded(transcript, false));
@@ -704,7 +691,9 @@ list<EditedTranscriptPath> Transcriptome::project_transcript_gbwt(const Transcri
 
                 int32_t offset = 0;
 
-                if (node_id == exon_border_start_node.node_id()) {
+                // Adjust start position from exon border (last position in upstream intron)
+                // to first position in exon. Do not adjust if first position in path.
+                if ((cur_transcript.exons.at(exon_idx).first > 0) && (node_id == exon_border_start_node.node_id())) {
 
                     if (exon_border_start_node.offset() + 1 == node_length) {
 
@@ -722,8 +711,8 @@ list<EditedTranscriptPath> Transcriptome::project_transcript_gbwt(const Transcri
                 int32_t edit_length = node_length - offset;
 
                 // Adjust end position from exon border (first position in downstream intron)
-                // to last position in exon.
-                if (node_id == exon_border_end_nodes.node_id()) {
+                // to last position in exon. Do not adjust if last position in path.
+                if ((cur_transcript.exons.at(exon_idx).second < cur_transcript.chrom_length - 1) && (node_id == exon_border_end_nodes.node_id())) {
 
                     if (exon_border_end_nodes.offset() == 0) {
 
@@ -754,25 +743,31 @@ list<EditedTranscriptPath> Transcriptome::project_transcript_gbwt(const Transcri
             }
         }
 
-        assert(edited_transcript_paths.back().path.mapping_size() > 0);
+        if (edited_transcript_paths.back().path.mapping_size() == 0) {
 
-        if (cur_transcript.is_reverse) {
+            // Delete empty paths.
+            edited_transcript_paths.pop_back();
 
-            // Reverse complement transcript paths that are on the '-' strand.
-            reverse_complement_path_in_place(&(edited_transcript_paths.back().path), [&](size_t node_id) {return _splice_graph->get_length(_splice_graph->get_handle(node_id, false));});
-        }
+        } else {
 
-        // Copy paths if collapse of identical transcript paths is not wanted.
-        if (!collapse_transcript_paths && edited_transcript_paths.back().haplotype_origin_ids.size() > 1) {
+            if (cur_transcript.is_reverse) {
 
-            auto all_haplotype_origin_ids = edited_transcript_paths.back().haplotype_origin_ids;
-            edited_transcript_paths.back().haplotype_origin_ids = {edited_transcript_paths.back().haplotype_origin_ids.front()};
+                // Reverse complement transcript paths that are on the '-' strand.
+                reverse_complement_path_in_place(&(edited_transcript_paths.back().path), [&](size_t node_id) {return _splice_graph->get_length(_splice_graph->get_handle(node_id, false));});
+            }
 
-            // Create identical copies of all haplotype origins.
-            for (size_t i = 1; i < all_haplotype_origin_ids.size(); ++i) {
+            // Copy paths if collapse of identical transcript paths is not wanted.
+            if (!collapse_transcript_paths && edited_transcript_paths.back().haplotype_origin_ids.size() > 1) {
 
-                edited_transcript_paths.emplace_back(edited_transcript_paths.back());
-                edited_transcript_paths.back().haplotype_origin_ids.front() = {all_haplotype_origin_ids.at(i)};
+                auto all_haplotype_origin_ids = edited_transcript_paths.back().haplotype_origin_ids;
+                edited_transcript_paths.back().haplotype_origin_ids = {edited_transcript_paths.back().haplotype_origin_ids.front()};
+
+                // Create identical copies of all haplotype origins.
+                for (size_t i = 1; i < all_haplotype_origin_ids.size(); ++i) {
+
+                    edited_transcript_paths.emplace_back(edited_transcript_paths.back());
+                    edited_transcript_paths.back().haplotype_origin_ids.front() = {all_haplotype_origin_ids.at(i)};
+                }
             }
         }
     }
@@ -934,10 +929,10 @@ list<EditedTranscriptPath> Transcriptome::project_transcript_embedded(const Tran
             continue;
         }
 
-        // Construct only transcript paths originating from a reference chromosome/contig.
         if (path_origin_name != cur_transcript.chrom) {
-            
-            if (reference_only || (!use_embedded_paths && use_reference_paths)) {
+
+            // Construct only transcript paths originating from a reference chromosome/contig.        
+            if (reference_only || !use_all_paths) {
 
                 continue;
             }
@@ -953,7 +948,14 @@ list<EditedTranscriptPath> Transcriptome::project_transcript_embedded(const Tran
 
         } else {
 
-            edited_transcript_paths.back().haplotype_origin_ids.emplace_back(-1 * as_integer(path_steps_start.first));
+            if (edited_transcript_paths.back().path_origin_names.empty()) {
+
+                edited_transcript_paths.back().path_origin_names = path_origin_name;
+
+            } else {
+
+                edited_transcript_paths.back().path_origin_names.append("," + path_origin_name);
+            }
         }
 
         bool is_partial = false;
@@ -978,6 +980,18 @@ list<EditedTranscriptPath> Transcriptome::project_transcript_embedded(const Tran
             // Get path mapping at exon end if exon end node is in the current path.
             auto haplotype_path_end_step = haplotype_path_end_it->second;
 
+            bool is_path_origin_reverse = _splice_graph->get_is_reverse(_splice_graph->get_handle_of_step(haplotype_path_start_step));
+
+            // Transcript paths are partial if the orientation of start 
+            // and end exon path step is different. Partial transcripts 
+            // are currently not supported.            
+            // TODO: Add support for this really special case.
+            if (is_path_origin_reverse != _splice_graph->get_is_reverse(_splice_graph->get_handle_of_step(haplotype_path_end_step))) {
+
+                is_partial = true;
+                break;
+            }
+
             bool is_first_step = true;
 
             while (true) {
@@ -986,15 +1000,23 @@ list<EditedTranscriptPath> Transcriptome::project_transcript_embedded(const Tran
                 int32_t offset = 0;
 
                 // Adjust start position from exon border (last position in upstream intron)
-                // to first position in exon.
-                if (is_first_step) {
+                // to first position in exon. Do not adjust if first position in path.
+                if ((cur_transcript.exons.at(exon_idx).first > 0) && is_first_step) {
 
                     if (cur_transcript.exon_border_nodes.at(exon_idx).first.offset() + 1 == node_length) {
 
                         assert(haplotype_path_start_step != haplotype_path_end_step);
-                        haplotype_path_start_step = _splice_graph->get_next_step(haplotype_path_start_step);
-                        is_first_step = false;
 
+                        if (is_path_origin_reverse) {
+
+                            haplotype_path_start_step = _splice_graph->get_previous_step(haplotype_path_start_step);
+
+                        } else {
+
+                            haplotype_path_start_step = _splice_graph->get_next_step(haplotype_path_start_step);
+                        }
+
+                        is_first_step = false;
                         continue;
                     
                     } else {
@@ -1006,8 +1028,8 @@ list<EditedTranscriptPath> Transcriptome::project_transcript_embedded(const Tran
                 int32_t edit_length = node_length - offset;
 
                 // Adjust end position from exon border (first position in downstream intron)
-                // to last position in exon.
-                if (haplotype_path_start_step == haplotype_path_end_step) {
+                // to last position in exon. Do not adjust if last position in path.
+                if ((cur_transcript.exons.at(exon_idx).second < cur_transcript.chrom_length - 1) && (haplotype_path_start_step == haplotype_path_end_step)) {
 
                     if (cur_transcript.exon_border_nodes.at(exon_idx).second.offset() == 0) {
 
@@ -1038,7 +1060,15 @@ list<EditedTranscriptPath> Transcriptome::project_transcript_embedded(const Tran
                 
                 if (haplotype_path_start_step == haplotype_path_end_step) { break; }
 
-                haplotype_path_start_step = _splice_graph->get_next_step(haplotype_path_start_step);
+                if (is_path_origin_reverse) {
+
+                    haplotype_path_start_step = _splice_graph->get_previous_step(haplotype_path_start_step);
+
+                } else {
+
+                    haplotype_path_start_step = _splice_graph->get_next_step(haplotype_path_start_step);
+                }
+
                 is_first_step = false;
             }
         }
@@ -1048,9 +1078,12 @@ list<EditedTranscriptPath> Transcriptome::project_transcript_embedded(const Tran
             // Delete partial transcript paths.
             edited_transcript_paths.pop_back();
         
-        } else {
+        } else if (edited_transcript_paths.back().path.mapping_size() == 0) {
 
-            assert(edited_transcript_paths.back().path.mapping_size() > 0);
+            // Delete empty paths.
+            edited_transcript_paths.pop_back();
+
+        } else {
 
             // Reverse complement transcript paths that are on the '-' strand.
             if (cur_transcript.is_reverse) {
@@ -1092,6 +1125,16 @@ void Transcriptome::append_transcript_paths(list<CompletedTranscriptPath> * comp
 
                     // Merge haplotype origin ids.
                     completed_transcript_path.haplotype_origin_ids.insert(completed_transcript_path.haplotype_origin_ids.end(), new_completed_transcript_path.haplotype_origin_ids.begin(), new_completed_transcript_path.haplotype_origin_ids.end());
+
+                    // Merge embedded path origin names.
+                    if (completed_transcript_path.path_origin_names.empty()) {
+
+                        completed_transcript_path.path_origin_names = new_completed_transcript_path.path_origin_names;
+
+                    } else if (!new_completed_transcript_path.path_origin_names.empty()) {
+
+                        completed_transcript_path.path_origin_names.append("," + new_completed_transcript_path.path_origin_names);
+                    }
                     
                     new_path_unqiue = false;
                     break;
@@ -1121,6 +1164,7 @@ list<CompletedTranscriptPath> Transcriptome::construct_completed_transcript_path
         completed_transcript_paths.back().name = transcript_path.name;
         completed_transcript_paths.back().reference_origin = transcript_path.reference_origin;
         completed_transcript_paths.back().haplotype_origin_ids = transcript_path.haplotype_origin_ids;
+        completed_transcript_paths.back().path_origin_names = transcript_path.path_origin_names;
         completed_transcript_paths.back().path = path_to_handles(transcript_path.path);
     }
 
@@ -1187,12 +1231,11 @@ bool Transcriptome::has_novel_exon_boundaries(const list<EditedTranscriptPath> &
     return false;
 }
 
-void Transcriptome::augment_splice_graph(list<EditedTranscriptPath> * edited_transcript_paths, gbwt::GBWT * haplotype_index, const bool break_at_transcript_ends) {
+void Transcriptome::augment_splice_graph(list<EditedTranscriptPath> * edited_transcript_paths, unique_ptr<gbwt::GBWT> & haplotype_index, const bool break_at_transcript_ends) {
 
     assert(_transcript_paths.empty());
     _splice_graph_node_updated = true;
 
-
     // Move paths to data structure compatible with augment.
     vector<Path> edited_paths;
     edited_paths.reserve(edited_transcript_paths->size());
@@ -1228,7 +1271,7 @@ void Transcriptome::augment_splice_graph(list<EditedTranscriptPath> * edited_tra
     }
 }
 
-void Transcriptome::update_haplotype_index(gbwt::GBWT * haplotype_index, const vector<Translation> & translations) const {
+void Transcriptome::update_haplotype_index(unique_ptr<gbwt::GBWT> & haplotype_index, const vector<Translation> & translations) const {
 
 #ifdef transcriptome_debug
     double time_translation_1 = gcsa::readTimer();
@@ -1323,7 +1366,7 @@ void Transcriptome::update_haplotype_index(gbwt::GBWT * haplotype_index, const v
 
     // Finish contruction and recode index.
     gbwt_builder.finish();
-    *haplotype_index = gbwt::GBWT(gbwt_builder.index);
+    haplotype_index.reset(new gbwt::GBWT(gbwt_builder.index));
     
 #ifdef transcriptome_debug
     cerr << "\tDEBUG update end: " << gcsa::readTimer() - time_update_1 << " seconds, " << gcsa::inGigabytes(gcsa::memoryUsage()) << " GB" << endl;
@@ -1390,7 +1433,7 @@ void Transcriptome::remove_non_transcribed(const bool new_reference_paths) {
         path_handles.emplace_back(path_handle);
     }));    
 
-    // Remove non transcript paths.
+    // Remove all paths.
     for (auto & path_handle: path_handles) {
 
         _splice_graph->destroy_path(path_handle);
@@ -1444,11 +1487,13 @@ int32_t Transcriptome::embed_transcript_paths(const bool add_reference_paths, co
     // Add transcript paths to graph
     for (auto & transcript_path: _transcript_paths) {
 
-        assert(!transcript_path.haplotype_origin_ids.empty() || !transcript_path.reference_origin.empty());
+        assert(!transcript_path.reference_origin.empty() || !transcript_path.haplotype_origin_ids.empty() || !transcript_path.path_origin_names.empty());
 
-        if ((add_reference_paths && !transcript_path.reference_origin.empty()) || (add_non_reference_paths && !transcript_path.haplotype_origin_ids.empty())) {
+        if ((add_reference_paths && !transcript_path.reference_origin.empty()) || (add_non_reference_paths && (!transcript_path.haplotype_origin_ids.empty() || !transcript_path.path_origin_names.empty()))) {
 
             ++num_embedded_paths;
+
+            assert(!_splice_graph->has_path(transcript_path.name));
             auto path_handle = _splice_graph->create_path_handle(transcript_path.name);
 
             for (auto & handle: transcript_path.path) {
@@ -1461,7 +1506,7 @@ int32_t Transcriptome::embed_transcript_paths(const bool add_reference_paths, co
     return num_embedded_paths;
 }
 
-int32_t Transcriptome::construct_gbwt(gbwt::GBWTBuilder * gbwt_builder, const bool output_reference_transcripts, const bool add_bidirectional) const {
+int32_t Transcriptome::add_transcripts_to_gbwt(gbwt::GBWTBuilder * gbwt_builder, const bool output_reference_transcripts, const bool add_bidirectional) const {
 
     int32_t num_added_threads = 0;
 
@@ -1473,8 +1518,9 @@ int32_t Transcriptome::construct_gbwt(gbwt::GBWTBuilder * gbwt_builder, const bo
 
     for (auto & transcript_path: _transcript_paths) {
 
-        assert(!transcript_path.haplotype_origin_ids.empty() || !transcript_path.reference_origin.empty());
-        if (!transcript_path.haplotype_origin_ids.empty() || output_reference_transcripts) {
+        assert(!transcript_path.reference_origin.empty() || !transcript_path.haplotype_origin_ids.empty() || !transcript_path.path_origin_names.empty());
+
+        if (output_reference_transcripts || !transcript_path.haplotype_origin_ids.empty() || !transcript_path.path_origin_names.empty()) {
 
             ++num_added_threads;
 
@@ -1507,8 +1553,9 @@ int32_t Transcriptome::write_sequences(ostream * fasta_ostream, const bool outpu
 
     for (auto & transcript_path: _transcript_paths) {
 
-        assert(!transcript_path.haplotype_origin_ids.empty() || !transcript_path.reference_origin.empty());
-        if (!transcript_path.haplotype_origin_ids.empty() || output_reference_transcripts) {
+        assert(!transcript_path.reference_origin.empty() || !transcript_path.haplotype_origin_ids.empty() || !transcript_path.path_origin_names.empty());
+
+        if (output_reference_transcripts || !transcript_path.haplotype_origin_ids.empty() || !transcript_path.path_origin_names.empty()) {
 
             ++num_written_sequences;
 
@@ -1535,8 +1582,9 @@ int32_t Transcriptome::write_info(ostream * tsv_ostream, const gbwt::GBWT & hapl
 
     for (auto & transcript_path: _transcript_paths) {
 
-        assert(!transcript_path.haplotype_origin_ids.empty() || !transcript_path.reference_origin.empty());
-        if (!transcript_path.haplotype_origin_ids.empty() || output_reference_transcripts) {
+        assert(!transcript_path.reference_origin.empty() || !transcript_path.haplotype_origin_ids.empty() || !transcript_path.path_origin_names.empty());
+
+        if (output_reference_transcripts || !transcript_path.haplotype_origin_ids.empty() || !transcript_path.path_origin_names.empty()) {
 
             ++num_written_info;
 
@@ -1560,32 +1608,34 @@ int32_t Transcriptome::write_info(ostream * tsv_ostream, const gbwt::GBWT & hapl
                 *tsv_ostream << "\t" << transcript_path.reference_origin;
             }
 
-            if (transcript_path.haplotype_origin_ids.empty()) {
+            if (transcript_path.haplotype_origin_ids.empty() && transcript_path.path_origin_names.empty()) {
 
                 *tsv_ostream << "\t-";            
 
             } else {
 
                 *tsv_ostream << "\t";
-                auto haplotype_origin_ids_it = transcript_path.haplotype_origin_ids.begin();
+                bool is_first = true;
 
-                while (haplotype_origin_ids_it != transcript_path.haplotype_origin_ids.end()) {
+                for (auto & id: transcript_path.haplotype_origin_ids) {
 
-                    if (haplotype_origin_ids_it != transcript_path.haplotype_origin_ids.begin()) {
+                    if (!is_first) {
 
                         *tsv_ostream << ",";
-                    }
+                    } 
 
-                    if (*haplotype_origin_ids_it < 0) {
+                    is_first = false;             
+                    *tsv_ostream << thread_name(haplotype_index, id);
+                }
 
-                        *tsv_ostream << _splice_graph->get_path_name(handlegraph::as_path_handle(-1 * *haplotype_origin_ids_it));
+                if (!transcript_path.path_origin_names.empty()) {
 
-                    } else {
+                    if (!is_first) {
 
-                        *tsv_ostream << thread_name(haplotype_index, *haplotype_origin_ids_it);
-                    }
+                        *tsv_ostream << ",";
+                    } 
 
-                    ++haplotype_origin_ids_it;
+                    *tsv_ostream << transcript_path.path_origin_names;
                 }
             }
 
