@@ -27,6 +27,7 @@
 #include "snarls.hpp"
 #include "path_index.hpp"
 #include "genotypekit.hpp"
+#include "gbwt_helper.hpp"
 
 namespace vg {
 
@@ -558,6 +559,85 @@ protected:
                                 
 };
 
+/** Finds traversals with the most flow.  Node and edge weights are specified 
+ * using the callbacks and can be used, ex, to yield read supports.  
+ * If one traversal is requested, then the path with the highest flow (whose
+ * node or edge with the minimum weight is maximum) is returned.  If K
+ * traversals are specified, then the K highest flow traversals are returned.
+ * This is designed to be a replacement for RepresentativeTraversalFinder.
+ * It should do a better job of enumerating off-reference traversals, and will
+ * of course guarantee to return all the optimal traversals (in the context of max flow).
+ * Unlike RepresentativeTraversalFinder, it does not currently support nested
+ * snarls, so all traversals returned are explicit.  
+ * It is possible that it will blow up on massive snarls, espeically for large Ks. 
+ */ 
+class FlowTraversalFinder : public TraversalFinder {
+    
+protected:
+    const HandleGraph& graph;
+    
+    SnarlManager& snarl_manager;
+
+    /// The K-best traversals are returned
+    size_t K;
+
+    /// Callbacks to get supports
+    function<double(handle_t)> node_weight_callback;
+    function<double(edge_t)> edge_weight_callback;
+    
+public:
+    
+    // if path_names not empty, only those paths will be considered
+    FlowTraversalFinder(const HandleGraph& graph, SnarlManager& snarl_manager,
+                        size_t K,
+                        function<double(handle_t)> node_weight_callback,
+                        function<double(edge_t)> edge_weight_callback);
+
+    /**
+     * Return the K widest (most flow) traversals through the site
+     * The reference traversal will be returned first (regardless of its flow).
+     * After, the traversals are listed in decreasing order
+     */
+    virtual vector<SnarlTraversal> find_traversals(const Snarl& site);
+
+    /**
+     * Return the K widest traversals, along with their flows
+     */
+    virtual pair<vector<SnarlTraversal>, vector<double>> find_weighted_traversals(const Snarl& site,
+                                                                                  bool greedy_avg = false);
+
+    /// Set K
+    void setK(size_t k);
+
+};    
+
+/** Rerturn all traversals of a snarl that correspond to haplotypes stored in a GBWT
+ */
+class GBWTTraversalFinder : public TraversalFinder {
+
+protected:
+    
+    const HandleGraph& graph;
+    const gbwt::GBWT& gbwt;
+    
+public:
+    
+    GBWTTraversalFinder(const HandleGraph& graph, const gbwt::GBWT& gbwt);
+    
+    virtual ~GBWTTraversalFinder();
+
+    virtual vector<SnarlTraversal> find_traversals(const Snarl& site);
+
+protected:
+
+    /**
+     * Breadth first search from the start to the end, only branching if there's a haplotype 
+     * in the GBWT, and returning all unique haplotypes found. 
+     */
+    vector<vector<gbwt::node_type>> get_spanning_haplotypes(handle_t start, handle_t end);
+    
+};
+
 }
 
 #endif
