@@ -15,6 +15,8 @@
 #include <iostream>
 #include <unordered_map>
 
+#include <bdsg/hash_graph.hpp>
+
 namespace vg {
 namespace unittest {
 
@@ -287,6 +289,60 @@ Graph construct_test_graph(string fasta_data, string vcf_data, size_t max_node_s
     return built;
 }
 
+/**
+ * Testing wrapper to build a whole graph from a VCF string into a HandleGraph
+ */
+unique_ptr<PathHandleGraph> construct_test_handle_graph(string fasta_data, string vcf_data, 
+    size_t max_node_size, bool do_svs, bool use_flat_alts = false) {
+  
+    // Make an empty graph
+    auto build_to = new bdsg::HashGraph();
+    // Make a unique_ptr own our graph
+    unique_ptr<PathHandleGraph> graph(build_to);
+    
+    // Make a stream out of the VCF data
+    std::stringstream vcf_stream(vcf_data);
+    
+    // Load it up in vcflib
+    vcflib::VariantCallFile vcf;
+    vcf.open(vcf_stream);
+    
+    // Put it in a vector
+    vector<vcflib::VariantCallFile*> vcf_pointers {&vcf};
+    
+    // We have to write the FASTA to a file
+    string fasta_filename = temp_file::create();
+    ofstream fasta_stream(fasta_filename);
+    fasta_stream << fasta_data;
+    fasta_stream.close(); 
+    
+    // Make a FastaReference out of it
+    FastaReference reference;
+    reference.open(fasta_filename);
+    
+    // Put it in a vector
+    vector<FastaReference*> fasta_pointers {&reference};
+    
+    // Make an empty vector of insertion files
+    vector<FastaReference*> ins_pointers;
+    
+    Constructor constructor;
+    constructor.alt_paths = true;
+    constructor.do_svs = do_svs;
+    constructor.flat = use_flat_alts;
+    // Make sure we can test the node splitting behavior at reasonable sizes
+    constructor.max_node_size = max_node_size;
+
+    // Construct the graph    
+    constructor.construct_graph(fasta_pointers, vcf_pointers, ins_pointers, build_to);
+    
+    // Delete our temporary file
+    temp_file::remove(fasta_filename);
+    
+    // Return the result
+    return std::move(graph);
+}
+
 TEST_CASE( "A SNP can be constructed", "[constructor]" ) {
 
     // We'll work on this tiny VCF
@@ -1222,6 +1278,38 @@ ref	11	.	TAG	T	29	PASS	.	GT
 
 }
 
+TEST_CASE( "A graph can be constructed to a HandleGraph", "[constructor]" ) {
+
+    auto vcf_data = R"(##fileformat=VCFv4.0
+##fileDate=20090805
+##source=myImputationProgramV3.1
+##reference=1000GenomesPilot-NCBI36
+##phasing=partial
+##FILTER=<ID=q10,Description="Quality below 10">
+##FILTER=<ID=s50,Description="Less than 50% of samples have data">
+##FORMAT=<ID=GT,Number=1,Type=String,Description="Genotype">
+#CHROM	POS	ID	REF	ALT	QUAL	FILTER	INFO	FORMAT
+ref	5	.	A	T	29	PASS	.	GT
+)";
+
+    auto fasta_data = R"(>ref
+GATTACACATTAG
+)";
+
+    // Build the graph
+    unique_ptr<PathHandleGraph> result = construct_test_handle_graph(fasta_data, vcf_data, 50, false);
+    
+#ifdef debug
+    std::cerr << pb2json(result) << std::endl;
+#endif
+
+    SECTION("graph is the correct size") {
+        REQUIRE(result->get_node_count() == 4);
+    }
+
+}
+
+
 TEST_CASE( "A VCF and FASTA on two contigs make a graph with a consistent ID space", "[constructor]" ) {
 
     auto vcf_data = R"(##fileformat=VCFv4.0
