@@ -1,9 +1,13 @@
 #include "utility.hpp"
 
-#include <cstdio>
 #include <set>
 #include <mutex>
 #include <dirent.h>
+#include <thread>
+#include <cstdio>
+#include <cmath>
+#include <fstream>
+#include <iostream>
 
 namespace vg {
 
@@ -99,6 +103,50 @@ int get_thread_count(void) {
     return thread_count;
 }
 
+void choose_good_thread_count() {
+    // If we leave this at 0, we won't apply any thread count and leave whatever OMP defaults to.
+    int count = 0;
+
+    if (count == 0) {
+        // First priority: OMP_NUM_THREADS
+        const char* value = getenv("OMP_NUM_THREADS");
+        if (value) {
+            // Read the value. Throws if it isn't a legit number.
+            count = std::stoi(value);
+        }
+    }
+
+    if (count == 0) {
+        // Next priority: /sys/fs/cgroup/cpu/cpu.cfs_quota_us over /sys/fs/cgroup/cpu/cpu.cfs_period_us
+        ifstream quota_file("/sys/fs/cgroup/cpu/cpu.cfs_quota_us");
+        ifstream period_file("/sys/fs/cgroup/cpu/cpu.cfs_period_us");
+
+        if (quota_file && period_file) {
+            // Read the period and quota
+            int64_t quota;
+            quota_file >> quota;
+            int64_t period;
+            period_file >> period;
+
+            if (quota >= 0 && period != 0) {
+                // Compute how many threads we may use.
+                // May come out to 0, in which case it is ignored.
+                count = (int) ceil(quota / (double) period);
+            }
+        }
+    }
+
+    if (count == 0) {
+        // Next priority: hardware concurrency as reported by the STL.
+        // This may itself be 0 if ungettable.
+        count = std::thread::hardware_concurrency();
+    }
+    
+    if (count != 0) {
+        omp_set_num_threads(count);
+    }
+}
+
 std::vector<std::string> &split_delims(const std::string &s, const std::string& delims, std::vector<std::string> &elems) {
     char* tok;
     char cchars [s.size()+1];
@@ -150,6 +198,10 @@ bool is_number(const std::string& s) {
                         [](char c) { return !std::isdigit(c); }) == s.end();
 }
 
+bool isATGC(const char& b) {
+    return (b == 'A' || b == 'T' || b == 'G' || b == 'C');
+}
+
 bool allATGC(const string& s) {
     for (string::const_iterator c = s.begin(); c != s.end(); ++c) {
         char b = *c;
@@ -181,6 +233,20 @@ string nonATGCNtoN(const string& s) {
     return n;
 }
 
+string allAmbiguousToN(const string& s) {
+    auto n = s;
+    for (string::iterator c = n.begin(); c != n.end(); ++c) {
+        char b = *c;
+        if (b == 'M' || b == 'R' || b == 'W' || b == 'S' || b == 'Y' ||
+            b == 'K' || b == 'V' || b == 'H' || b == 'D' || b == 'B') {
+            // Replace known IUPAC ambiguity codes with N.
+            // Leave weird things like '-' or '*' which should be errors.
+            *c = 'N';
+        }
+    }
+    return n;
+}
+
 string toUppercase(const string& s) {
     auto n = s;
     for (string::iterator c = n.begin(); c != n.end(); ++c) {
@@ -428,6 +494,30 @@ void get_input_file(const string& file_name, function<void(istream&)> callback)
     
 }
 
+pair<string, string> split_ext(const string& filename) {
+    pair<string, string> parts;
+
+    size_t dot = filename.rfind('.');
+    
+    if (dot == string::npos) {
+        // Put it all in the first part
+        parts.first = filename;
+    } else {
+        // Split on either side of the dot.
+        parts.first = filename.substr(0, dot);
+        parts.second = filename.substr(dot + 1);
+    }
+    
+    return parts;
+}
+
+bool file_exists(const string& filename) {
+    // TODO: use C++17 features to actually poll existence.
+    // For now we see if we can open it.
+    ifstream in(filename);
+    return in.is_open();
+}
+
     
 void create_ref_allele(vcflib::Variant& variant, const std::string& allele) {
     // Set the ref allele
