@@ -37,6 +37,10 @@ double get_fraction_of_ns(const string& seq);
 /// Return the number of threads that OMP will produce for a parallel section.
 /// TODO: Assumes that this is the same for every parallel section.
 int get_thread_count(void);
+/// Decide on and apply a sensible OMP thread count. Pay attention to
+/// OMP_NUM_THREADS if set, the "hardware concurrency", and container limit
+/// information that may be available in /proc.
+void choose_good_thread_count();
 string wrap_text(const string& str, size_t width);
 bool is_number(const string& s);
 
@@ -47,9 +51,14 @@ std::vector<std::string> split_delims(const std::string &s, const std::string& d
 const std::string sha1sum(const std::string& data);
 const std::string sha1head(const std::string& data, size_t head);
 
+/// Return true if a character is an uppercase A, C, G, or T, and false otherwise.
+bool isATGC(const char& b);
 bool allATGC(const string& s);
 bool allATGCN(const string& s);
 string nonATGCNtoN(const string& s);
+/// Convert known IUPAC ambiguity codes (which we don't support) to N (which we
+/// do), while leaving any other garbage to trigger validation checks later.
+string allAmbiguousToN(const string& s);
 // Convert ASCII-encoded DNA to upper case
 string toUppercase(const string& s);
 
@@ -423,6 +432,13 @@ string get_output_file_name(int& optind, int argc, char** argv);
 /// only until the callback returns.
 void get_input_file(const string& file_name, function<void(istream&)> callback);
 
+/// Split off the extension from a filename and return both parts. 
+pair<string, string> split_ext(const string& filename);
+
+/// Determine if a file exists.
+/// Only works for files readable by the current user.
+bool file_exists(const string& filename);
+
 /// Parse a command-line argument string. Exits with an error if the string
 /// does not contain exactly an item fo the appropriate type.
 template<typename Result>
